`#include <stdio.h>
`#include <stdlib.h>
`#include <string.h>

`#ifdef WIN32
`#include <windows.h>
`#include <winsock.h>
`#include <io.h>
`#else  /* WIN32 */
`#include <unistd.h>
`#endif /* WIN32 */

`#include <fcntl.h>
`#include <sys/types.h>
`#include <sys/stat.h>

`#define  STRCHR strchr
`#define  MEMSET(x,y,z)  memset(x,y,z)
`#define  MEMCPY(x,y,z)  memcpy(x,y,z)

`#ifndef   _INT_F
`#define   _INT_F
typedef int (*int_f)();
`#endif /* _INT_F */

`#ifndef   _VOID_F
`#define   _VOID_F
typedef void (*void_f)();
`#endif /* _VOID_F */

`#ifndef   _VOIDP_F
`#define   _VOIDP_F
typedef void * (*voidp_f)();
`#endif /* _VOIDP_F */

`#ifndef   _STRING_F
`#define   _STRING_F
typedef char * (*string_f)();
`#endif /* _STRING_F */
/*end*/

`#ifdef    WIN32
`#define IS_INVALID_SOCKET(x)    (x == INVALID_SOCKET)
`#else  /* WIN32 */
`#define INVALID_SOCKET -1
`#define IS_INVALID_SOCKET(x)    (x == -1)
`#endif /* WIN32 */

typedef struct InputStream_s
{
	struct InputStream_s * parent;
	int_f  available;
	int_f  readChar;
	int_f  read;
	int_f  skip;
	void_f close;
	void_f delete;
} InputStream;

typedef InputStream * (*InputStream_f)();

typedef struct StdInputStream_s
{
	/* InputStream fields */
	InputStream * parent;
	int_f  available;
	int_f  readChar;
	int_f  read;
	int_f  skip;
	void_f close;
	void_f delete;
	/* its own fields */
	int id;
	int ateof;
	int dontblock;
} StdInputStream;

typedef struct FileInputStream_s 
{
	/* InputStream fields */
	InputStream * parent;
	int_f  available;
	int_f  readChar;
	int_f  read;
	int_f  skip;
	void_f close;
	void_f delete;
	/* its own fields */
	StdInputStream * sid;
} FileInputStream;

`#ifdef    WIN32
typedef SOCKET SocketHandle;
`#else  /* WIN32 */
typedef int SocketHandle;
`#endif /* WIN32 */

typedef struct SocketInputStream_s 
{
	/* InputStream fields */
	InputStream * parent;
	int_f  available;
	int_f  readChar;
	int_f  read;
	int_f  skip;
	void_f close;
	void_f delete;
	/* its own fields */
	StdInputStream * sid;

`#ifdef     WIN32
	SocketHandle     handle;
`#endif  /* WIN32 */
} SocketInputStream;


typedef struct CachedInputStream_s
{
	/* InputStream fields */
	InputStream * parent;
	int_f  available;
	int_f  readChar;
	int_f  read;
	int_f  skip;
	void_f close;
	void_f delete;
	/* its own fields */
	InputStream * istream;
	char        * cache;
	char        * cacheptr;
	int           cachelen;
	void_f        setCache;
} CachedInputStream;

typedef struct BufferedInputStream_s
{
	/* InputStream fields */
	InputStream * parent;
	int_f  available;
	int_f  readChar;
	int_f  read;
	int_f  skip;
	void_f close;
	void_f delete;
	/* BufferedInputStream fields */
	InputStream * inputstream;
	void        * data;
	char        * dptr;
	int           bufsize;
	int           buflen;
	int_f         load;
} BufferedInputStream;

typedef struct DataInputStream_s
{
	BufferedInputStream * bs;
	string_f              readLine;
	void_f delete;
} DataInputStream;


typedef struct InputData_s
{
	BufferedInputStream * bs;
	void *  data;
	int     size;
	int     loaded;
	int_f   getSize;
	voidp_f getData;
	void_f delete;
} InputData;

typedef InputData * (*InputData_f)();

typedef struct OutputStream_s 
{
	struct OutputStream_s * parent;
	int_f  write;
	void_f flush;
	void_f close;
	void_f delete;
} OutputStream;

typedef OutputStream * (*OutputStream_f)();

typedef struct BufferedOutputStream_s 
{
	/* common OutputStream members */
	OutputStream * parent;
	int_f  write;
	void_f flush;
	void_f close;
	void_f delete;
	/* its own */
	OutputStream * outputstream;
	char * data;
	char * dptr;
	int bufsize;
	int buflen;
} BufferedOutputStream;

typedef struct StdOutputStream_s 
{
	/* common OutputStream members */
	OutputStream * parent;
	int_f  write;
	void_f flush;
	void_f close;
	void_f delete;
	/* its own */
	int id;
	int length;
	int_f getID;
} StdOutputStream;

`#ifdef    WIN32
typedef struct SocketOutputStream_s 
{
	/* common OutputStream members */
	OutputStream * parent;
	int_f  write;
	void_f flush;
	void_f close;
	void_f delete;
	/* its own */
	SocketHandle handle;
} SocketOutputStream;
`#else  /* WIN32 */
typedef StdOutputStream SocketOutputStream; /* FIXME */
`#endif /* WIN32 */

typedef struct FileOutputStream_s 
{
	/* common OutputStream members */
	OutputStream * parent;
	int_f  write;
	void_f flush;
	void_f close;
	void_f delete;
	/* its own */
	StdOutputStream * ostream;
	char * filename;
	int_f  getID;
} FileOutputStream;

typedef struct TempFileOutputStream_s 
{
	/* common OutputStream members */
	OutputStream * parent;
	int_f  write;
	void_f flush;
	void_f close;
	void_f delete;
	/* its own */
	FileOutputStream * ostream;
	void_f copy;
	int_f  getContentLength;
} TempFileOutputStream;

typedef struct PrintWriter_s 
{
	OutputStream * os;
	char * convbuf;
	void_f print;
	void_f println;
	void_f printlnInt;
	void_f printBoolean;
	void_f printChar;
	void_f printInt;
	void_f printLong;
	void_f printFloat;
	void_f printDouble;
	void_f delete;
} PrintWriter;
