`#include "server_transport.h"
`#include "Ustreams.h"

`#ifndef   _NO_PROTO
static int         server_transport_open(server_transport *);
static int         server_transport_close(server_transport *);
static sc_data_t * server_transport_receive(server_transport *);
static void        server_transport_send(server_transport *, sc_data_t *);
`#else  /* _NO_PROTO */
static int         server_transport_open();
static int         server_transport_close();
static sc_data_t * server_transport_receive();
static void        server_transport_send();
`#endif /* _NO_PROTO */

`#ifndef   _NO_PROTO
void delete_server_transport( server_transport * ht)
`#else  /* _NO_PROTO */
void delete_server_transport( ht)
	server_transport * ht ;
`#endif /* _NO_PROTO */
{
	(void) free ((void*) ht);
}

`#ifndef   _NO_PROTO
server_transport * new_server_transport( void)
`#else  /* _NO_PROTO */
server_transport * new_server_transport()
`#endif /* _NO_PROTO */
{
	server_transport * ht = (server_transport*) malloc ((unsigned) sizeof(server_transport));

	ht->delete  = delete_server_transport;
	ht->open    = server_transport_open;
	ht->close   = server_transport_close;
	ht->receive = server_transport_receive;
	ht->send    = server_transport_send;

	return ht;
}

`#ifndef   _NO_PROTO
server_transport * getNew_server_transport( void)
`#else  /* _NO_PROTO */
server_transport * getNew_server_transport()
`#endif /* _NO_PROTO */
{
	return new_server_transport();
}

`#ifndef   _NO_PROTO
static int server_transport_open( server_transport * ht)
`#else  /* _NO_PROTO */
static int server_transport_open( ht)
	server_transport * ht ;
`#endif /* _NO_PROTO */
{
	return 1;
}

`#ifndef   _NO_PROTO
static int server_transport_close( server_transport * ht)
`#else  /* _NO_PROTO */
static int server_transport_close( ht)
	server_transport * ht ;
`#endif /* _NO_PROTO */
{
	return 1;
}

`#define  StartState  0
`#define  DataState   1
`#define  GoalState   2

`#ifndef   _NO_PROTO
static int server_transport_splitLineAt( char c, char * line, char ** keyp, char ** eolp)
`#else  /* _NO_PROTO */
static int server_transport_splitLineAt( c, line, keyp, eolp)
	char    c ;
	char  * line ;
	char ** keyp ;
	char ** eolp ;
`#endif /* _NO_PROTO */
{
	char * p;

	for (p = line; p && *p; p++) {
		if (*p != c)
			continue;
		*p = '\0';
		*keyp = line;
		*eolp = p+1;
		return 1;
	}
	return 0;
}

`#ifndef   _NO_PROTO
static void server_transport_serverDeSerialize( AnyControl_t * sc, char * toeol, DataInputStream * ds)
`#else  /* _NO_PROTO */
static void server_transport_serverDeSerialize( sc, toeol, ds)
	AnyControl_t    * sc ;
	char            * toeol ;
	DataInputStream * ds ;
`#endif /* _NO_PROTO */
{
	int i;

	if (!sc)
		return;

	i = sc->type;
	if (i == ScUnknownType)
		return;
	switch (i) {
		case ScBooleanType:
			{  int tv = (strcmp( toeol, "true") == 0);

			   SC_SETCONTROL(sc, tv);
			}
			break;
		case ScIntType:
			   SC_SETCONTROL(sc, atoi(toeol));
			break;
		case ScStringType:
			   if (toeol)
				SC_SETCONTROL(sc, strdup(toeol));
			break;
		case ScStringArrayType:
			{
				char ** values;
				char ** pp;
				int i = atoi(toeol);
				int j;
				if (i == 0)
					break;
				values = (char**) malloc ((unsigned) (i+1) * sizeof(char*));
				for (j=0, pp=values; j < i; j++, pp++) {
					*pp = (*ds->readLine)(ds);
					if (!*pp)
						break;
					*pp = strdup(*pp);
				}
				*pp = (char*)0;
				SC_SETCONTROL(sc, values);
			}
			break;
		default:
			break;
	}
}

`#ifndef   _NO_PROTO
static sc_data_t * server_transport_load( server_transport * hd, InputStream * i)
`#else  /* _NO_PROTO */
static sc_data_t * server_transport_load( hd, i)
	server_transport * hd ;
	InputStream      * i ;
`#endif /* _NO_PROTO */
{
	extern sc_data_t * newData();
	extern DataInputStream * newDataInputStream();

	DataInputStream * ds = newDataInputStream( i);
	char * nextline;

	int    pollvalue    = 0;
	char * callbackname = (char*)0;
	int    state          = StartState;
	char * key;
	char * toeol = (char*)0;
	sc_data_t  * mydata = (sc_data_t*)0;
	AnyGroup_t * mygroup;

	while (state != GoalState) {
		nextline = (*ds->readLine)(ds);
		if (!nextline)
			break;

		if (!server_transport_splitLineAt('=', nextline, &key, &toeol))
			continue;

		switch (state) {
			case StartState:
				if (strcmp(key,"sc-start"))
					break;
				mydata = newData( toeol);
				if (!mydata) {
					state = GoalState;
					break;
				}
				mygroup = mydata->group;
				state = DataState;
				break;
			case DataState:
				if (strcmp(key,"sc-end") == 0) {
					state = GoalState;
					break;
				}
				if (strcmp(key,"sc-poll") == 0) {
					mydata->poll = atoi( toeol);
					break;
				}
				if (strcmp(key,"sc-callback") == 0) {
					mydata->name = strdup(toeol);
					break;
				}
				{
					AnyControl_t * sc = (*mygroup->findAppElement)( mygroup, key);
					if (!sc)
						break;
					server_transport_serverDeSerialize( sc, toeol,  ds);
					break;
				}
			default:
				break;
		}
	}

	/* (*ds->close)( ds); */
	(*ds->delete)( ds);

	return mydata;
}

`#ifndef   _NO_PROTO
static sc_data_t * server_transport_receive( server_transport * ht)
`#else  /* _NO_PROTO */
static sc_data_t * server_transport_receive( ht)
	server_transport * ht ;
`#endif /* _NO_PROTO */
{
	extern StdInputStream * newStdInputStream();

	StdInputStream * si = newStdInputStream(0);
	sc_data_t *    rval = server_transport_load( ht, (InputStream*)si);

	(*si->delete)( si);

	return rval;
}

`#ifndef   _NO_PROTO
static void server_transport_send_header( PrintWriter * pw, char * mimeType)
`#else  /* _NO_PROTO */
static void server_transport_send_header( pw, mimeType)
	PrintWriter * pw ;
	char        * mimeType ;
`#endif /* _NO_PROTO */
{
	(*pw->print)( pw, "Content-type: ");
		(*pw->println)( pw, mimeType);
		(*pw->println)( pw, (char*)0);
}

`#ifndef   _NO_PROTO
void server_transport_serverSerialize( AnyControl_t* c, PrintWriter * pw)
`#else  /* _NO_PROTO */
void server_transport_serverSerialize( c, pw)
	AnyControl_t* c ;
	PrintWriter * pw ;
`#endif /* _NO_PROTO */
{
	int i = c->type;
	if (i == ScUnknownType)
		return;
	(*pw->print)( pw, SC_GETNAME(c));
		(*pw->print)( pw, "=");
	switch (i) {
		case ScBooleanType:
			(*pw->printBoolean)( pw, SC_GETCONTROL(c));
			(*pw->println) ( pw, (char*)0);
			break;
		case ScIntType:
			(*pw->printInt)( pw, SC_GETCONTROL(c));
			(*pw->println) ( pw, (char*)0);
			break;
		case ScStringType:
			(*pw->print)( pw, SC_GETCONTROL(c));
			(*pw->println) ( pw, (char*)0);
			break;
		case ScStringArrayType:
			{
			char ** data = SC_GETCONTROL(c);
			char ** pp;
			int i;

			for (i = 0, pp = data; pp && *pp; pp++, i++);

			(*pw->printInt)( pw, i);
			(*pw->println) ( pw, (char*)0);

			while (data && *data) {
				(*pw->println) ( pw, *data++);
			}
			}
			break;
		default:
			break;
	}
}

`#ifndef   _NO_PROTO
static void server_transport_send_data( PrintWriter * pw, sc_data_t * sd)
`#else  /* _NO_PROTO */
static void server_transport_send_data( pw, sd)
	PrintWriter * pw ;
	sc_data_t   * sd ;
`#endif /* _NO_PROTO */
{
	AnyGroup_t  * g  = sd->group;
	char *        mt = g->mimeType;

	(*pw->print)( pw, "sc-start=");
		(*pw->println)( pw, mt);
	(*pw->print)( pw, "sc-callback=");
		(*pw->println)( pw, sd->name);

	if (sd->poll) {
		(*pw->print)( pw, "sc-poll=");
			(*pw->printlnInt)( pw, sd->poll);
	}

	(*g->startAppElementEnumeration)( g);

	while ((*g->hasMoreAppElements)(g)) {
		server_transport_serverSerialize( (*g->getNextAppElement)(g), pw);
	}

	(*pw->print)( pw, "sc-end=");
		(*pw->println)( pw, mt);
	(*pw->println)( pw, (char*)0);

}

`#ifndef   _NO_PROTO
static void server_transport_send( server_transport * ht, sc_data_t * sd)
`#else  /* _NO_PROTO */
static void server_transport_send( ht, sd)
	server_transport * ht ;
	sc_data_t        * sd ;
`#endif /* _NO_PROTO */
{
	extern PrintWriter *     newPrintWriter();
	extern StdOutputStream * newStdOutputStream();

	StdOutputStream * so = newStdOutputStream(1);
	PrintWriter     * pw = newPrintWriter((OutputStream*)so);

	server_transport_send_header( pw, sd->group->mimeType);
	server_transport_send_data( pw, sd);

	(*pw->delete) (pw);
	(*so->delete) (so);
}
