
/*
**LIBS: -lnsl -lsocket -lgen
*/

`#include <stdio.h>
`#include "URL.h"

ProxyNameList * HttpURLConnection::ignore_list = 0;
int HttpURLConnection::proxyport = -50;
char * HttpURLConnection::proxy  = (char*)0;


URL::URL( char * url, char * urldata)
{
	bad = !parseUrl( url, urldata);
	if (bad)
		setSCError("bad URL");
	connection = (URLConnection*)0;
}

URL::~URL()
{
	if (connection) {
		connection->close();
		delete connection;
	}
	if (urlstring)
		delete urlstring;
}

int
URL::parseUrl( char * url, char * urldata)
{
	if (!url)
		return 0;
	int    i = 0;
	char * s;
	for ( s = url; *s && (*s != ':'); s++, i++);
	if (!s || !*s)
		return 0;

	char * p = new char[i+1];
	for ( i = 0, s = url; *s && (*s != ':'); s++, i++)
		p[i] = tolower(*s);

	p[i] = '\0';

	if (strcmp( p, "file") == 0)
		type = FILE_PROTOCOL;
	else
	if (strcmp( p, "http") == 0)
		type = HTTP_PROTOCOL;
	// add more protocols here. this could of course be dynamic and table driven

	delete p;

	if (!type)
		return 0;

	s++;  // skip the :
	if (urldata)
		i = strlen(urldata) + 2;
	else
		i = 0;

	urlstring = new char[strlen(s)+i + 1];
	(void) strcpy( urlstring, s);
	if (i) {
		strcat(urlstring,"?");
		strcat(urlstring,urldata);
	}

	return 1;
}

URL *
URL::getURL( char * url, char * urldata)
{
	URL * u = new URL( url, urldata);
	if (!u->isOk()) {
		delete u;
		return (URL*)0;
	}
	return u;
}


URLConnection*
URL::openConnection()
{
	if (connection)
		return connection;

	switch (type) {
		case HTTP_PROTOCOL:
			connection = new HttpURLConnection( urlstring);
			break;
		case FILE_PROTOCOL:
			connection = new FileURLConnection( urlstring);
			break;
		default:
			break;
	}
	if (!connection) {
		setSCError("no connection");
		bad = 1;
		return (URLConnection*)0;
	}
	if (!connection->isOk()) {
		setSCError("bad connection\n");
		delete connection;
		connection = (URLConnection*)0;
		bad = 1;
		return (URLConnection*)0;
	}

	return connection;
}

/* --------------------- socket routines for http ----------------------*/
`#ifdef WIN32
// although this is implemented as a static in URLSocket, you may
// prefer to do this in the program main routine

int URLSocket::wsaBad = -1;
WSADATA URLSocket::wsaData;

int
URLSocket::winsock_startup()
{

	if (wsaBad == 1)
		return 0;

	if (!wsaBad)
		return 1;

	if (WSAStartup( 0x202, &URLSocket::wsaData) == SOCKET_ERROR) {
		wsaBad = 1;
		return 0;
	}

	wsaBad = 0;
	return 1;
}

void
URLSocket::winsock_cleanup()
{
	WSACleanup();
}

`#else
`#include <sys/socket.h>
`#include <netinet/in.h>
`#include <arpa/inet.h>
`#include <netdb.h>
`#endif

URLSocket::URLSocket( char *hostname, int port)
{
	struct sockaddr_in sktin;
	struct hostent * host;

	sock_stdin  = (SocketInputStream*)0;
	sock_stdout = (SocketOutputStream*)0;

`#ifdef WIN32
	if (!URLSocket::winsock_startup()) {
		urlErrno = E_CannotInitializeSockets;
		return;
	}
`#endif
	sock = INVALID_SOCKET;

	MEMSET ( &sktin, 0, sizeof( sktin));
	sktin.sin_family = AF_INET;
	sktin.sin_port   = htons( port);

	if ((host = gethostbyname( hostname)) != (struct hostent *)0) {
		MEMCPY ( &sktin.sin_addr, host->h_addr, host->h_length);
	} else
	if ((sktin.sin_addr.s_addr = inet_addr( hostname)) == (unsigned long)-1) {
		urlErrno = E_NoRouteToHost;
		return;
	}

	sock = socket( PF_INET, SOCK_STREAM, 0);
	if (sock == INVALID_SOCKET) {
		perror("socket()");
		urlErrno = E_CreateSocket;
		return;
	} else if (sock == 0) {
`#if 0
		if (debugfid)
			(void) fprintf ( debugfid, "zero on %s %d<p>\n", hostname, port);
`#endif
	}

	if ( connect( sock,(struct sockaddr*) &sktin, sizeof( sktin)) < 0) {
		(void) close ( sock);
		sock = INVALID_SOCKET;
		perror("connect()");
		urlErrno = E_CannotConnectToHost;
		return;
	}
	return;
}

URLSocket::~URLSocket()
{
	if (sock == INVALID_SOCKET)
		(void) close ( sock);
	if (sock_stdin)
		delete sock_stdin;
	if (sock_stdout)
		delete sock_stdout;
}


int
URLSocket::write(char * buf)
{
`#ifdef WIN32
	return ::send( sock, buf, strlen(buf), 0);
`#else
	return ::write( sock, buf, strlen( buf));
`#endif
}


URLConnection::~URLConnection()
{
	FREE(urlstring);
}

HttpHeader::~HttpHeader() {
	if (istream)
		delete istream;
	if (header)
		delete header;
}

int
HttpURLConnection::parseURL( char * s)
{
	char * p = (char*)0;
	char * q;

	if (*s++ != '/')
		return 0;
	if (*s++ != '/')
		return 0;
	for ( q = s; q && *q && (*q != '/'); q++)
		if (*q == ':')
			p = q;

	if (q && *q)
		*q++ = '\0';
	if (p && *p)
		*p++ = '\0';

	serverhost = s;
	if (p)
		serverport = atoi(p);
	else
		serverport = 80;

	if (q) {
		resource = q;
	} else
		resource = "";

	return 1;
}

int
HttpURLConnection::open()
{
	char * host = (char*)0;
	int    port;

	if (needsProxy( serverhost)) {
		host = getProxyHost();
		port = getProxyPort();
		url_usesproxy = 1;
	} else {
		host = serverhost;
		port = serverport;
		url_usesproxy = 0;
	}
	sock = new URLSocket( host, port);

	if (!sock || !sock->isConnected()) {
		if (sock)
			delete sock;
		return -1;
	}

	data = (void*)0;

	setConnected();
	return 0;
}

OutputStream *
HttpURLConnection::getOutputStream()
{
	if (!doOutput)
		return (OutputStream*)0;
	tmpStream = new TempFileOutputStream();

	return tmpStream;
}

int
HttpURLConnection::writeContentHeader( char * type)
{
	PathData p;
	char *   ptr;

	ptr = (char*)p;

	(void) sprintf( ptr, "%s ", type);
	ptr += strlen(ptr);

	if ( url_usesproxy) {
		(void) sprintf( ptr, "%s://%s:%d",
					getProtocol(),
					serverhost, serverport);
		ptr += strlen(ptr);
	}

	(void) sprintf( ptr, "/%s %s", resource, GETTRAILER);

	if (sock->write( p) == -1)
		return -1;
`#if 0
	if (getPostType() != (char*)0)
		setRequestProperty("Content-Type", getPostType());
`#endif
	startPairEnumeration();
	while (getUserPair( p)) {
		if (sock->write( (char*)p) == -1)
			return -1;
		if (sock->write( "\r\n") == -1)
			return -1;
	}

	return 0;
}


int
HttpURLConnection::writeEndOfHeader()
{
	if (sock->write( EXTRALINE) == -1)
		return -1;
	// uc->connected = 1;
	return 0;
}

char *
HttpHeader::processHeaderLine( char * q)
{
	char * p = getValue( q);
	char * v;

	if (!p)
		return endofline( q);

	for ( v = q; v && *v; v++)
		*v = tolower(*v);

	if ((!server) && (strcmp( q, "server") == 0))
		server = p;
	else if ((!mime_version) && (strcmp( q, "mime-version") == 0))
		mime_version = p;
	else if ((!content_type) && (strcmp( q, "content-type") == 0))
		content_type = p;
	else if ((content_length <= 0) && (strcmp( q, "content-length") == 0))
		content_length = atoi(p);

	return endofline( p);
}

int
HttpHeader::parse()
{
	char * p;
	char * q;
	int    len;

	header[0] = '\0';
	if ((len = istream->read( header, CONN_SIZE)) == -1)
		return -1;

	for ( p = header; p && *p;  p++) {
		if (*p != 'H')
			continue;
		if ( strncmp( p, "HTTP/", 5) == 0)
			break;
	}
	if (!p || !*p)
		return -2;

	p += 5;
	if ((p = skipWord( p)) == (char*)0)
		return -3;

	q = p;
	if ((p = nextwhite( p)) == (char*)0)
		return -4;

	/* status = atoi( q); */

	if ((p = overwhite( p)) == (char*)0)
		return -5;

	/* reply = p; */

	if ((q = endofline( p)) == (char*)0)
		return -6;

	q = overnullreplacement( q);

	while (q && *q && !isendofline(q)) { // the header ends with a blank line
		q = processHeaderLine( q);
		if (!q)
			return -7;
		q = overnullreplacement( q);
	}
	if (q) {
		q = overnullreplacement( q);

		if (q && *q) {
			len = len - (q - header);
			istream->setCache( q, len);
		}
	}
	return 1;
}

void
HttpHeader::debugHeader()
{
	if (server)
		(void) fprintf ( stderr, "Server:%s<p>\n", server);
	if (mime_version)
		(void) fprintf ( stderr, "Version:%s<p>\n", mime_version);
	if (content_type)
		(void) fprintf ( stderr, "Type:%s<p>\n", content_type);
	if (content_length)
		(void) fprintf ( stderr, "Length:%d<p>\n", content_length);
}



InputStream *
HttpURLConnection::readHeader()
{
	InputStream * i = sock->getInputStream();
	int rval;

	header = new HttpHeader( i);

	rval = header->parse();
	if (rval < 0) {
`#ifdef WIN32
		char buf[32];
		(void) sprintf ( buf, "Header Parse fail `#%d\n", rval);
`#ifdef _AFXDLL
		AfxMessageBox( buf);
`#endif
`#else
		(void) fprintf ( stderr, "Header Parse fail `#%d\n", rval);
`#endif
		delete i;
		delete header;
		return (InputStream *)0;
	}

	// header->debugHeader();
	return header->getInputStream();
}


int
HttpURLConnection::writeContentLength( int length)
{
	PathData p;

	(void) sprintf ( p,"Content-Length: %d \r\n", length);
	if (sock->write( (char*)p) == -1)
		return -1;

	return 0;
}


InputStream  *
HttpURLConnection::getInputStream()
{
	char * method = (char *)(doOutput ? POSTHEADER : GETHEADER);
	int length    = 0;

	if (doOutput) {
		if (!tmpStream) {
			return (InputStream*)0;
		}
		length = tmpStream->getContentLength();
		if (!length) {
			return (InputStream*)0;
		}
	}

	if (!isConnected())
		connect();

	if (!isOk()) {
		return (InputStream*)0;
	}

	if (writeContentHeader( method) == -1)
		return (InputStream*)0;

	if (doOutput) {
		if (writeContentLength( length) == -1)
			return (InputStream*)0;
	}

	if (writeEndOfHeader() == -1)
		return (InputStream*)0;

	if (doOutput) {
		OutputStream * o = sock->getOutputStream();

		tmpStream->copy( o);

		delete tmpStream;
	}

	return readHeader();
}

int
HttpURLConnection::close()
{
	if (header)
		delete header;
	PropertyPair * pp;
	while (properties) {
		pp = properties;
		properties = properties->next;
		delete pp;
	}
	if (post_type)
		(void) FREE(post_type);
	closeSocket();
	return 0;
}

int
FileURLConnection::open()
{
	return 0;
}

int
FileURLConnection::close()
{
	return 0;
}

`#define MYTESTURL "http://ist.co.uk/index.html"

void
testURL()
{
	URL * u = URL::getURL( MYTESTURL);
	InputStream * i = u->openStream();

	if (!i) {
		return;
	}
	DataInputStream * ds = new DataInputStream( i);
	char * s;

	while ((s = ds->readLine()) != (char*)0)
		printf("=> %s\n", s);

	delete ds;
	delete u;
}

`#ifdef TESTURL
int
main()
{
`#ifdef WIN32
       WSADATA wsadata;

	if (WSAStartup(0x202, &wsadata) == SOCKET_ERROR)
			exit(-1);
`#endif
	testURL();
`#ifdef WIN32
	WSACleanup();
`#endif
	return 0;
}
`#endif
