
`#ifndef _URL_H
`#define _URL_H

`#include "Ustreams.h"

`#include <sys/types.h>
`#include <ctype.h>


`#define  HTTP_PROTOCOL 1
`#define  FILE_PROTOCOL 2
`#define E_CreateSocket          -1
`#define E_NoRouteToHost         -2
`#define E_CannotConnectToHost   -3
`#define E_CannotInitializeSockets  -4

`#define MAXPATHDATA 1024
`#define CONN_SIZE   512

typedef char    PathData[MAXPATHDATA+1];

class URLConnection
{
	protected:
		int doOutput;
		int doInput;
		int allowUserInteraction;
		int ok;
		int isconnected;
		char * urlstring;

		void setConnected() { isconnected = 1; }
		void setBad()       { isconnected = -1; }


	public:
		URLConnection( char * u)
		{
			urlstring = strdup( u);
			isconnected = 0;
			doOutput = 0;
			doInput = 1;
		}

		~URLConnection();

		virtual int parseURL( char * s) = 0;

		int isConnected() { return (isconnected == 1); }


		int  isOk() { return (isconnected != -1); }

		void setDoOutput( int i) { doOutput = i;}
		void setDoInput( int i)  { doInput  = i;}
		void setAllowUserInteraction( int i) { allowUserInteraction = i; }

		virtual OutputStream * getOutputStream() = 0;
		virtual InputStream  * getInputStream()  = 0;

		virtual char * getContentType()   { return "text/plain"; }
		virtual int    getContentLength() { return -1; }

		InputData * getContent() { return (InputData*)0; }
		virtual char * getProtocol()    = 0;
		virtual int    getOutputSizeNeeded() = 0;
		virtual void   setRequestProperty( char * key, char * value) {}

		virtual int open() = 0;
		virtual int close() = 0;

		void connect()
		{
			if (!parseURL(urlstring)) {
				setBad();
				return;
			}
			if (open() == -1)
				setBad();
		}

};

`#ifndef WIN32
`#define INVALID_SOCKET -1
`#endif

class URLSocket
{
	private:
		SocketHandle sock;
		SocketInputStream  * sock_stdin;
		SocketOutputStream * sock_stdout;
`#ifdef WIN32
		static WSADATA wsaData;
		static int wsaBad;
`#endif
	public:

`#ifdef WIN32
		static int winsock_startup();
		static void winsock_cleanup();
`#endif
		int urlErrno;
		URLSocket( char *hostname, int port);
		~URLSocket();

		int write( char * buf);
		int isConnected() {
			return (sock != INVALID_SOCKET);
		}

		InputStream * getInputStream() {
			if (sock_stdin)
				return sock_stdin;
			sock_stdin =  new SocketInputStream( sock);
			return sock_stdin;
		}
		OutputStream * getOutputStream() {
			if (sock_stdout)
				return sock_stdout;
			sock_stdout =  new SocketOutputStream( sock);
			return sock_stdout;
		}
};

class ProxyNameList
{
	public:
		char * name;
		ProxyNameList * next;

		ProxyNameList( char * s) {
			name = strdup( s);
			next = (ProxyNameList*)0;
		}

		~ProxyNameList()
		{
			if (name)
				delete name;
		}

		int matches( char * s)
		{
			return (strcmp( s, name) == 0);
		}

		static ProxyNameList * add( char * s, ProxyNameList * l)
		{
			ProxyNameList * lt;
			if (!l)
				return new ProxyNameList( s);
			for (lt = l; lt;  lt = lt->next) {
				if (!l->matches(s))
					return l;
			}
			lt = new ProxyNameList( s);
			lt->next = l;
			return lt;
		}

		static ProxyNameList * remove( char * s, ProxyNameList * l)
		{
			ProxyNameList * lt;
			ProxyNameList * prev = (ProxyNameList*)0;

			for (lt = l; lt; prev = lt, lt = lt->next) {
				if (!l->matches(s))
					continue;
				if (!prev)
					l = lt->next;
				else {
					prev->next = lt->next;
				}
				delete lt;
				break;
			}
			return l;
		}

};


class SocketURLConnection: public URLConnection
{
	protected:
		URLSocket *   sock;
		char      *   serverhost;
		int           serverport;
		int url_usesproxy;

		int socketIsOk()
		{
			if (!sock)
				return 0;
			if (!sock->isConnected())
				return 0;
			return 1;
		}

	public:
		SocketURLConnection( char * s)
			: URLConnection( s) { sock = (URLSocket*)0;}

		void closeSocket() {
			if (sock)
				delete sock;
		}

		virtual int    getProxyPort() {
			return -1;
		}

		virtual char * getProxyHost() {
			return (char*)0;
		}

		virtual void    setProxyPort( int i) {}
		virtual void    setProxyHost( char * s) {}

		virtual ProxyNameList * getProxyIgnoreList() = 0;
		virtual char          * getProxyEnvVar() = 0;

		int isInList( char * el, ProxyNameList * l)
		{
			if (!l)
				return 0;
			if (strcmp( el, l->name) == 0)
				return 1;
			return isInList( el, l->next);
		}

		int needsProxy( char * host) {
			if (!hasProxy())
				return 0;
			ProxyNameList * pl = getProxyIgnoreList();
			if (!pl)
				return 1;
			if (isInList( host, pl))
				return 0;
			return 1;
		}

		int     hasProxy() {
			if (getProxyPort() == -1)
				return 0;
			if (!getProxyHost())
				return checkProxy();
			return 1;
		}

		int     setProxy( char * s) {
			char * q;

			if ((q = STRCHR( s, ':')) == (char*)0)
				return 0;
			*q = '\0';
			setProxyHost( s);
			setProxyPort( atoi(q+1));

			*q = ':';

			return 1;
		}

		int     checkProxy() {
			char * ev = getProxyEnvVar();
			if (!ev)
				return 0;
			ev = getenv( ev);
			if (ev) {
				if (setProxy(ev))
					return 1;
			}
			setProxyPort(-1);
			return 0;
		}
};


`#define HEADHEADER       "HEAD"
`#define PUTHEADER        "PUT"
`#define POSTHEADER        "POST"

`#define GETHEADER       "GET"
`#define GETTRAILER      "HTTP/1.0\r\n"
`#define EXTRALINE       "\r\n"

class URLStringUtils
{
	public:
		char * nextwhite( char * s)
		{
			if (!s)
				return (char*)0;

			while (*s && !isspace(*s))
				s++;
			return s;
		}

		char * overwhite( char * s)
		{
			if (!s)
				return (char*)0;

			while (*s && isspace(*s))
				s++;

			return s;
		}

		int isendofline( char * s) { return  (*s == '\r' || *s == '\n'); }

		char * endofline( char * s)
		{
			if (!s)
				return (char*)0;
			for (; *s && !isendofline(s); s++);

			return s;
		}

		char * overnullreplacement( char * s)
		{
			if (*s == '\r')
				*s++ = '\0';
			if (*s == '\n')
				*s++ = '\0';
			return s;
		}

		char * skipWord( char * p)
		{
			p = nextwhite( p);
			if (!p)
				return (char*)0;

			p = overwhite( p);
			if (!p)
				return (char*)0;
			return p;
		}
};


class HttpHeader: public URLStringUtils
{
	private:
		CachedInputStream * istream;

	protected:
		char * server;
		char * mime_version;
		char * content_type;
		int    content_length;
		char * header;

	public:
		HttpHeader( InputStream * i)
		{
			istream = new CachedInputStream( i);
			header = new char[CONN_SIZE+1];
			server = (char*)0;
			mime_version = (char*)0;
			content_type = (char*)0;
			content_length = 0;
		}
		~HttpHeader();

		int parse();
		void debugHeader();

		InputStream * getInputStream() { return istream; }

		char * getContentType()   { return content_type; }
		int    getContentLength() { return content_length; }

	private:

		char * getValue( char * q)
		{ // a:b split
			char * p;

			if ((p = STRCHR( q, ':')) == (char*)0)
				return (char*)0;
			q = p + 1;
			if ((q = overwhite( q)) == (char*)0)
				return (char*)0;

			*p = '\0';
			return q;
		}


		char * processHeaderLine( char * q);
};

class PropertyPair
{
	public:
		char * key;
		char * value;
		PropertyPair* next;

		PropertyPair( char * k, char * v)
		{
			key   = k;
			value = strdup(v);
			next  = (PropertyPair*)0;
		}

		~PropertyPair()
		{
			FREE( key);
			FREE(value);
		}

		void update ( char * v)
		{
			if (value)
				(void) FREE(value);
			value = strdup( v);
		}

		void write( char * buf)
		{
			(void) sprintf ( buf, "%s: %s", key, value);
		}

};

`#define IST_AGENT       "IST Application URL Library/c++ v1.0"

class HttpURLConnection: public SocketURLConnection
{
	private:
		void *    data;

		static char * proxy;
		static int    proxyport;
		static ProxyNameList * ignore_list;
		TempFileOutputStream   * tmpStream;
		PropertyPair * properties;

		char * resource;
		char * post_type;
		char * getPostType() {
			if (post_type)
				return post_type;
			return (char*)0;
		}
		int writeEndOfHeader();
		int writeContentHeader( char * type);
		int writeContentLength( int length);
		PropertyPair * current_property;

		void startPairEnumeration() {
			current_property = properties;
		}
		int getUserPair( char * buf) {
			if (!current_property)
				return 0;
			current_property->write( buf);
			current_property = current_property->next;
			return 1;
		}

		InputStream * readHeader();

		int getOutputLength() { return 0; }

		HttpHeader * header;
	public:
		HttpURLConnection( char * s)
			: SocketURLConnection( s)
		{
			post_type = (char*)0;
			header    = (HttpHeader*)0;
			// proxyport = -50;
			properties = (PropertyPair*)0;
			setRequestProperty("User-Agent", IST_AGENT);
			setRequestProperty("Accept","text/plain; q=1");
			setRequestProperty("Content-Type", "text/plain");
		}

		virtual void   setRequestProperty( char * key, char * value)
		{
			char * lkey = strdup(key);
			char * s;
			PropertyPair * pp;

			s = lkey;
			*s = toupper(*s);

			for (s++; s && *s; s++) {
				if (*(s-1) == '-')
					*s = toupper(*s);
				else
					*s = tolower(*s);
			}

			for ( pp = properties; pp; pp = pp->next) {
				if (strcmp(lkey, pp->key) == 0) {
					pp->update( value);
					(void) FREE(lkey);
					return;
				}
			}
			PropertyPair * p = new PropertyPair( lkey, value);
			p->next   = properties;
			properties = p;
		}

		int parseURL( char * s);

		void setPostContentType( char * s)
		{
			post_type = strdup(s);
		}

		static void setProxy( char * ph, int pp) {
			proxy     = strdup( ph);
			proxyport = pp;
		}

		static void setNoProxy() {
			if (proxy)
				delete proxy;
			proxy = (char*)0;
			proxyport = -1;
		}

		virtual void    setProxyPort( int i) { proxyport = i;}
		virtual void    setProxyHost( char * s) {
					if (proxy)
						delete proxy;
					if (!s)
						setNoProxy();
					else
						proxy = strdup( s);
		}

		int    getProxyPort() {
			return proxyport;
		}

		char * getProxyHost() {
			return proxy;
		}

		static void setHostNeedsProxy( char * host, int tf) {
			if (!tf)
				ignore_list = ProxyNameList::add( host, ignore_list);
			else
				ignore_list = ProxyNameList::remove( host, ignore_list);
		}

		ProxyNameList * getProxyIgnoreList() { return ignore_list; }

	public:
		char * getProtocol() {
			return "http";
		}

		char * getProxyEnvVar() {
			return "HTTP_PROXY";
		}

		int    getOutputSizeNeeded() { return 1; }

		int open();
		int close();

		OutputStream * getOutputStream();
		InputStream  * getInputStream();

		void postData()
		{
			if (!tmpStream)
				return;
		}

		char * getContentType()
		{
			if (!header)
				return (char*)0;
			return header->getContentType();
		}

		int    getContentLength()
		{
			if (!header)
				return 0;
			return header->getContentLength();
		}


};

class FileURLConnection: public URLConnection
{
	public:
		FileURLConnection( char * s)
			: URLConnection( s) {}

	public:
		char * getProtocol() {
			return "file";
		}

		int    getOutputSizeNeeded() { return 0; }

		int parseURL( char * s) { return 0; }
		int open();
		int close();

		OutputStream * getOutputStream() { return (OutputStream*)0; }
		InputStream  * getInputStream()  { return (InputStream*)0; }


};

class URL
{
	private:
		int    type;
		int    bad;
		char * urlstring;

		URLConnection * connection;

		int parseUrl( char * url, char * urldata);

		int isOk() { return !bad; }

		int checkConnection()
		{
			if (!isOk())
				return 0;
			if (!connection)
				connection = openConnection();
			return isOk();
		}

	public:
		URL( char * url, char * urldata=(char*)0);
		~URL();

		static URL * getURL( char * url, char * urldata=(char*)0);


		URLConnection* openConnection();

		URLConnection* getConnection() { return connection; }

		InputStream * openStream()
		{
			if (!checkConnection())
				return (InputStream*)0;
			return connection->getInputStream();
		}

		InputData * getContent()
		{
			if (!checkConnection())
				return (InputData*)0;
			return connection->getContent();
		}

};
`#endif
