`#include <stdio.h>
`#include <stdlib.h>

`#include "sc_groups_cpp.h"

int
http_transport_c::connect()
{
	return 1; // no permanent connections
}

void
http_transport_c::setProperties( sc_properties_c * p)
{
	properties = (http_properties_c*)p;
}

void
http_transport_c::setupProxyConfiguration()
{
	char * proxyHost = properties->getproxyHost();
	char * proxyPort = properties->getproxyPort();

	if (!proxyHost || !*proxyHost) {
		HttpURLConnection::setNoProxy();
		return;
	}

	int port;

	if (!proxyPort)
		port = 80;
	else
		port = atoi(proxyPort);

	HttpURLConnection::setProxy( proxyHost, port);
}

URL*
http_transport_c::getURL()
{
	char * url = properties->geturl();

	if (!url) {
		setSCError( "No URL");
		return (URL*)0;
	}
	char * urlData = properties->geturlData();

	return URL::getURL( url, urlData);
}


sc_idata_c *
http_transport_c::chatToServer( sc_stdcs_c * csdata)
{
	URL * u = getURL();
	if (!u) {
		csdata->warn( getSCError());
		return (http_idata_c*)0;
	}

	setupProxyConfiguration();

	URLConnection * uc = u->openConnection();
	if (!uc) {
		csdata->warn( getSCError());
		return (http_idata_c*)0;
	}
	if (csdata->hasOutputData()) {
		uc->setDoOutput(1);
		uc->setDoInput(1);
		uc->setAllowUserInteraction(0); 
		if (csdata->hasUserSendDataHandler()) {
			sc_outputhandler_f odh = csdata->getUserSendDataHandler();
			(*odh)( uc, csdata);
		} else {
			submitData( uc, csdata); // send data;
		}
	}
	return (sc_idata_c*) new http_idata_c( u);
}

void
http_transport_c::submitData( URLConnection * uc, sc_stdcs_c * csdata)
{
	PrintWriter * ps = new PrintWriter( uc->getOutputStream());
	sc_group_c  * g  = csdata->getGroup();

	ps->print("Content-type: ");
		ps->println( g->getMimeType());
	ps->println("");
	ps->print("sc-start=");
		ps->println(g->getMimeType());
	ps->print("sc-callback=");
		ps->println(csdata->getName());

	if (csdata->polling()) {
		ps->print("sc-poll=");
			ps->println(csdata->getPollInterval());
	}

	g->startAppElementEnumeration();

	while (g->hasMoreAppElements()) {
		httpSerialize( g->getNextAppElement(), ps);
	}

	ps->print("sc-end=");
		ps->println(g->getMimeType());

	ps->close();
	delete ps;
}

void
http_transport_c::httpSerialize( sc_control_c * control, PrintWriter * ps)
{
	int i = control->getType();
	if (i == ScUnknownType)
		return;
	ps->print( control->getName());
		ps->print("=");
	switch (i) {
		case ScBooleanType:
			{ int tv = ((sc_boolean_control_c*)control)->getControl();
			      ps->println( (char *)(tv ? "true" : "false"));
			}
			break;
		case ScIntType:
			ps->println( ((sc_int_control_c*)control)->getControl());
			break;
		case ScStringType:
			{ 
			  sc_string_control_c * sc = (sc_string_control_c*)control;

			  char * s = sc->getControl();
			  ps->println( s);
			  sc->free(s);
			}
			break;
		case ScStringArrayType:
			{
			char ** data = ((sc_stringArray_control_c*)control)->getControl();
			char ** pp;
			int i;

			for (i = 0, pp = data; pp && *pp; pp++, i++);

			ps->println( i);

			while (data && *data)
				ps->println( *data++);
			}
			break;
		default:
			break;
	}
}

sc_transport_c *
getNew_http_transport()
{
	return (sc_transport_c*) new http_transport_c();
}


void
http_idata_c::close()
{
	if (url)
		delete url;
}


`#define  StartState  0
`#define  DataState   1
`#define  GoalState   2


int
http_idata_c::splitLineAt( char c, char * line, char ** keyp, char ** eolp)
{
	char * p;

	for (p = line; p && *p; p++) {
		if (*p != c)
			continue;
		*p = '\0';
		*keyp = line;
		*eolp = p+1;
		return 1;
	}
	return 0;
}

int
http_idata_c::load( sc_stdcs_c * data)
{
	sc_group_c * g = data->getGroup();
	InputStream    * i  = getInputStream();
	DataInputStream * ds = new DataInputStream( i);
	char * nextline;

	char *callbackname = (char*)0;
	int state = StartState;
	char * key;
	char * toeol;

	while (state != GoalState) {
		nextline = ds->readLine();
		if (!nextline)
			break;

		if (!splitLineAt('=', nextline, &key, &toeol))
			break;

		switch (state) {
			case StartState:
				if (strcmp(key,"sc-start"))
					break;
				if (!g->isFrom(toeol)) {
					state = GoalState;
					break;
				}
				state = DataState;
				break;
			case DataState:
				if (strcmp(key,"sc-end") == 0) {
					state = GoalState;
					break;
				}
				if (strcmp(key,"sc-poll") == 0) {
					data->setPollInterval(atoi( toeol));
					break;
				}
				if (strcmp(key,"sc-callback") == 0) {
					
					break;
				}
				{
					sc_control_c * sc = g->findAppElement( key);
					if (!sc)
						break;
					httpDeSerialize( sc, toeol,  ds);
					break;
				}
			default:
				break;
		}
	}

	ds->close();

	delete ds;

	return 1;
}


void
http_idata_c::httpDeSerialize( sc_control_c * sc, char * toeol, DataInputStream * ds)
{
	if (!sc)
		return;
	int i = sc->getType();
	if (i == ScUnknownType)
		return;
	switch (i) {
		case ScBooleanType:
			{  int tv = (strcmp( toeol, "true") == 0);

			   ((sc_boolean_control_c*)sc)->setControlValue( tv);
			}
			break;
		case ScIntType:
			((sc_int_control_c*)sc)->setControlValue(atoi(toeol));
			break;
		case ScStringType:
			((sc_string_control_c*)sc)->setControlValue( toeol);
			break;
		case ScStringArrayType:
			{
				char ** values;
				char ** pp;
				int i = atoi(toeol);
				int j;
				if (i == 0)
					break;
				values = (char**) malloc ((unsigned) (i+1) * sizeof(char*));
				for (j=0, pp=values; j < i; j++, pp++) {
					*pp = ds->readLine();
					if (!*pp)
						break;
					*pp = strdup(*pp);
				}
				*pp = (char*)0;
				((sc_stringArray_control_c*)sc)->setControlValue( values);
			}
			break;
		default:
			break;
	}
}

void
rePollServer ( void *data, void* id)
{
	sc_stdcs_c * scd    = (sc_stdcs_c *)data;
	sc_transport_c * t  = scd->getTransport();

	t->chatAsync( scd);
}
