
`#ifndef _HTTP_TRANSPORT_H
`#define _HTTP_TRANSPORT_H

`#include "URL.h"

class http_idata_c: public sc_idata_c
{
	private:
		URL             * url;
		URLConnection   * connection;
		InputStream     * istream;

	public:

		http_idata_c( URL * u)
		{
			url        = u;
			connection = url->getConnection();
			istream    = connection->getInputStream(); // this does the handshake
		}
		void close();

		char * getMimeType() { return connection->getContentType(); }
		InputStream * getInputStream() { return istream; }
		int getContentLength() { return connection->getContentLength(); }

		int load( sc_stdcs_c * data);

	private:
		void httpDeSerialize( sc_control_c * sc, char * toeol, DataInputStream * ds);
		int splitLineAt( char c, char * line, char ** keyp, char ** eolp);

};

class http_properties_c: public sc_properties_c
{
	private:
		void * url;
		int    urltype;
		void * urldata;
		int    urldatatype;
		void * proxyhost;
		int    proxyhosttype;
		void * proxyport;
		int    proxyporttype;

	public:
		void   seturl( char* val)
		{
			url = (void*)val;
			urltype = PROP_CONSTANT;
		}

		void   seturl( char** val)
		{
			url = (void*)val;
			urltype = PROP_VARIABLE;
		}

		void   seturl( sc_getset_c * val)
		{
			url = (void*)val;
			urltype = PROP_FUNCTION;
		}

		char * geturl()
		{
			return derefValue(url, urltype);
		}

		void   seturlData( char* val)
		{
			urldata = (void*)val;
			urldatatype = PROP_CONSTANT;
		}

		void   seturlData( char** val)
		{
			urldata = (void*)val;
			urldatatype = PROP_VARIABLE;
		}

		void   seturlData( sc_getset_c * val)
		{
			urldata = (void*)val;
			urldatatype = PROP_FUNCTION;
		}

		char * geturlData()
		{
			return derefValue(urldata, urldatatype);
		}

		void   setproxyHost( char* val)
		{
			proxyhost = (void*)val;
			proxyhosttype = PROP_CONSTANT;
		}

		void   setproxyHost( char** val)
		{
			proxyhost = (void*)val;
			proxyhosttype = PROP_VARIABLE;
		}

		void   setproxyHost( sc_getset_c * val)
		{
			proxyhost = (void*)val;
			proxyhosttype = PROP_FUNCTION;
		}

		char * getproxyHost()
		{
			return derefValue(proxyhost, proxyhosttype);
		}

		void   setproxyPort( char* val)
		{
			proxyport = (void*)val;
			proxyporttype = PROP_CONSTANT;
		}

		void   setproxyPort( char** val)
		{
			proxyport = (void*)val;
			proxyporttype = PROP_VARIABLE;
		}

		void   setproxyPort( sc_getset_c * val)
		{
			proxyport = (void*)val;
			proxyporttype = PROP_FUNCTION;
		}

		char * getproxyPort()
		{
			return derefValue(proxyport, proxyporttype);
		}
};

class http_transport_c: public sc_transport_impl_c
{
	private:
		http_properties_c * properties;

		int connect();
		void setProperties( sc_properties_c * p);
		sc_idata_c * chatToServer( sc_stdcs_c * csdata);

		void setupProxyConfiguration();
		URL * getURL();
		void submitData( URLConnection * uc, sc_stdcs_c * csdata);
		void httpSerialize( sc_control_c * control, PrintWriter * ps);
};

`#endif
