`#include <stdio.h>

`#include "sc_types.h"
`#include "server.h"
`#include "Ustreams.h"
`#include "server_transport.h"


void
server_transport_c::httpDeSerialize( sc_control_c * sc, char * toeol, DataInputStream * ds)
{
	if (!sc)
		return;
	int i = sc->getType();
	if (i == ScUnknownType)
		return;
	switch (i) {
		case ScBooleanType:
			{  int tv = (strcmp( toeol, "true") == 0);

			   ((sc_boolean_control_c*)sc)->set( tv);
			}
			break;
		case ScIntType:
			((sc_int_control_c*)sc)->set(atoi(toeol));
			break;
		case ScStringType:
			((sc_string_control_c*)sc)->set( toeol);
			break;
		case ScStringArrayType:
			{
				char ** values;
				char ** pp;
				int i = atoi(toeol);
				int j;
				if (i == 0)
					break;
				values = (char**) malloc ((unsigned) (i+1) * sizeof(char*));
				for (j=0, pp=values; j < i; j++, pp++) {
					*pp = ds->readLine();
					if (!*pp)
						break;
					*pp = strdup(*pp);
				}
				*pp = (char*)0;
				((sc_stringArray_control_c*)sc)->set(values);
			}
			break;
		default:
			break;
	}
}


`#define  StartState  0
`#define  DataState   1
`#define  GoalState   2


int
server_transport_c::splitLineAt( char c, char * line, char ** keyp, char ** eolp)
{
	char * p;

	for (p = line; p && *p; p++) {
		if (*p != c)
			continue;
		*p = '\0';
		*keyp = line;
		*eolp = p+1;
		return 1;
	}
	return 0;
}


sc_data_c *
server_transport_c::load( server_c * server, InputStream * i)
{
	DataInputStream * ds = new DataInputStream( i);
	char * nextline;

	int   pollvalue = 0;
	char *callbackname = (char*)0;
	int state = StartState;
	char * key;
	char * toeol;
	char   mimeType[255];
	sc_data_c * mydata = (sc_data_c*)0;

	while (state != GoalState) {
		nextline = ds->readLine();
		if (!nextline)
			break;

		if (!splitLineAt('=', nextline, &key, &toeol))
			continue;

		switch (state) {
			case StartState:
				if (strcmp(key,"sc-start"))
					break;
				(void) strcpy( mimeType, toeol);
				state = DataState;
				break;
			case DataState:
				if (strcmp(key,"sc-callback") == 0)
					mydata = server->newData( toeol, mimeType);
				if (!mydata) {
					state = GoalState;
					break;
				}
				if (strcmp(key,"sc-end") == 0) {
					state = GoalState;
					break;
				}
				if (strcmp(key,"sc-poll") == 0) {
					mydata->setPoll( atoi( toeol));
					break;
				}
				{
					sc_control_c * sc = mydata->getGroup()->findAppElement( key);
					if (!sc)
						break;
					httpDeSerialize( sc, toeol,  ds);
					break;
				}
			default:
				break;
		}
	}

	ds->close();

	delete ds;

	return mydata;
}

sc_data_c *
server_transport_c::receive( server_c * sc)
{
	StdInputStream * si = new StdInputStream(0);
	sc_data_c * rval    = load( sc, si);

	delete si;
	return rval;
}

void
server_transport_c::httpSerialize( sc_control_c * control, PrintWriter * ps)
{
	int i = control->getType();
	if (i == ScUnknownType)
		return;
	ps->print( control->getName());
		ps->print((char *) "=");
	switch (i) {
		case ScBooleanType:
			{ int tv = ((sc_boolean_control_c*)control)->get();
			      ps->println( (char *) (tv ? "true" : "false"));
			}
			break;
		case ScIntType:
			ps->println( ((sc_int_control_c*)control)->get());
			break;
		case ScStringType:
			{ // its a copy
			  sc_string_control_c * sc = (sc_string_control_c*)control;

			  char * s = sc->get();
			  ps->println( s);
			  sc->free(s);
			}
			break;
		case ScStringArrayType:
			{
			char ** data = ((sc_stringArray_control_c*)control)->get();
			char ** pp;
			int i;

			for (i = 0, pp = data; pp && *pp; pp++, i++);

			ps->println( i);

			while (data && *data)
				ps->println( *data++);
			}
			break;
		default:
			break;
	}
}


void
server_transport_c::send_data( PrintWriter * pw, sc_data_c* sd)
{
	sc_group_c  * g  = sd->getGroup();

	pw->print((char *) "sc-start=");
		pw->println(g->getMimeType());
	pw->print((char *) "sc-callback=");
		pw->println(sd->getName());

	if (sd->getPoll()) {
		pw->print((char *) "sc-poll=");
			pw->println(sd->getPoll());
	}

	g->startAppElementEnumeration();

	while (g->hasMoreAppElements()) {
		httpSerialize( g->getNextAppElement(), pw);
	}

	pw->print((char *) "sc-end=");
		pw->println(g->getMimeType());

}

void
server_transport_c::send_header( PrintWriter * pw, char * mimeType)
{
	pw->print( (char *) "Content-type: ");
		pw->println( mimeType);
		pw->println( (char*)0);
}

void
server_transport_c::send( sc_data_c* sd)
{
	StdOutputStream * so = new StdOutputStream(1);
	PrintWriter     * pw = new PrintWriter(so);

	send_header( pw, sd->getGroup()->getMimeType());
	send_data( pw, sd);

	pw->close();
	delete pw;
	delete so;
}
