#include "Motif.m"
#include "SimpleSpinBoxBase.m"
#include "XmStringUtils.m"

#module   SimpleSpinBox
#bind     SimpleSpinBoxBase
#bind     XmStringUtils
#bind     Motif

#def ExtraHeaders(void)
`#include "Xm/Literal(SSpi)nB.h"
#enddef

#include "motifprelude.std"

#def    motif.Value.get(w)
	XmString     *table        = (XmString *) 0 ;
	int           maximum      = 0 ;
	int           minimum      = 0 ;
	int           increment    = 0 ;
	int           position     = 0 ;
	int           positionType = XmPOSITION_VALUE ;
	int           table_count  = 0 ;
	unsigned char type         = XmSTRING ;
	char         *value        = (char *) 0 ;
	char          buffer[32] ;

	XtVaGetValues(w, XmNvalues,           &table,
			 XmNnumValues,        &table_count,
			 XmNpositionType,     &positionType,
			 XmNposition,         &position,
			 XmNminimumValue,     &minimum,
			 XmNmaximumValue,     &maximum,
			 XmNincrementValue,   &increment,
			 XmNspinBoxChildType, &type,
			 NULL) ;

	if (type == XmSTRING) {
		if (position > table_count) {
			return (char *) 0 ;
		}

		if (table == (XmString *) 0) {
			return (char *) 0 ;
		}

		if (table[position] == (XmString) 0) {
			return (char *) 0 ;
		}

		cvtXmStringToText(table[position], value) ;

		return value ;
	}

	if (positionType == XmPOSITION_VALUE) {
		(void) sprintf(buffer, "%d", position) ;
	}
	else {
		int ivalue = minimum + (increment * position) ;

		if (ivalue > maximum) {
			ivalue = maximum ;
		}

		(void) sprintf(buffer, "%d", ivalue) ;
	}
		
	return XtNewString(buffer) ;
#enddef

#def    motif.Value.set(w,val)
	int           maximum      = 0 ;
	int           minimum      = 0 ;
	int           increment    = 0 ;
	int           position     = 0 ;
	int           max_position = 0 ;
	int           min_position = 0 ;
	int           positionType = XmPOSITION_VALUE ;
	unsigned char type         = XmSTRING ;
	
	XtVaGetValues(w, XmNpositionType,     &positionType,
			 XmNminimumValue,     &minimum,
			 XmNmaximumValue,     &maximum,
			 XmNincrementValue,   &increment,
			 XmNspinBoxChildType, &type,
			 NULL) ;

	if (type == XmSTRING) {
		XmString xms = (XmString) 0 ;

		cvtTextToXmString(val, xms) ;

		XmSimpleSpinBoxSetItem(w, xms) ;
	}
	else {
		position = atoi(val) ;

		if (positionType == XmPOSITION_INDEX) {
			if (increment < 1) {
				increment = 1 ;
			}

			max_position = ((maximum - minimum) / increment) ;
			min_position = 0 ;
		}
		else {
			max_position = maximum ;
			min_position = minimum ;
		}

		if (position < min_position) {
			position = min_position ;
		}

		if (position > max_position) {
			position = max_position ;
		}

		XtVaSetValues(w, XmNposition, position, NULL) ;
	}
#enddef

#def    motif.Items.get(w)
	XmStringTable xmst   = (XmStringTable) 0 ;
	int           cnt    = 0 ;
	char        **values = (char **) 0 ;
	unsigned char type ;

        XtVaGetValues(w, XmNnumValues,        &cnt,
                         XmNvalues,           &xmst,
	                 XmNspinBoxChildType, &type,
			 NULL) ;

	if (type == XmSTRING) {
        	XmStringUtils.cvtXmStringTableToStringArray(xmst,values,cnt) ;
	}

	return values ;
#enddef

#def    motif.Items.set(w,sdata)
        XmStringTable xmst = (XmStringTable) 0 ;
        int           cnt  = 0 ;
        /* 1= w 2= sdata */

        XmStringUtils.cvtStringArrayToXmStringTable(sdata,xmst,cnt)

	XtVaSetValues(w, XmNnumValues, cnt,
			 XmNvalues,    xmst,
			 NULL) ;
				 
	XmStringUtils.freeXmStringTable(xmst,cnt)
#enddef

#include "BINDING/Literal(SimpleSpi)nBox.m"

#endmodule
