/*
**LIBS: -lsocket -lnsl -lgen
*/

#include <stdio.h>
#include <sys/types.h>

#include "URL.h"

/* This C code follows the same URL/URLConnection API as the C++
** version. This is a subset of the Java(TM) API for URLs.
** Anything you write using this API you should be able to rewrite in
** Java easily.
**
** The "public" objects are
**
** URL
** abstract URLConnection
**      (abstract SocketURLConnection)
**              HttpURLConnection
**      FileURLConnection
**
** (URLSocket)
** (HttpHeader)
** (ProxyNameList)
*/

/* Constructors Destructors and declarations */

/* URL */

#ifndef   _NO_PROTO
static URLConnection* URL_openConnection(URL *);
static URLConnection* URL_getConnection(URL *);
static InputStream *  URL_openStream(URL *);
static InputData *    URL_getContent(URL *);
#else  /* _NO_PROTO */
static URLConnection* URL_openConnection();
static URLConnection* URL_getConnection();
static InputStream *  URL_openStream();
static InputData *    URL_getContent();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteURL( URL * url)
#else  /* _NO_PROTO */
void deleteURL( url)
	URL * url ;
#endif /* _NO_PROTO */
{
	if (url->connection) {
		(*url->connection->close)( url->connection);
		(*url->connection->delete)( url->connection);
	}
	if (url->urlstring) {
		(void) free ((void*) url->urlstring);
	}
	(void) free ((void*) url);
}

#ifndef   _NO_PROTO
static int URL_parseUrl(URL *u, char *url, char *urldata) ;
#else  /* _NO_PROTO */
static int URL_parseUrl() ;
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
URL * newURLandData( char * url, char * urldata)
#else  /* _NO_PROTO */
URL * newURLandData( url, urldata)
	char * url ;
	char * urldata ;
#endif /* _NO_PROTO */
{
	URL * u = (URL*) malloc ((unsigned) sizeof(URL));

	u->bad = !URL_parseUrl( u, url, urldata);
	if (u->bad) {
		deleteURL( u);
		return (URL*)0;
	}
	u->connection     = (URLConnection*)0;
	u->delete         =  deleteURL;
	u->openStream     = URL_openStream;
	u->getContent     = URL_getContent;
	u->openConnection = URL_openConnection;
	u->getConnection  = URL_getConnection;

	return u;
}

#ifndef   _NO_PROTO
URL * newURL( char * url)
#else  /* _NO_PROTO */
URL * newURL( url)
	char *url ;
#endif /* _NO_PROTO */
{
	return newURLandData( url, (char*)0);
}

/* URLConnectionData */

#ifndef   _NO_PROTO
void deleteURLConnectionData( URLConnectionData * ud)
#else  /* _NO_PROTO */
void deleteURLConnectionData( ud)
	URLConnectionData * ud ;
#endif /* _NO_PROTO */
{
	if (ud->urlstring)
		(void) free ((void*) ud->urlstring);
	(void) free ((void*) ud);
}

#ifndef   _NO_PROTO
URLConnectionData * newURLConnectionData( char * s)
#else  /* _NO_PROTO */
URLConnectionData * newURLConnectionData( s)
	char * s ;
#endif /* _NO_PROTO */
{
	URLConnectionData * ud = (URLConnectionData*) malloc ((unsigned) sizeof( URLConnectionData));

	ud->doOutput = 0;
	ud->doInput  = 1;
	ud->allowUserInteraction = 0;
	ud->ok       = 0;
	ud->isconnected = 0;
	ud->urlstring = strdup( s);
	ud->delete    = deleteURLConnectionData;

	return ud;
}

/* URLConnection is abstract */

#ifndef   _NO_PROTO
static int              URLConnection_isOk(URLConnection *);
static void             URLConnection_setDoOutput(URLConnection *, int);
static void             URLConnection_setDoInput(URLConnection *, int);
static void             URLConnection_setAllowUserInteraction(URLConnection *, int);
static void             URLConnection_setBad(URLConnection *);
static void             URLConnection_connect(URLConnection *);
static char *           URLConnection_getContentType(URLConnection *);
static int              URLConnection_getContentLength(URLConnection *);
static InputData *      URLConnection_getContent(URLConnection *);
#else  /* _NO_PROTO */
static int              URLConnection_isOk();
static void             URLConnection_setDoOutput();
static void             URLConnection_setDoInput();
static void             URLConnection_setAllowUserInteraction();
static void             URLConnection_setBad();
static void             URLConnection_connect();
static char *           URLConnection_getContentType();
static int              URLConnection_getContentLength();
static InputData *      URLConnection_getContent();
#endif /* _NO_PROTO */


/* FileURLConnection */

#ifndef   _NO_PROTO
static OutputStream * FileURLConnection_getOutputStream (FileURLConnection *);
static InputStream *  FileURLConnection_getInputStream (FileURLConnection *);
static int            FileURLConnection_open (FileURLConnection *);
static int            FileURLConnection_close(FileURLConnection *);
static int            FileURLConnection_getOutputSizeNeeded(FileURLConnection *);
static int            FileURLConnection_parseURL(FileURLConnection *, char *);
static char *         FileURLConnection_getProtocol(FileURLConnection *);
#else  /* _NO_PROTO */
static OutputStream * FileURLConnection_getOutputStream ();
static InputStream *  FileURLConnection_getInputStream ();
static int            FileURLConnection_open ();
static int            FileURLConnection_close();
static int            FileURLConnection_getOutputSizeNeeded();
static int            FileURLConnection_parseURL();
static char *         FileURLConnection_getProtocol();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteFileURLConnection( FileURLConnection * furlc)
#else  /* _NO_PROTO */
void deleteFileURLConnection( furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	(void) free ((void*) furlc);
}

#ifndef   _NO_PROTO
FileURLConnection * newFileURLConnection( char * urlstring)
#else  /* _NO_PROTO */
FileURLConnection * newFileURLConnection( urlstring)
	char * urlstring ;
#endif /* _NO_PROTO */
{
	FileURLConnection * furlc = (FileURLConnection*) malloc ((unsigned) sizeof(FileURLConnection));

	furlc->delete   = deleteFileURLConnection;
	furlc->rawdata  = newURLConnectionData( urlstring);
	furlc->setDoOutput = URLConnection_setDoOutput;
	furlc->setDoInput  = URLConnection_setDoInput;
	furlc->setAllowUserInteraction = URLConnection_setAllowUserInteraction;
	furlc->getContentType          = URLConnection_getContentType;
	furlc->getContentLength        = URLConnection_getContentLength;
	furlc->getContent              = URLConnection_getContent;
	furlc->isOk                    = URLConnection_isOk;

	furlc->getOutputStream  = FileURLConnection_getOutputStream;
	furlc->getInputStream   = FileURLConnection_getInputStream;
	furlc->open             = FileURLConnection_open;
	furlc->close            = FileURLConnection_close;
	furlc->getOutputSizeNeeded      = FileURLConnection_getOutputSizeNeeded;
	furlc->getProtocol      = FileURLConnection_getProtocol;
	furlc->parseURL         = FileURLConnection_parseURL;

	return furlc;
}

/* SocketURLConnectionData */

#ifndef   _NO_PROTO
static void deleteSocketURLConnectionData( SocketURLConnectionData* sd)
#else  /* _NO_PROTO */
static void deleteSocketURLConnectionData( sd)
	SocketURLConnectionData* sd ;
#endif /* _NO_PROTO */
{
	if (sd->sock) {
		(*sd->sock->delete) (sd->sock);
	}
	(void) free ((void*) sd);
}

#ifndef   _NO_PROTO
static SocketURLConnectionData * newSocketURLConnectionData(void)
#else  /* _NO_PROTO */
static SocketURLConnectionData * newSocketURLConnectionData()
#endif /* _NO_PROTO */
{
	SocketURLConnectionData * sd = (SocketURLConnectionData*) malloc ((unsigned) sizeof( SocketURLConnectionData));

	sd->sock = (URLSocket*)0;
	sd->serverhost = (char*)0;
	sd->serverport = 0;
	sd->url_usesproxy = 0;
	sd->proxyhost_p   = (char**)0;
	sd->proxyport_p   = (int*)0;
	sd->ignore_list_p = (ProxyNameList**)0;
	sd->delete        = deleteSocketURLConnectionData;
	return sd;

}

/* HttpURLConnectionData */

#ifndef   _NO_PROTO
void deleteHttpURLConnectionData( HttpURLConnectionData * hucd)
#else  /* _NO_PROTO */
void deleteHttpURLConnectionData( hucd)
	HttpURLConnectionData * hucd ;
#endif /* _NO_PROTO */
{
	(void) (free) ((void*) hucd);
}

#ifndef   _NO_PROTO
HttpURLConnectionData * newHttpURLConnectionData(void)
#else  /* _NO_PROTO */
HttpURLConnectionData * newHttpURLConnectionData()
#endif /* _NO_PROTO */
{
	HttpURLConnectionData * hcd = (HttpURLConnectionData*) malloc ((unsigned) sizeof(HttpURLConnectionData));

	hcd->delete = deleteHttpURLConnectionData;
	hcd->data   = (void*)0;
	hcd->tmpStream = (TempFileOutputStream*)0;
	hcd->resource  = (char*)0;
	hcd->post_type = (char*)0;
	hcd->header    = (HttpHeader*)0;

	return hcd;
}

#ifndef   _NO_PROTO
static ProxyNameList *SocketURLConnection_getProxyStopList(SocketURLConnection * suc) ;
#else  /* _NO_PROTO */
static ProxyNameList *SocketURLConnection_getProxyStopList() ;
#endif /* _NO_PROTO */

/* HttpURLConnection */

#ifndef   _NO_PROTO
static OutputStream *   HttpURLConnection_getOutputStream (HttpURLConnection *) ;
static InputStream *    HttpURLConnection_getInputStream (HttpURLConnection *);
static int              HttpURLConnection_open (HttpURLConnection *);
static int              HttpURLConnection_close(HttpURLConnection *);
static int              HttpURLConnection_getOutputSizeNeeded(HttpURLConnection *);
static int              HttpURLConnection_parseURL(HttpURLConnection *, char *);
static char *           HttpURLConnection_getProtocol(HttpURLConnection *);
static char *           HttpURLConnection_getProxyEnvVar(HttpURLConnection *);
static char *           HttpURLConnection_getContentType(HttpURLConnection *);
static int              HttpURLConnection_getContentLength(HttpURLConnection *);
static InputData *      HttpURLConnection_getContent(HttpURLConnection *);
#else  /* _NO_PROTO */
static OutputStream *   HttpURLConnection_getOutputStream ();
static InputStream *    HttpURLConnection_getInputStream ();
static int              HttpURLConnection_open ();
static int              HttpURLConnection_close();
static int              HttpURLConnection_getOutputSizeNeeded();
static int              HttpURLConnection_parseURL();
static char *           HttpURLConnection_getProtocol();
static char *           HttpURLConnection_getProxyEnvVar();
static char *           HttpURLConnection_getContentType();
static int              HttpURLConnection_getContentLength();
static InputData *      HttpURLConnection_getContent();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteHttpURLConnection( HttpURLConnection * hurlc)
#else  /* _NO_PROTO */
void deleteHttpURLConnection( hurlc)
	HttpURLConnection * hurlc ;
#endif /* _NO_PROTO */
{
	PropertyPair * pp;
	PropertyPair * ppd;

	if (hurlc->rawdata)
		(*hurlc->rawdata->delete)( hurlc->rawdata);
	if (hurlc->rawsocketdata)
		(*hurlc->rawsocketdata->delete)( hurlc->rawsocketdata);
	if (hurlc->rawhttpdata)
		(*hurlc->rawhttpdata->delete)( hurlc->rawhttpdata);

	pp = hurlc->properties;
	while (pp) {
		ppd = pp;
		pp = pp->next;
		(void) free ((void*) ppd->key);
		(void) free ((void*) ppd->value);
		(void) free ((void*) ppd);
	}
	(void) free ((void*) hurlc);
}


static char          * http_proxyhost     = (char*)0;
static int             http_proxyport     = -50;
static ProxyNameList * http_proxystoplist = (ProxyNameList*)0;

/* ab add */
#ifndef   _NO_PROTO
void HttpURLConnection_setNoProxy(void)
#else  /* _NO_PROTO */
void HttpURLConnection_setNoProxy()
#endif /* _NO_PROTO */
{
	if (http_proxyhost)
		(void) free ((void*)http_proxyhost);
	http_proxyhost = (char*)0;
	http_proxyport = -50;
}

#ifndef   _NO_PROTO
void HttpURLConnection_setProxy( char * host, int port)
#else  /* _NO_PROTO */
void HttpURLConnection_setProxy( host, port)
	char *host ;
	int   port ;
#endif /* _NO_PROTO */
{
	if (http_proxyhost)
		(void) free ((void*)http_proxyhost);
	http_proxyhost = strdup( host);
	http_proxyport = port;
}

#ifndef   _NO_PROTO
static void HttpURLConnection_setRequestProperty(HttpURLConnection *, char *, char *);
#else  /* _NO_PROTO */
static void HttpURLConnection_setRequestProperty();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
HttpURLConnection * newHttpURLConnection( char * urlstring)
#else  /* _NO_PROTO */
HttpURLConnection * newHttpURLConnection( urlstring)
	char * urlstring ;
#endif /* _NO_PROTO */
{
	HttpURLConnection * hc = (HttpURLConnection*) malloc ((unsigned) sizeof(HttpURLConnection));

	hc->delete         = deleteHttpURLConnection;
	hc->rawdata        = newURLConnectionData( urlstring);
	hc->rawsocketdata  = newSocketURLConnectionData();

	/* two statics */
	hc->rawsocketdata->proxyhost_p = &http_proxyhost;
	hc->rawsocketdata->proxyport_p = &http_proxyport;
	hc->rawsocketdata->ignore_list_p = &http_proxystoplist;

	hc->rawhttpdata    = newHttpURLConnectionData();
	hc->setDoOutput = URLConnection_setDoOutput;
	hc->setDoInput  = URLConnection_setDoInput;
	hc->setAllowUserInteraction = URLConnection_setAllowUserInteraction;
	hc->getContentType          = HttpURLConnection_getContentType;
	hc->getContentLength        = HttpURLConnection_getContentLength;
	hc->getContent              = HttpURLConnection_getContent;
	hc->isOk                    = URLConnection_isOk;

	hc->getOutputStream             = HttpURLConnection_getOutputStream;
	hc->getInputStream              = HttpURLConnection_getInputStream;
	hc->open                        = HttpURLConnection_open;
	hc->close                       = HttpURLConnection_close;
	hc->getOutputSizeNeeded         = HttpURLConnection_getOutputSizeNeeded;
	hc->getProtocol                 = HttpURLConnection_getProtocol;
	hc->parseURL                    = HttpURLConnection_parseURL;

	hc->getProxyStopList            = SocketURLConnection_getProxyStopList;
	hc->getProxyEnvVar              = HttpURLConnection_getProxyEnvVar;
	hc->setRequestProperty          = HttpURLConnection_setRequestProperty;
	hc->properties                  = (PropertyPair*)0;

	HttpURLConnection_setRequestProperty( hc, "Accept", "text/plain; q=1");
	HttpURLConnection_setRequestProperty( hc, "Content-Type", "text/plain");
	HttpURLConnection_setRequestProperty( hc, "User-Agent", IST_AGENT);

	return hc;
}


/* HttpHeader */

#ifndef   _NO_PROTO
static int           HttpHeader_parse(HttpHeader *);
static InputStream * HttpHeader_getInputStream(HttpHeader *);
static char *        HttpHeader_getContentType(HttpHeader *);
static int           HttpHeader_getContentLength(HttpHeader *);
#else  /* _NO_PROTO */
static int           HttpHeader_parse();
static InputStream * HttpHeader_getInputStream();
static char *        HttpHeader_getContentType();
static int           HttpHeader_getContentLength();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteHttpHeader( HttpHeader * hh)
#else  /* _NO_PROTO */
void deleteHttpHeader( hh)
	HttpHeader * hh ;
#endif /* _NO_PROTO */
{
	if (hh->istream)
		(*hh->istream->delete)(hh->istream);
	if (hh->header)
		(void) free ((void*)hh->header);
	(void) free ((void*)hh);
}

#ifndef   _NO_PROTO
HttpHeader * newHttpHeader( InputStream * i)
#else  /* _NO_PROTO */
HttpHeader * newHttpHeader( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	extern CachedInputStream * newCachedInputStream();

	HttpHeader * hh = (HttpHeader*) malloc ((unsigned) sizeof(HttpHeader));

	hh->istream = newCachedInputStream( i);
	hh->header  = (char*) malloc ((unsigned)(CONN_SIZE+1) * sizeof(char));
	hh->server  = (char*)0;
	hh->mime_version = (char*)0;
	hh->content_type = (char*)0;
	hh->content_length = 0;

	hh->delete           = deleteHttpHeader;
	hh->parse            = HttpHeader_parse;
	hh->getInputStream   = HttpHeader_getInputStream;
	hh->getContentType   = HttpHeader_getContentType;
	hh->getContentLength = HttpHeader_getContentLength;

	return hh;
}


/* URLSocket */

#ifndef   _NO_PROTO
static int              URLSocket_write(URLSocket *, char *);
static int              URLSocket_isConnected(URLSocket *);
static InputStream *    URLSocket_getInputStream(URLSocket *);
static OutputStream *   URLSocket_getOutputStream(URLSocket *);
static SocketHandle     URLSocket_open(URLSocket *, char *, int);
#else  /* _NO_PROTO */
static int              URLSocket_write();
static int              URLSocket_isConnected();
static InputStream *    URLSocket_getInputStream();
static OutputStream *   URLSocket_getOutputStream();
static SocketHandle     URLSocket_open();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteURLSocket( URLSocket * usock)
#else  /* _NO_PROTO */
void deleteURLSocket( usock)
	URLSocket * usock ;
#endif /* _NO_PROTO */
{
	if (usock->sock > 0)
		(void) close ( usock->sock);

	if (usock->sock_stdin)
		(*usock->sock_stdin->delete)( usock->sock_stdin);
	if (usock->sock_stdout)
		(*usock->sock_stdout->delete)( usock->sock_stdout);

	(void) free ((void*) usock);
}

#ifndef   _NO_PROTO
URLSocket * newURLSocket( char *hostname, int port)
#else  /* _NO_PROTO */
URLSocket * newURLSocket( hostname, port)
	char *hostname ;
	int   port ;
#endif /* _NO_PROTO */
{
	URLSocket * usock = (URLSocket*) malloc ((unsigned) sizeof(URLSocket));

	usock->sock_stdin  = (SocketInputStream*)0;
	usock->sock_stdout = (SocketOutputStream*)0;
	usock->urlErrno    = 0;
	usock->sock        = URLSocket_open( usock, hostname, port);

	usock->delete          = deleteURLSocket;
	usock->write           = URLSocket_write;
	usock->isConnected     = URLSocket_isConnected;
	usock->getInputStream  = URLSocket_getInputStream;
	usock->getOutputStream = URLSocket_getOutputStream;

	return usock;
}

/* ProxyNameList */

#ifndef   _NO_PROTO
void deleteProxyNameList( ProxyNameList * pnl)
#else  /* _NO_PROTO */
void deleteProxyNameList( pnl)
	ProxyNameList * pnl ;
#endif /* _NO_PROTO */
{
	if (pnl->name)
		(void) free ((void*) pnl->name);
	(void) free ((void*) pnl);
}

#ifndef   _NO_PROTO
ProxyNameList * newProxyNameList( char * s)
#else  /* _NO_PROTO */
ProxyNameList * newProxyNameList( s)
	char * s ;
#endif /* _NO_PROTO */
{
	ProxyNameList * pnl = (ProxyNameList*) malloc ((unsigned) sizeof(ProxyNameList));
			pnl->name = strdup( s);
			pnl->next = (ProxyNameList*)0;

	return pnl;
}


/* METHODS */

/* URLSocket */

#ifndef   _NO_PROTO
static SocketHandle URLSocket_open( URLSocket * usock, char *hostname, int port)
#else  /* _NO_PROTO */
static SocketHandle URLSocket_open( usock, hostname, port)
	URLSocket *usock ;
	char      *hostname ;
	int        port ;
#endif /* _NO_PROTO */
{
	struct sockaddr_in sktin;
	struct hostent * host;
	SocketHandle     sock = INVALID_SOCKET;

	MEMSET ( &sktin, 0, sizeof( sktin));
	sktin.sin_family = AF_INET;
	sktin.sin_port   = htons( port);

	if ((host = gethostbyname( hostname)) != (struct hostent *)0) {
		MEMCPY ( &sktin.sin_addr, host->h_addr, host->h_length);
	} else
	if ((sktin.sin_addr.s_addr = inet_addr( hostname)) == (unsigned long)-1) {
		usock->urlErrno = E_NoRouteToHost;
		return INVALID_SOCKET;
	}

	sock = socket( PF_INET, SOCK_STREAM, 0);
	if (IS_INVALID_SOCKET(sock)) {
		perror("socket()");
		usock->urlErrno = E_CreateSocket;
		return INVALID_SOCKET;
	} else if (sock == 0) {
#if 0
		if (debugfid)
			(void) fprintf ( debugfid, "zero on %s %d<p>\n", hostname, port);
#endif
	}

	if ( connect( sock,(struct sockaddr*) &sktin, sizeof( sktin)) < 0) {
		(void) close ( sock);
		sock = INVALID_SOCKET;
		perror("connect()");
		usock->urlErrno = E_CannotConnectToHost;
		return INVALID_SOCKET;
	}

	return sock;
}

#ifndef   _NO_PROTO
static int URLSocket_write( URLSocket * usock, char * buf)
#else  /* _NO_PROTO */
static int URLSocket_write(  usock, buf)
	URLSocket * usock ;
	char      * buf ;
#endif /* _NO_PROTO */
{
#ifdef    WIN32
	return send( usock->sock, buf, strlen( buf), 0);
#else  /* WIN32 */
	return write( usock->sock, buf, strlen( buf));
#endif /* WIN32 */
}

#ifndef   _NO_PROTO
static int URLSocket_isConnected( URLSocket * usock)
#else  /* _NO_PROTO */
static int URLSocket_isConnected( usock)
	URLSocket * usock ;
#endif /* _NO_PROTO */
{
	return !(IS_INVALID_SOCKET(usock->sock));
}

#ifndef   _NO_PROTO
static InputStream * URLSocket_getInputStream( URLSocket * usock)
#else  /* _NO_PROTO */
static InputStream * URLSocket_getInputStream( usock)
	URLSocket * usock ;
#endif /* _NO_PROTO */
{
	extern SocketInputStream * newSocketInputStream();
	if (usock->sock_stdin)
		return (InputStream*)(usock->sock_stdin);
	usock->sock_stdin =  newSocketInputStream( usock->sock);
	return (InputStream*)(usock->sock_stdin);
}

#ifndef   _NO_PROTO
static OutputStream * URLSocket_getOutputStream( URLSocket * usock)
#else  /* _NO_PROTO */
static OutputStream * URLSocket_getOutputStream( usock)
	URLSocket * usock ;
#endif /* _NO_PROTO */
{
	extern SocketOutputStream * newSocketOutputStream();
	if (usock->sock_stdout)
		return (OutputStream*)(usock->sock_stdout);
	usock->sock_stdout =  newSocketOutputStream( usock->sock);
	return (OutputStream*)(usock->sock_stdout);
}

#ifndef   _NO_PROTO
static int ProxyNameList_matches( char * s, char * name)
#else  /* _NO_PROTO */
static int ProxyNameList_matches( s, name)
	char * s ;
	char * name ;
#endif /* _NO_PROTO */
{
	return (strcmp( s, name) == 0);
}

#ifndef   _NO_PROTO
static ProxyNameList * ProxyNameList_add( char * s, ProxyNameList * l)
#else  /* _NO_PROTO */
static ProxyNameList * ProxyNameList_add( s, l)
	char          * s ;
	ProxyNameList * l ;
#endif /* _NO_PROTO */
{
	ProxyNameList * lt;
	if (!l)
		return newProxyNameList( s);
	for (lt = l; lt; lt = lt->next) {
		if (ProxyNameList_matches(l->name, s))
			return l; /* already here */
	}
	lt = newProxyNameList( s);
	lt->next = l;

	return lt;
}

#ifndef   _NO_PROTO
static ProxyNameList * ProxyNameList_remove( char * s, ProxyNameList * l)
#else  /* _NO_PROTO */
static ProxyNameList * ProxyNameList_remove( s, l)
	char          * s ;
	ProxyNameList * l ;
#endif /* _NO_PROTO */
{
	ProxyNameList * lt;
	ProxyNameList * prev = (ProxyNameList*)0;

	for (lt = l; lt; prev = lt, lt = lt->next) {
		if (!ProxyNameList_matches(l->name, s))
			continue;
		if (!prev)
			l = lt->next;
		else {
			prev->next = lt->next;
		}
		deleteProxyNameList( lt);
		break;
	}
	return l;
}

/* end of OK */

/* OK from here */

#ifndef   _NO_PROTO
static char * usu_nextwhite( char * s)
#else  /* _NO_PROTO */
static char * usu_nextwhite( s)
	char * s ;
#endif /* _NO_PROTO */
{
	if (!s)
		return (char*)0;

	while (*s && !isspace(*s))
		s++;

	return s;
}

#ifndef   _NO_PROTO
static char * usu_overwhite( char * s)
#else  /* _NO_PROTO */
static char * usu_overwhite( s)
	char * s ;
#endif /* _NO_PROTO */
{
	if (!s)
		return (char*)0;

	while (*s && isspace(*s))
		s++;

	return s;
}

#ifndef   _NO_PROTO
static int usu_isendofline( char * s)
#else  /* _NO_PROTO */
static int usu_isendofline( s)
	char * s ;
#endif /* _NO_PROTO */
{
	return  (*s == '\r' || *s == '\n');
}

#ifndef   _NO_PROTO
static char * usu_endofline( char * s)
#else  /* _NO_PROTO */
static char * usu_endofline( s)
	char * s ;
#endif /* _NO_PROTO */
{
	if (!s)
		return (char*)0;
	for (; *s && !usu_isendofline(s); s++);

	return s;
}

#ifndef   _NO_PROTO
static char * usu_overnullreplacement( char * s)
#else  /* _NO_PROTO */
static char * usu_overnullreplacement( s)
	char * s ;
#endif /* _NO_PROTO */
{
	if (*s == '\r')
		*s++ = '\0';
	if (*s == '\n')
		*s++ = '\0';
	return s;
}

#ifndef   _NO_PROTO
static char * usu_skipWord( char * p)
#else  /* _NO_PROTO */
static char * usu_skipWord( p)
	char * p ;
#endif /* _NO_PROTO */
{
	p = usu_nextwhite( p);
	if (!p)
		return (char*)0;

	p = usu_overwhite( p);
	if (!p)
		return (char*)0;
	return p;
}

#ifndef   _NO_PROTO
static void HttpHeader_debugHeader( HttpHeader * hh)
#else  /* _NO_PROTO */
static void HttpHeader_debugHeader( hh)
	HttpHeader * hh ;
#endif /* _NO_PROTO */
{
	if (hh->server)
		(void) fprintf ( stderr, "Server:%s<p>\n", hh->server);
	if (hh->mime_version)
		(void) fprintf ( stderr, "Version:%s<p>\n", hh->mime_version);
	if (hh->content_type)
		(void) fprintf ( stderr, "Type:%s<p>\n", hh->content_type);
	if (hh->content_length)
		(void) fprintf ( stderr, "Length:%d<p>\n", hh->content_length);
}

#ifndef   _NO_PROTO
static char * HttpHeader_getValue( char * q)
#else  /* _NO_PROTO */
static char * HttpHeader_getValue( q)
	char * q ;
#endif /* _NO_PROTO */
{ /* a:b split */
	char * p;

	if ((p = STRCHR( q, ':')) == (char*)0)
		return (char*)0;
	q = p + 1;
	if ((q = usu_overwhite( q)) == (char*)0)
		return (char*)0;

	*p = '\0';
	return q;
}

#ifndef   _NO_PROTO
static char * HttpHeader_processHeaderLine( HttpHeader * hh, char * q)
#else  /* _NO_PROTO */
static char * HttpHeader_processHeaderLine( hh, q)
	HttpHeader * hh ;
	char       * q ;
#endif /* _NO_PROTO */
{
	char * p = HttpHeader_getValue( q);
	char * v;

	if (!p)
		return usu_endofline( q);

	for ( v = q; v && *v; v++)
		*v = tolower(*v);

	if ((!hh->server) && (strcmp( q, "server") == 0))
		hh->server = p;
	else if ((!hh->mime_version) && (strcmp( q, "mime-version") == 0))
		hh->mime_version = p;
	else if ((!hh->content_type) && (strcmp( q, "content-type") == 0))
		hh->content_type = p;
	else if ((hh->content_length <= 0) && (strcmp( q, "content-length") == 0))
		hh->content_length = atoi(p);

	return usu_endofline( p);
}

#ifndef   _NO_PROTO
static int HttpHeader_parse( HttpHeader * hh)
#else  /* _NO_PROTO */
static int HttpHeader_parse( hh)
	HttpHeader * hh ;
#endif /* _NO_PROTO */
{
	char * p;
	char * q;
	int    len;

	if ((len = (*hh->istream->read)( hh->istream, hh->header, CONN_SIZE)) == -1)
		return -1;

	for ( p = hh->header; p && *p;  p++) {
		if (*p != 'H')
			continue;
		if ( strncmp( p, "HTTP/", 5) == 0)
			break;
	}
	if (!p || !*p)
		return -2;

	p += 5;
	if ((p = usu_skipWord( p)) == (char*)0)
		return -3;

	q = p;
	if ((p = usu_nextwhite( p)) == (char*)0)
		return -4;

	/* status = atoi( q); */

	if ((p = usu_overwhite( p)) == (char*)0)
		return -5;

	/* reply = p; */

	if ((q = usu_endofline( p)) == (char*)0)
		return -6;

	q = usu_overnullreplacement( q);

	while (q && *q && !usu_isendofline(q)) { /* the header ends with a blank line */
		q = HttpHeader_processHeaderLine( hh, q);
		if (!q)
			return -7;
		q = usu_overnullreplacement( q);
	}
	if (q) {
		q = usu_overnullreplacement( q);

		if (q && *q) {
			len = len - (q - hh->header);
			(*hh->istream->setCache)( hh->istream, q, len);
		}
	}
	return 1;
}

#ifndef   _NO_PROTO
static InputStream * HttpHeader_getInputStream( HttpHeader * hh)
#else  /* _NO_PROTO */
static InputStream * HttpHeader_getInputStream( hh)
	HttpHeader * hh ;
#endif /* _NO_PROTO */
{
	return (InputStream*)(hh->istream);
}

#ifndef   _NO_PROTO
static char * HttpHeader_getContentType( HttpHeader * hh)
#else  /* _NO_PROTO */
static char * HttpHeader_getContentType( hh)
	HttpHeader * hh ;
#endif /* _NO_PROTO */
{
	return hh->content_type;
}

#ifndef   _NO_PROTO
static int HttpHeader_getContentLength( HttpHeader * hh)
#else  /* _NO_PROTO */
static int HttpHeader_getContentLength( hh)
	HttpHeader * hh ;
#endif /* _NO_PROTO */
{
	return hh->content_length;
}


/* end of OK */

/* public */

#ifndef   _NO_PROTO
static URLConnection* URL_openConnection( URL * u)
#else  /* _NO_PROTO */
static URLConnection* URL_openConnection( u)
	URL * u ;
#endif /* _NO_PROTO */
{
	extern HttpURLConnection * newHttpURLConnection();
	extern FileURLConnection * newFileURLConnection();

	if (u->connection)
		return u->connection;

	switch (u->type) {
		case HTTP_PROTOCOL:
			u->connection = (URLConnection*)newHttpURLConnection( u->urlstring);
			break;
		case FILE_PROTOCOL:
			u->connection = (URLConnection*)newFileURLConnection( u->urlstring);
			break;
		default:
			break;
	}
	if (!u->connection) {
		printf("no connection\n");
		u->bad = 1;
		return (URLConnection*)0;
	}
	if (!(*u->connection->isOk)(u->connection)) {
		printf("bad connection\n");
		(*u->connection->delete)( u->connection);
		u->connection = (URLConnection*)0;
		u->bad = 1;
		return (URLConnection*)0;
	}

	return u->connection;
}

#ifndef   _NO_PROTO
static URLConnection* URL_getConnection( URL * u)
#else  /* _NO_PROTO */
static URLConnection* URL_getConnection( u)
	URL * u ;
#endif /* _NO_PROTO */
{
	return u->connection;
}

#ifndef   _NO_PROTO
static int URL_checkConnection(URL *u);
#else  /* _NO_PROTO */
static int URL_checkConnection();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
static InputStream * URL_openStream( URL * u)
#else  /* _NO_PROTO */
static InputStream * URL_openStream( u)
	URL * u ;
#endif /* _NO_PROTO */
{
	if (!URL_checkConnection( u)) {
		printf("no connection\n");
		return (InputStream*)0;
	}
	return (*u->connection->getInputStream)(u->connection);
}

#ifndef   _NO_PROTO
static InputData * URL_getContent( URL * u)
#else  /* _NO_PROTO */
static InputData * URL_getContent( u)
	URL * u ;
#endif /* _NO_PROTO */
{
	if (!URL_checkConnection( u))
		return (InputData*)0;
	return (*u->connection->getContent)(u->connection);
}

/* private */

#ifndef   _NO_PROTO
static int URL_checkConnection( URL * u)
#else  /* _NO_PROTO */
static int URL_checkConnection( u)
	URL * u ;
#endif /* _NO_PROTO */
{
	if (u->bad)
		return 0;
	if (!u->connection)
		u->connection = (*u->openConnection)(u);
	if (u->bad)
		return 0;
	return 1;
}

#ifndef   _NO_PROTO
static int URL_parseUrl( URL * u, char * url, char * urldata)
#else  /* _NO_PROTO */
static int URL_parseUrl( u, url, urldata)
	URL * u ;
	char * url ;
	char * urldata ;
#endif /* _NO_PROTO */
{
	int    i = 0;
	char * s;
	char * p;
	if (!url)
		return 0;
	for ( s = url; *s && (*s != ':'); s++, i++);
	if (!s || !*s)
		return 0;

	p = (char*) malloc ((unsigned)(i+1)*sizeof(char));
	for ( i = 0, s = url; *s && (*s != ':'); s++, i++)
		p[i] = tolower(*s);

	p[i] = '\0';

	if (strcmp( p, "file") == 0)
		u->type = FILE_PROTOCOL;
	else
	if (strcmp( p, "http") == 0)
		u->type = HTTP_PROTOCOL;
	/* add more protocols here. this could of course be dynamic and table driven */

	(void) free ((void*) p);

	if (!u->type)
		return 0;

	s++;  /* skip the : */
	if (urldata)
		i = strlen(urldata) + 2;
	else
		i = 0;

	u->urlstring = (char*) malloc ((unsigned) (strlen(s)+i + 1) * sizeof(char));
	(void) strcpy( u->urlstring, s);
	if (i) {
		strcat(u->urlstring,"?");
		strcat(u->urlstring,urldata);
	}

	return 1;
}

/* URLConnection is abstract */

#ifndef   _NO_PROTO
static int URLConnection_isOk( URLConnection * uc)
#else  /* _NO_PROTO */
static int URLConnection_isOk( uc)
	URLConnection * uc ;
#endif /* _NO_PROTO */
{
	return (uc->rawdata->isconnected != -1);
}

#ifndef   _NO_PROTO
static void URLConnection_setDoOutput( URLConnection * uc, int i)
#else  /* _NO_PROTO */
static void URLConnection_setDoOutput( uc, i)
	URLConnection * uc ;
	int             i ;
#endif /* _NO_PROTO */
{
	uc->rawdata->doOutput = i;
}

#ifndef   _NO_PROTO
static void URLConnection_setDoInput( URLConnection * uc, int i)
#else  /* _NO_PROTO */
static void URLConnection_setDoInput( uc, i)
	URLConnection * uc ;
	int             i ;
#endif /* _NO_PROTO */
{
	uc->rawdata->doInput  = i;
}

#ifndef   _NO_PROTO
static void URLConnection_setAllowUserInteraction( URLConnection * uc, int i)
#else  /* _NO_PROTO */
static void URLConnection_setAllowUserInteraction( uc, i)
	URLConnection * uc ;
	int             i ;
#endif /* _NO_PROTO */
{
	uc->rawdata->allowUserInteraction = i;
}

#ifndef   _NO_PROTO
static void URLConnection_setBad( URLConnection * uc)
#else  /* _NO_PROTO */
static void URLConnection_setBad( uc)
	URLConnection * uc ;
#endif /* _NO_PROTO */
{
	uc->rawdata->isconnected = -1;
}

#ifndef   _NO_PROTO
static void URLConnection_connect( URLConnection * uc)
#else  /* _NO_PROTO */
static void URLConnection_connect( uc)
	URLConnection * uc ;
#endif /* _NO_PROTO */
{
	if (!(*uc->parseURL)(uc, uc->rawdata->urlstring)) {
		URLConnection_setBad( uc);
		return;
	}
	if ((*uc->open)( uc) == -1)
		URLConnection_setBad( uc);
	return;
}

#ifndef   _NO_PROTO
static void URLConnection_setRequestProperty( URLConnection * uc, char * key, char * value)
#else  /* _NO_PROTO */
static void URLConnection_setRequestProperty( uc, key, value)
	URLConnection * uc ;
	char          * key ;
	char          * value ;
#endif /* _NO_PROTO */
{
	/* does nothing */ 
}

#ifndef   _NO_PROTO
static void HttpURLConnection_setRequestProperty( HttpURLConnection * huc, char * key, char * value)
#else  /* _NO_PROTO */
static void HttpURLConnection_setRequestProperty( huc, key, value)
	HttpURLConnection * huc ;
	char              * key ;
	char              * value ;
#endif /* _NO_PROTO */
{
	char * lkey = strdup(key);
	char * s;
	PropertyPair * pp;

	s = lkey;
	*s = toupper(*s);

	for (s++; s && *s; s++) {
		if (*(s-1) == '-')
			*s = toupper(*s);
		else
			*s = tolower(*s);
	}

	for ( pp = huc->properties; pp; pp = pp->next) {
		if (strcmp(lkey, pp->key) == 0) {
			(void) free ((void*)lkey);
			if (pp->value)
				(void) free ((void*)pp->value);
			pp->value = strdup(value);
			return;
		}
	}
	pp = (PropertyPair*) malloc ((unsigned) sizeof(PropertyPair));
	pp->next   = huc->properties;
	pp->key    = lkey;
	pp->value  = strdup(value);
	huc->properties = pp;
}

#ifndef   _NO_PROTO
static void HttpURLConnection_startPairEnumeration( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static void HttpURLConnection_startPairEnumeration( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	huc->current_property = huc->properties;
}

#ifndef   _NO_PROTO
static int HttpURLConnection_getUserPair( HttpURLConnection * huc, char * buf)
#else  /* _NO_PROTO */
static int HttpURLConnection_getUserPair( huc, buf)
	HttpURLConnection * huc ;
	char              * buf ;
#endif /* _NO_PROTO */
{
	if (!huc->current_property)
		return 0;
	(void) sprintf( buf, "%s: %s", huc->current_property->key,
				       huc->current_property->value);
	huc->current_property = huc->current_property->next;
	return 1;
}

#ifndef   _NO_PROTO
static char * URLConnection_getContentType( URLConnection * uc)
#else  /* _NO_PROTO */
static char * URLConnection_getContentType( uc)
	URLConnection * uc ;
#endif /* _NO_PROTO */
{
	return "text/plain";
}

#ifndef   _NO_PROTO
static int URLConnection_getContentLength( URLConnection * uc)
#else  /* _NO_PROTO */
static int URLConnection_getContentLength( uc)
	URLConnection * uc ;
#endif /* _NO_PROTO */
{
	return -1;
}

#ifndef   _NO_PROTO
static InputData * URLConnection_getContent( URLConnection * uc)
#else  /* _NO_PROTO */
static InputData * URLConnection_getContent( uc)
	URLConnection * uc ;
#endif /* _NO_PROTO */
{
	return (InputData*)0;
}

#ifndef   _NO_PROTO
static OutputStream * FileURLConnection_getOutputStream (FileURLConnection * furlc)
#else  /* _NO_PROTO */
static OutputStream * FileURLConnection_getOutputStream (furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	return (OutputStream*)0;
}

#ifndef   _NO_PROTO
static InputStream * FileURLConnection_getInputStream (FileURLConnection * furlc)
#else  /* _NO_PROTO */
static InputStream * FileURLConnection_getInputStream (furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	return (InputStream*)0;
}

#ifndef   _NO_PROTO
static int FileURLConnection_open( FileURLConnection * furlc)
#else  /* _NO_PROTO */
static int FileURLConnection_open( furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	return 0;
}

#ifndef   _NO_PROTO
static int FileURLConnection_close( FileURLConnection * furlc)
#else  /* _NO_PROTO */
static int FileURLConnection_close( furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	return 0;
}

#ifndef   _NO_PROTO
static int FileURLConnection_parseURL( FileURLConnection * furlc, char * url)
#else  /* _NO_PROTO */
static int FileURLConnection_parseURL( furlc, url)
	FileURLConnection * furlc ;
	char              * url ;
#endif /* _NO_PROTO */

	{ return 0; /* FIXME */
}

#ifndef   _NO_PROTO
static char * FileURLConnection_getProtocol( FileURLConnection * furlc)
#else  /* _NO_PROTO */
static char * FileURLConnection_getProtocol( furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	return "file";
}

#ifndef   _NO_PROTO
static int FileURLConnection_getOutputSizeNeeded( FileURLConnection * furlc)
#else  /* _NO_PROTO */
static int FileURLConnection_getOutputSizeNeeded( furlc)
	FileURLConnection * furlc ;
#endif /* _NO_PROTO */
{
	return 0;
}


/* SocketURLConnection is also abstract */

#ifndef   _NO_PROTO
static int SocketURLConnection_socketIsOk( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static int SocketURLConnection_socketIsOk( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	URLSocket * sock = suc->rawsocketdata->sock;

	if (!sock)
		return 0;
	if (!(*sock->isConnected)( sock))
		return 0;
	return 1;
}

#ifndef   _NO_PROTO
static void SocketURLConnection_closeSocket( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static void SocketURLConnection_closeSocket( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	URLSocket * sock = suc->rawsocketdata->sock;

	if (sock) {
		(*sock->delete)( sock);
		suc->rawsocketdata->sock = (URLSocket*)0;
	}
}

#ifndef   _NO_PROTO
static void SocketURLConnection_setHostNeedsProxy( SocketURLConnection * suc, char * host, int tf)
#else  /* _NO_PROTO */
static void SocketURLConnection_setHostNeedsProxy( suc, host, tf)
	SocketURLConnection * suc ;
	char                * host ;
	int                   tf ;
#endif /* _NO_PROTO */
{
	ProxyNameList ** plp = suc->rawsocketdata->ignore_list_p;

	if (!tf)
		*plp = ProxyNameList_add( host, *plp);
	else
		*plp = ProxyNameList_remove( host, *plp);
}

#ifndef   _NO_PROTO
static ProxyNameList * SocketURLConnection_getProxyStopList( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static ProxyNameList * SocketURLConnection_getProxyStopList( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	ProxyNameList ** plp = suc->rawsocketdata->ignore_list_p;

	return *plp;
}

#ifndef   _NO_PROTO
static int SocketURLConnection_getProxyPort( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static int SocketURLConnection_getProxyPort( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	return *(suc->rawsocketdata->proxyport_p);
}

#ifndef   _NO_PROTO
static char * SocketURLConnection_getProxyHost( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static char * SocketURLConnection_getProxyHost( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	return *(suc->rawsocketdata->proxyhost_p);
}

#ifndef   _NO_PROTO
static void SocketURLConnection_setProxyPort( SocketURLConnection * suc, int i)
#else  /* _NO_PROTO */
static void SocketURLConnection_setProxyPort( suc, i)
	SocketURLConnection * suc ;
	int                   i ;
#endif /* _NO_PROTO */
{
	*(suc->rawsocketdata->proxyport_p) = i;
}

#ifndef   _NO_PROTO
static void SocketURLConnection_setNoProxy( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static void SocketURLConnection_setNoProxy( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	char ** php = (suc->rawsocketdata->proxyhost_p);
	if (*php)
		(void) free ((void*)*php);
	*php = (char*)0;
	SocketURLConnection_setProxyPort( suc, -1);
}

#ifndef   _NO_PROTO
static void SocketURLConnection_setProxyHost( SocketURLConnection * suc, char * s)
#else  /* _NO_PROTO */
static void SocketURLConnection_setProxyHost( suc, s)
	SocketURLConnection * suc ;
	char                * s ;
#endif /* _NO_PROTO */
{
	char ** php;

	if (!s) {
		SocketURLConnection_setNoProxy( suc);
		return;
	}

	php = (suc->rawsocketdata->proxyhost_p);

	if (*php)
		(void) free ((void*)*php);
	if (s)
		*php = strdup( s);
	else
		*php = (char*)0;
}

#ifndef   _NO_PROTO
static int SocketURLConnection_isInList( char * el, ProxyNameList * l)
#else  /* _NO_PROTO */
static int SocketURLConnection_isInList( el, l)
	char          * el ;
	ProxyNameList * l ;
#endif /* _NO_PROTO */
{
	if (!l)
		return 0;
	if (strcmp( el, l->name) == 0)
		return 1;
	return SocketURLConnection_isInList( el, l->next);
}

#ifndef   _NO_PROTO
static int SocketURLConnection_checkProxy( SocketURLConnection * suc) ;
#else  /* _NO_PROTO */
static int SocketURLConnection_checkProxy() ;
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
static int SocketURLConnection_hasProxy( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static int SocketURLConnection_hasProxy( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	if (SocketURLConnection_getProxyPort( suc) == -1)
		return 0;
	if (!SocketURLConnection_getProxyHost( suc))
		return SocketURLConnection_checkProxy( suc);
	return 1;
}

#ifndef   _NO_PROTO
static int SocketURLConnection_needsProxy( SocketURLConnection * suc, char * host)
#else  /* _NO_PROTO */
static int SocketURLConnection_needsProxy( suc, host)
	SocketURLConnection * suc ;
	char                * host ;
#endif /* _NO_PROTO */
{
	ProxyNameList * pl;

	if (!SocketURLConnection_hasProxy( suc))
		return 0;
	pl = (*suc->getProxyStopList)( suc);
	if (!pl)
		return 1;
	if (SocketURLConnection_isInList( host, pl))
		return 0;
	return 1;
}

#ifndef   _NO_PROTO
static int SocketURLConnection_setProxy( SocketURLConnection * suc, char * s)
#else  /* _NO_PROTO */
static int SocketURLConnection_setProxy( suc, s)
	SocketURLConnection * suc ;
	char                * s ;
#endif /* _NO_PROTO */
{
	char * q;

	if ((q = STRCHR( s, ':')) == (char*)0)
		return 0;
	*q = '\0';

	SocketURLConnection_setProxyHost( suc, s);
	SocketURLConnection_setProxyPort( suc, atoi(q+1));

	*q = ':';

	return 1;
}

#ifndef   _NO_PROTO
static void SocketURLConnection_newProxy( SocketURLConnection * suc, char * ph, int pp)
#else  /* _NO_PROTO */
static void SocketURLConnection_newProxy( suc, ph, pp)
	SocketURLConnection * suc ;
	char                * ph ;
	int                   pp ;
#endif /* _NO_PROTO */
{
	SocketURLConnection_setProxyPort( suc, pp);
	SocketURLConnection_setProxyHost( suc, ph);
}

#ifndef   _NO_PROTO
static void SocketURLConnection_setConnected( SocketURLConnection* suc)
#else  /* _NO_PROTO */
static void SocketURLConnection_setConnected( suc)
	SocketURLConnection* suc ;
#endif /* _NO_PROTO */
{
	suc->rawdata->isconnected = 1;
}

#ifndef   _NO_PROTO
static int SocketURLConnection_checkProxy( SocketURLConnection * suc)
#else  /* _NO_PROTO */
static int SocketURLConnection_checkProxy( suc)
	SocketURLConnection * suc ;
#endif /* _NO_PROTO */
{
	char * ev = (*suc->getProxyEnvVar)( suc);
	if (!ev)
		return 0;
	ev = getenv( ev);
	if (ev) {
		if (SocketURLConnection_setProxy( suc, ev))
			return 1;
	}
	SocketURLConnection_setProxyPort( suc, -1);
	return 0;
}

#ifndef   _NO_PROTO
static char * HttpURLConnection_getPostType( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static char * HttpURLConnection_getPostType( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	char * pt = huc->rawhttpdata->post_type;
	if (pt)
		return pt;
	return "text/plain";
}

#ifndef   _NO_PROTO
static int HttpURLConnection_writeEndOfHeader( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static int HttpURLConnection_writeEndOfHeader( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	URLSocket * sock = huc->rawsocketdata->sock;

	if ((*sock->write)( sock, EXTRALINE) == -1)
		return -1;
	/* uc->connected = 1; */
	return 0;
}

#ifndef   _NO_PROTO
static int HttpURLConnection_writeContentHeader( HttpURLConnection * huc, char * type)
#else  /* _NO_PROTO */
static int HttpURLConnection_writeContentHeader( huc, type)
	HttpURLConnection * huc ;
	char              * type ;
#endif /* _NO_PROTO */
{
	PathData p;
	char *   ptr;
	URLSocket * sock = huc->rawsocketdata->sock;
	/* char * s; */

	ptr = (char*)p;

	(void) sprintf( ptr, "%s ", type);
	ptr += strlen(ptr);

	if ( huc->rawsocketdata->url_usesproxy) {
		(void) sprintf( ptr, "%s://%s:%d",
					(*huc->getProtocol)( huc),
					huc->rawsocketdata->serverhost,
					huc->rawsocketdata->serverport);
		ptr += strlen(ptr);
	}

	(void) sprintf( ptr, "/%s %s", huc->rawhttpdata->resource, GETTRAILER);

	if ((*sock->write)( sock, p) == -1)
		return -1;

#if 0
	s = HttpURLConnection_getPostType(huc);
	if (s)
		HttpURLConnection_setRequestProperty( huc, "Content-Type", s);
#endif

	HttpURLConnection_startPairEnumeration( huc);
	while (HttpURLConnection_getUserPair( huc, p)) {
		if ((*sock->write)( sock, (char*)p) == -1)
			return -1;
		if ((*sock->write)( sock, "\r\n") == -1)
			return -1;
	}

	return 0;
}

#ifndef   _NO_PROTO
static int HttpURLConnection_writeContentLength( HttpURLConnection * huc, int length)
#else  /* _NO_PROTO */
static int HttpURLConnection_writeContentLength( huc, length)
	HttpURLConnection * huc ;
	int                 length ;
#endif /* _NO_PROTO */
{
	URLSocket * sock = huc->rawsocketdata->sock;
	PathData p;

	(void) sprintf ( p,"Content-Length: %d \r\n", length);
	if ((*sock->write)( sock, (char*)p) == -1)
		return -1;

	return 0;
}

#ifndef   _NO_PROTO
static InputStream * HttpURLConnection_readHeader( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static InputStream * HttpURLConnection_readHeader( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	URLSocket * sock = huc->rawsocketdata->sock;
	InputStream * i  = (*sock->getInputStream)(sock);
	int rval;

	HttpHeader * header;

	if (!i) {
		return (InputStream*)0;
	}

	header = newHttpHeader( i);


	rval = (*header->parse)(header);
	if (rval < 0) {
		(void) fprintf ( stderr, "Header Parse fail #%d\n", rval);
		(*i->delete)(i);
		(*header->delete)(header);
		return (InputStream *)0;
	}
	huc->rawhttpdata->header = header;

	/* header->debugHeader(); */
	return (*header->getInputStream)(header);
}

#ifndef   _NO_PROTO
static int HttpURLConnection_getOutputLength( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static int HttpURLConnection_getOutputLength( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	return 0;
}

#ifndef   _NO_PROTO
static int HttpURLConnection_parseURL( HttpURLConnection * huc, char * s)
#else  /* _NO_PROTO */
static int HttpURLConnection_parseURL( huc, s)
	HttpURLConnection * huc ;
	char              * s ;
#endif /* _NO_PROTO */
{
	char * p = (char*)0;
	char * q;

	if (*s++ != '/')
		return 0;
	if (*s++ != '/')
		return 0;
	for ( q = s; q && *q && (*q != '/'); q++)
		if (*q == ':')
			p = q;

	if (p && *p)
		*p++ = '\0';
	if (q && *q)
		*q++ = '\0';

	huc->rawsocketdata->serverhost = s;
	if (p)
		huc->rawsocketdata->serverport = atoi(p);
	else
		huc->rawsocketdata->serverport = 80;

	if (q)
		huc->rawhttpdata->resource = q;
	else
		huc->rawhttpdata->resource = "";

	return 1;
}

#ifndef   _NO_PROTO
static void HttpURLConnection_setPostContentType( HttpURLConnection * huc, char * s)
#else  /* _NO_PROTO */
static void HttpURLConnection_setPostContentType( huc, s)
	HttpURLConnection * huc ;
	char              * s ;
#endif /* _NO_PROTO */
{
	char * p = huc->rawhttpdata->post_type;

	if (p)
		(void) free ((void*) p);

	huc->rawhttpdata->post_type = strdup(s);
}

#ifndef   _NO_PROTO
static char * HttpURLConnection_getProtocol( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static char * HttpURLConnection_getProtocol( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	return "http";
}

#ifndef   _NO_PROTO
static char * HttpURLConnection_getProxyEnvVar( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static char * HttpURLConnection_getProxyEnvVar( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	return "HTTP_PROXY";
}

#ifndef   _NO_PROTO
static int HttpURLConnection_getOutputSizeNeeded( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static int HttpURLConnection_getOutputSizeNeeded( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	return 1;
}

#ifndef   _NO_PROTO
static int HttpURLConnection_open( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static int HttpURLConnection_open( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	extern URLSocket * newURLSocket();

	URLSocket * sock;
	char * host = (char*)0;
	int    port;
	char * serverhost = huc->rawsocketdata->serverhost;
	int    serverport = huc->rawsocketdata->serverport;

	if (SocketURLConnection_needsProxy( (SocketURLConnection*)huc, serverhost)) {
		host = SocketURLConnection_getProxyHost( (SocketURLConnection*)huc);
		port = SocketURLConnection_getProxyPort( (SocketURLConnection*)huc);
		huc->rawsocketdata->url_usesproxy = 1;
	} else {
		host = serverhost;
		port = serverport;
		huc->rawsocketdata->url_usesproxy = 0;
	}
	sock = newURLSocket( host, port);

	if (!sock || !(*sock->isConnected)(sock)) {
		if (sock)
			(*sock->delete)(sock);
		return -1;
	}
	huc->rawsocketdata->sock = sock;
	huc->rawhttpdata->data   = (void*)0;

	SocketURLConnection_setConnected( (SocketURLConnection*)huc);

	return 0;
}

#ifndef   _NO_PROTO
static int HttpURLConnection_close( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static int HttpURLConnection_close( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	HttpHeader * h = huc->rawhttpdata->header;

	if (h) {
		(*h->delete)( h);
		huc->rawhttpdata->header = (HttpHeader*)0;
	}

	SocketURLConnection_closeSocket( (SocketURLConnection*)huc);
	return 0;
}

#ifndef   _NO_PROTO
static OutputStream * HttpURLConnection_getOutputStream( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static OutputStream * HttpURLConnection_getOutputStream( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	extern TempFileOutputStream * newTempFileOutputStream();
	int    doOutput = huc->rawdata->doOutput;
	TempFileOutputStream * tmpStream;

	if (!doOutput)
		return (OutputStream*)0;

	tmpStream = newTempFileOutputStream();
	huc->rawhttpdata->tmpStream = tmpStream;

	return (OutputStream*)tmpStream;
}

#ifndef   _NO_PROTO
static InputStream  * HttpURLConnection_getInputStream( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static InputStream  * HttpURLConnection_getInputStream( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	int    doOutput = huc->rawdata->doOutput;
	char * method   = (doOutput ? POSTHEADER : GETHEADER);
	int    length   = 0;

	if (doOutput) {
		TempFileOutputStream * tmpStream = huc->rawhttpdata->tmpStream;
		if (!tmpStream) {
			return (InputStream*)0;
		}
		length = (*tmpStream->getContentLength)( tmpStream);
		if (!length) {
			return (InputStream*)0;
		}
	}

	if (!huc->rawdata->isconnected)
		URLConnection_connect( (URLConnection*)huc);

	if (!(*huc->isOk)( huc)) {
		return (InputStream*)0;
	}
	if (HttpURLConnection_writeContentHeader( huc, method) == -1)
		return (InputStream*)0;

	if (doOutput) {
		if (HttpURLConnection_writeContentLength( huc, length) == -1)
			return (InputStream*)0;
	}

	if (HttpURLConnection_writeEndOfHeader( huc) == -1)
		return (InputStream*)0;

	if (doOutput) {
		URLSocket    * sock = huc->rawsocketdata->sock;
		OutputStream * o    = (*sock->getOutputStream)(sock);
		TempFileOutputStream * tmpStream = huc->rawhttpdata->tmpStream;

		(*tmpStream->copy)( tmpStream, o);

		(*tmpStream->delete)( tmpStream);
		huc->rawhttpdata->tmpStream = (TempFileOutputStream*)0;
	}

	return HttpURLConnection_readHeader( huc);
}

#ifndef   _NO_PROTO
static char * HttpURLConnection_getContentType( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static char * HttpURLConnection_getContentType( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	HttpHeader * h = huc->rawhttpdata->header;

	if (!h)
		return (char*)0;

	return (*h->getContentType)( h);
}

#ifndef   _NO_PROTO
static int HttpURLConnection_getContentLength( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static int HttpURLConnection_getContentLength( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	HttpHeader * h = huc->rawhttpdata->header;

	if (!h)
		return 0;
	return (*h->getContentLength)( h);
}

/* add to C++ too */

#ifndef   _NO_PROTO
static InputData * HttpURLConnection_getContent( HttpURLConnection * huc)
#else  /* _NO_PROTO */
static InputData * HttpURLConnection_getContent( huc)
	HttpURLConnection * huc ;
#endif /* _NO_PROTO */
{
	extern InputData * newInputData();

	return newInputData( (*huc->getInputStream)( huc));
}

#if      0
#ifndef   _NO_PROTO
void postData(void)
#else  /* _NO_PROTO */
void postData()
#endif /* _NO_PROTO */
{
	if (!tmpStream)
		return;
}
#endif /* 0 */


#define MYTESTURL       "http://iswarm/index.html"

#ifndef   _NO_PROTO
void testURL(void)
#else  /* _NO_PROTO */
void testURL()
#endif /* _NO_PROTO */
{
	extern DataInputStream * newDataInputStream();

	URL * u = newURL( MYTESTURL);
	InputStream * i;
	DataInputStream * ds;
	char * s;

	if (!u) {
		printf("NO URL\n");
		return;
	}
	i = (*u->openStream)( u);

	if (!i) {
		printf("NO input stream\n");
		return;
	}

	ds = newDataInputStream( i);

	while ((s = (*ds->readLine)(ds)) != (char*)0)
		printf("=> %s\n", s);

	(*ds->delete)(ds);
	(*u->delete)( u);
}

#ifdef     TESTURL
/* ARGSUSED */
void main(int ac, char **av)
{
#ifdef    WIN32
	WSADATA wsadata;

	if (WSAStartup( 0x202, &wsadata) == SOCKET_ERROR)
			exit(-1);
#endif /* WIN32 */

	testURL();

#ifdef    WIN32
	WSACleanup();
#endif /* WIN32 */
}
#endif /* TESTURL */
