#ifndef    _URL_H
#define    _URL_H

#include <sys/types.h>
#include <ctype.h>

#ifndef   WIN32
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#endif /* WIN32 */

#include "Ustreams.h"

#define  HTTP_PROTOCOL            1
#define  FILE_PROTOCOL            2
#define  E_CreateSocket          -1
#define  E_NoRouteToHost         -2
#define  E_CannotConnectToHost   -3

#define  MAXPATHDATA           1024
#define  CONN_SIZE              512

#define HEADHEADER            "HEAD"
#define PUTHEADER              "PUT"
#define POSTHEADER            "POST"

#define GETHEADER              "GET"
#define GETTRAILER    "HTTP/1.0\r\n"
#define EXTRALINE             "\r\n"

typedef char    PathData[MAXPATHDATA+1];

typedef struct URLSocket_s 
{
	/* private */
		SocketHandle         sock;
		SocketInputStream  * sock_stdin;
		SocketOutputStream * sock_stdout;

	/* public */
		int            urlErrno;
		int_f          write;
		int_f          isConnected;
		InputStream_f  getInputStream;
		OutputStream_f getOutputStream;
		void_f delete;
} URLSocket;

typedef struct ProxyNameList_s 
{
	char                   * name;
	struct ProxyNameList_s * next;
} ProxyNameList;

typedef struct HttpHeader_s 
{
	/* private */
		CachedInputStream * istream;
		char              * server;
		char              * mime_version;
		char              * content_type;
		int                 content_length;
		char              * header;

	/* public */
		void_f              delete;
		int_f               parse;
		InputStream_f       getInputStream;
		string_f            getContentType;
		int_f               getContentLength;
} HttpHeader;

typedef struct URLConnectionData_s
{
	/* private */
		int    doOutput;
		int    doInput;
		int    allowUserInteraction;
		int    ok;
		int    isconnected;
		char * urlstring;

	/* public */
		void_f delete;
} URLConnectionData;

typedef struct URLConnection_s {
	/* private */
		URLConnectionData * rawdata;

	/* public */
		void_f              delete;

	/* methods used by URL */
		OutputStream_f      getOutputStream;
		InputStream_f       getInputStream;
		InputData_f         getContent;
		int_f               open;
		int_f               close;
		int_f               isOk;
		int_f               getOutputSizeNeeded;
		string_f            getProtocol;
		int_f               parseURL;

	/* used publicly elsewhere */
		void_f              setDoOutput;
		void_f              setDoInput;
		void_f              setAllowUserInteraction;
		string_f            getContentType;
		int_f               getContentLength;
		void_f              setRequestProperty;
} URLConnection;

typedef struct FileURLConnection_s {
	/* inherited from URLConnection */
		URLConnectionData * rawdata;
		void_f              delete;
		OutputStream_f      getOutputStream;
		InputStream_f       getInputStream;
		InputData_f         getContent;
		int_f               open;
		int_f               close;
		int_f               isOk;
		int_f               getOutputSizeNeeded;
		string_f            getProtocol;
		int_f               parseURL;
		void_f              setDoOutput;
		void_f              setDoInput;
		void_f              setAllowUserInteraction;
		string_f            getContentType;
		int_f               getContentLength;
		void_f              setRequestProperty;

	/* local methods */
} FileURLConnection;

typedef struct SocketURLConnectionData_s {
	/* private */
		URLSocket *   sock;
		char      *   serverhost;
		int           serverport;
		int           url_usesproxy;
		char **       proxyhost_p;
		int  *        proxyport_p;
		ProxyNameList ** ignore_list_p;
	/* public */
		void_f           delete;
} SocketURLConnectionData;

typedef ProxyNameList * (*ProxyNameList_f)();

typedef struct SocketURLConnection_s 
{
	/* inherited from URLConnection */
		URLConnectionData * rawdata;
		void_f              delete;
		OutputStream_f      getOutputStream;
		InputStream_f       getInputStream;
		InputData_f         getContent;
		int_f               open;
		int_f               close;
		int_f               isOk;
		int_f               getOutputSizeNeeded;
		string_f            getProtocol;
		int_f               parseURL;
		void_f              setDoOutput;
		void_f              setDoInput;
		void_f              setAllowUserInteraction;
		string_f            getContentType;
		int_f               getContentLength;
		void_f              setRequestProperty;

	/* local */

	/* private */
		SocketURLConnectionData * rawsocketdata;
		ProxyNameList_f           getProxyStopList;
		string_f                  getProxyEnvVar;
} SocketURLConnection;

typedef struct HttpURLConnectionData_s 
{
	/* private */
		void                 * data;
		TempFileOutputStream * tmpStream;
		char                 * resource;
		char                 * post_type;
		HttpHeader           * header;

	/* public */
		void_f                 delete;
} HttpURLConnectionData;

#define IST_AGENT       "IST Application URL Library/c v1.0"

typedef struct PropertyPair_s 
{
	char                  * key;
	char                  * value;
	struct PropertyPair_s * next;
} PropertyPair;

typedef struct HttpURLConnection_s 
{
	/* inherited from URLConnection */
		URLConnectionData * rawdata;
		void_f              delete;
		OutputStream_f      getOutputStream;
		InputStream_f       getInputStream;
		InputData_f         getContent;
		int_f               open;
		int_f               close;
		int_f               isOk;
		int_f               getOutputSizeNeeded;
		string_f            getProtocol;
		int_f               parseURL;
		void_f              setDoOutput;
		void_f              setDoInput;
		void_f              setAllowUserInteraction;
		string_f            getContentType;
		int_f               getContentLength;
		void_f              setRequestProperty;

	/* inherited from SocketURLConnection */
		SocketURLConnectionData * rawsocketdata;
		ProxyNameList_f           getProxyStopList;
		string_f                  getProxyEnvVar;

	/* local */

	/* private */
		HttpURLConnectionData * rawhttpdata;
		PropertyPair *          properties;
		PropertyPair *          current_property;
} HttpURLConnection;


typedef URLConnection * (*URLConnection_f)();

typedef struct URL_s 
{
	/* private */
		int             type;
		int             bad;
		char          * urlstring;
		URLConnection * connection;

	/* public */
		URLConnection_f openConnection;
		URLConnection_f getConnection;
		InputStream_f   openStream;
		InputData_f     getContent;
		void_f          delete;
} URL;

#endif /* _URL_H */
