
/*
**LIBS: -lnsl -lsocket -lgen
*/

#include <stdio.h>
#include "Ustreams.h"

/* This C code follows the same I/O API as the C++
** version. This is a subset of the Java(TM) IO (java.io) API.
** Anything you write using this API you should be able to rewrite in
** Java easily. The one extra public class/object is InputData, to allow
** you to read a data object into core in one command.
**
** The objects are ... (private objects are in parenthesis. These may have no Java
** equivalent)
**
** abstract InputStream
**      (StdInputStream)
**              FileInputStream
**      BufferedInputStream
**      DataInputStream
**
** InputData
**
** abstract OutputStream
**      (StdOutputStream)
**              FileOutputStream
**                      (TempFileOutputStream)
**      BufferedOutputStream
**
** PrintWriter
**
**
**
*/

/* Routines to pick up default implementations */

#ifndef   _NO_PROTO
static int      _inheritRead(InputStream *, char *, int);
static int      _inheritReadChar(InputStream *);
static int      _inheritAvailable(InputStream *);
static int      _inheritSkip(InputStream *, int);
static void     _inheritReadClose(InputStream *);
static void     _inheritWriteClose(OutputStream *);
static int      _inheritWrite(OutputStream *, char *, int);
static void     _inheritFlush(OutputStream *);
#else  /* _NO_PROTO */
static int      _inheritRead();
static int      _inheritReadChar();
static int      _inheritAvailable();
static int      _inheritSkip();
static void     _inheritReadClose();
static void     _inheritWriteClose();
static int      _inheritWrite();
static void     _inheritFlush();
#endif /* _NO_PROTO */

/* Constructors Destructors and declarations */

/* StdInputStream */

#ifndef   _NO_PROTO
static int      StdInputStream_readChar(StdInputStream *);
static int      StdInputStream_read(StdInputStream *, char *, int);
static int      StdInputStream_skip(StdInputStream *, int);
static int      StdInputStream_available(StdInputStream *);
static void     StdInputStream_close(StdInputStream *);
#else  /* _NO_PROTO */
static int      StdInputStream_readChar();
static int      StdInputStream_read();
static int      StdInputStream_skip();
static int      StdInputStream_available();
static void     StdInputStream_close();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteStdInputStream( StdInputStream * si)
#else  /* _NO_PROTO */
void deleteStdInputStream( si)
	StdInputStream * si ;
#endif /* _NO_PROTO */
{
	/* close the fid ? */
	(void) free ((void*) si);
}

#ifndef   _NO_PROTO
StdInputStream * newStdInputStream( int i)
#else  /* _NO_PROTO */
StdInputStream * newStdInputStream( i)
	int i ;
#endif /* _NO_PROTO */
{
	StdInputStream * si = (StdInputStream *) malloc ((unsigned) sizeof(StdInputStream));

	si->dontblock = 0;
	si->id        = i;
	si->ateof     = 0;

	si->readChar  = StdInputStream_readChar;
	si->read      = StdInputStream_read;
	si->available = StdInputStream_available;
	si->close     = StdInputStream_close;
	si->skip      = StdInputStream_skip;
	si->delete    = deleteStdInputStream;

	return si;
}

#ifndef   _NO_PROTO
StdInputStream * newStdInputStream_anon(void)
#else  /* _NO_PROTO */
StdInputStream * newStdInputStream_anon()
#endif /* _NO_PROTO */
{
	return newStdInputStream( -1);
}

/* FileInputStream */

#ifndef   _NO_PROTO
void deleteFileInputStream( FileInputStream * fid)
#else  /* _NO_PROTO */
void deleteFileInputStream( fid)
	FileInputStream * fid ;
#endif /* _NO_PROTO */
{
	if (fid->sid)
		(*fid->sid->delete)( fid->sid);

	(void) free ((void*) fid);
}

#ifndef   _NO_PROTO
FileInputStream * newFileInputStream( char * filename)
#else  /* _NO_PROTO */
FileInputStream * newFileInputStream( filename)
	char * filename ;
#endif /* _NO_PROTO */
{
	FileInputStream * fid;

	int id = open( filename, O_RDONLY, 0);
	if (id == -1)
		return (FileInputStream*)0;

	fid = (FileInputStream*) malloc ((unsigned) sizeof(FileInputStream));
	fid->sid = newStdInputStream( id);
	fid->parent = (InputStream*) fid->sid;

	fid->available = _inheritAvailable;
	fid->readChar  = _inheritReadChar;
	fid->read      = _inheritRead;
	fid->skip      = _inheritSkip;
	fid->close     = _inheritReadClose;
	fid->delete    = deleteFileInputStream;

	return fid;
}


/* CachedInputStream */

#ifndef   _NO_PROTO
static int  CachedInputStream_available(CachedInputStream *);
static int  CachedInputStream_readChar(CachedInputStream *);
static int  CachedInputStream_read(CachedInputStream *, char *, int);
static int  CachedInputStream_skip(CachedInputStream *, int);
static void CachedInputStream_close(CachedInputStream *);
static void CachedInputStream_setCache(CachedInputStream *, char *, int);
#else  /* _NO_PROTO */
static int  CachedInputStream_available();
static int  CachedInputStream_readChar();
static int  CachedInputStream_read();
static int  CachedInputStream_skip();
static void CachedInputStream_close();
static void CachedInputStream_setCache();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteCachedInputStream( CachedInputStream * ca)
#else  /* _NO_PROTO */
void deleteCachedInputStream( ca)
	CachedInputStream * ca ;
#endif /* _NO_PROTO */
{
	(void) free((void*) ca);
}

#ifndef   _NO_PROTO
CachedInputStream * newCachedInputStream( InputStream * i)
#else  /* _NO_PROTO */
CachedInputStream * newCachedInputStream( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	CachedInputStream * ca = (CachedInputStream *) malloc ((unsigned) sizeof(CachedInputStream));

	ca->istream   = i;
	ca->cachelen  = 0;
	ca->available = CachedInputStream_available;
	ca->readChar  = CachedInputStream_readChar;
	ca->read      = CachedInputStream_read;
	ca->skip      = CachedInputStream_skip;
	ca->close     = CachedInputStream_close;
	ca->setCache  = CachedInputStream_setCache;
	ca->delete    = deleteCachedInputStream;

	return ca;
}

/* BufferedInputStream */

#ifndef   DEFAULT_BUFSIZE
#define   DEFAULT_BUFSIZE    5012
#endif /* DEFAULT_BUFSIZE */

#ifndef   _NO_PROTO
static int  BufferedInputStream_available(BufferedInputStream *);
static int  BufferedInputStream_fillBuffer(BufferedInputStream *);
static int  BufferedInputStream_readChar(BufferedInputStream *);
static int  BufferedInputStream_read(BufferedInputStream *, char *, int);
static int  BufferedInputStream_skip(BufferedInputStream *, int);
static void BufferedInputStream_close(BufferedInputStream *);
#else  /* _NO_PROTO */
static int  BufferedInputStream_available();
static int  BufferedInputStream_fillBuffer();
static int  BufferedInputStream_readChar();
static int  BufferedInputStream_read();
static int  BufferedInputStream_skip();
static void BufferedInputStream_close();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteBufferedInputStream( BufferedInputStream * bs)
#else  /* _NO_PROTO */
void deleteBufferedInputStream( bs)
	BufferedInputStream * bs ;
#endif /* _NO_PROTO */
{
	if (bs->data)
		(void) free ((void*) bs->data);
	(void) free ((void*) bs);
}

#ifndef   _NO_PROTO
BufferedInputStream * newBufferedInputStreamSized( InputStream * i, int bufsize)
#else  /* _NO_PROTO */
BufferedInputStream * newBufferedInputStreamSized( i, bufsize)
	InputStream * i ;
	int           bufsize ;
#endif /* _NO_PROTO */
{
	BufferedInputStream * bs = (BufferedInputStream *) malloc ((unsigned) sizeof(BufferedInputStream));
	bs->inputstream = i;
	bs->bufsize     = bufsize;
	bs->buflen      = 0;
	bs->data        = (void*)0;

	bs->available   = BufferedInputStream_available;
	bs->readChar    = BufferedInputStream_readChar;
	bs->read        = BufferedInputStream_read;
	bs->skip        = BufferedInputStream_skip;
	bs->close       = BufferedInputStream_close;
	bs->load        = BufferedInputStream_fillBuffer;
	bs->delete      = deleteBufferedInputStream;

	return bs;
}

#ifndef   _NO_PROTO
BufferedInputStream * newBufferedInputStream( InputStream * i)
#else  /* _NO_PROTO */
BufferedInputStream * newBufferedInputStream( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	return newBufferedInputStreamSized( i, DEFAULT_BUFSIZE);
}

/* DataInputStream - only residual */


#ifndef   _NO_PROTO
static char * DataInputStream_readLine( DataInputStream * ds);
#else  /* _NO_PROTO */
static char * DataInputStream_readLine();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteDataInputStream( DataInputStream * ds)
#else  /* _NO_PROTO */
void deleteDataInputStream( ds)
	DataInputStream * ds ;
#endif /* _NO_PROTO */
{
	if (ds->bs)
		(*ds->bs->delete)( ds->bs);
	(void) free ( (void*) ds);
}

#ifndef   _NO_PROTO
DataInputStream * newDataInputStream( InputStream * i)
#else  /* _NO_PROTO */
DataInputStream * newDataInputStream( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	DataInputStream * ds;
	if (!i)
		return (DataInputStream*)0;

	ds  = (DataInputStream*) malloc ((unsigned) sizeof(DataInputStream));

	ds->bs       = newBufferedInputStream( i);
	ds->readLine = DataInputStream_readLine;
	ds->delete   = deleteDataInputStream;

	return ds;
}

/* InputData */

#ifndef   _NO_PROTO
static void * InputData_getData(InputData *);
static int    InputData_getSize(InputData *);
#else  /* _NO_PROTO */
static void * InputData_getData();
static int    InputData_getSize();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteInputData( InputData * id)
#else  /* _NO_PROTO */
void deleteInputData( id)
	InputData * id ;
#endif /* _NO_PROTO */
{
	if (id->data)
		(void) free ((void*) id->data);
	if (id->bs)
		(*id->bs->delete)( id->bs);

	(void) free ((void*) id);
}

#ifndef   _NO_PROTO
InputData * newInputData( InputStream* i)
#else  /* _NO_PROTO */
InputData * newInputData( i)
	InputStream* i ;
#endif /* _NO_PROTO */
{
	InputData * id = (InputData*) malloc ((unsigned) sizeof(InputData));

	id->bs       = newBufferedInputStream( i);
	id->getData  = InputData_getData;
	id->getSize  = InputData_getSize;
	id->loaded   = 0;
	id->delete   = deleteInputData;

	return id;
}


/* BufferedOutputStream */

#ifndef   _NO_PROTO
static int  BufferedOutputStream_write(BufferedOutputStream *, char *, int);
static void BufferedOutputStream_close(BufferedOutputStream *);
static void BufferedOutputStream_flush(BufferedOutputStream *);
#else  /* _NO_PROTO */
static int  BufferedOutputStream_write();
static void BufferedOutputStream_close();
static void BufferedOutputStream_flush();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteBufferedOutputStream( BufferedOutputStream * bs)
#else  /* _NO_PROTO */
void deleteBufferedOutputStream( bs)
	BufferedOutputStream * bs ;
#endif /* _NO_PROTO */
{
	BufferedOutputStream_flush( bs);
	(void) free ((void*) bs);
}

#ifndef   _NO_PROTO
BufferedOutputStream * newBufferedOutputStreamSized( OutputStream * o, int bufsize)
#else  /* _NO_PROTO */
BufferedOutputStream * newBufferedOutputStreamSized( o, bufsize)
	OutputStream * o ;
	int            bufsize ;
#endif /* _NO_PROTO */
{
	BufferedOutputStream * bs = (BufferedOutputStream*) malloc ((unsigned) sizeof(BufferedOutputStream));

	bs->outputstream = o;
	bs->bufsize      = bufsize;

	bs->write  = BufferedOutputStream_write;
	bs->close  = BufferedOutputStream_close;
	bs->flush  = BufferedOutputStream_flush;
	bs->delete = deleteBufferedOutputStream;

	return bs;
}

#ifndef   _NO_PROTO
BufferedOutputStream * newBufferedOutputStream( OutputStream * o)
#else  /* _NO_PROTO */
BufferedOutputStream * newBufferedOutputStream( o)
	OutputStream * o ;
#endif /* _NO_PROTO */
{
	return newBufferedOutputStreamSized( o, DEFAULT_BUFSIZE);
}


/* StdOutputStream */

#ifndef   _NO_PROTO
static int  StdOutputStream_write(StdOutputStream *, char *, int);
static void StdOutputStream_close(StdOutputStream *);
static void StdOutputStream_flush(StdOutputStream *);
static int  StdOutputStream_getID(StdOutputStream *);
#else  /* _NO_PROTO */
static int  StdOutputStream_write();
static void StdOutputStream_close();
static void StdOutputStream_flush();
static int  StdOutputStream_getID();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteStdOutputStream( StdOutputStream * sos)
#else  /* _NO_PROTO */
void deleteStdOutputStream( sos)
	StdOutputStream * sos ;
#endif /* _NO_PROTO */
{
	(void) free ((void*) sos);
}

#ifndef   _NO_PROTO
StdOutputStream * newStdOutputStream( int i)
#else  /* _NO_PROTO */
StdOutputStream * newStdOutputStream( i)
	int i ;
#endif /* _NO_PROTO */
{
	StdOutputStream * sos = (StdOutputStream*) malloc ((unsigned) sizeof(StdOutputStream));

	sos->id = i;
	sos->length = 0;

	sos->write   = StdOutputStream_write;
	sos->close   = StdOutputStream_close;
	sos->flush   = StdOutputStream_flush;
	sos->getID   = StdOutputStream_getID;
	sos->delete  = deleteStdOutputStream;

	return sos;
}

#ifndef   _NO_PROTO
StdOutputStream * newStdOutputStream_anon(void)
#else  /* _NO_PROTO */
StdOutputStream * newStdOutputStream_anon()
#endif /* _NO_PROTO */
{
	return newStdOutputStream( -1);
}


/* FileOutputStream */

#ifndef   _NO_PROTO
static int  FileOutputStream_getID(FileOutputStream *);
#else  /* _NO_PROTO */
static int  FileOutputStream_getID();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteFileOutputStream( FileOutputStream * fos)
#else  /* _NO_PROTO */
void deleteFileOutputStream( fos)
	FileOutputStream * fos ;
#endif /* _NO_PROTO */
{
	if (fos->filename)
		(void) free ((void*)fos->filename);
	if (fos->ostream)
		(*fos->ostream->delete)( fos->ostream);
	(void) free ((void*)fos);
}

#ifndef   _NO_PROTO
FileOutputStream * newFileOutputStreamFull( char * f, int a, int rw)
#else  /* _NO_PROTO */
FileOutputStream * newFileOutputStreamFull( f, a, rw)
	char * f ;
	int    a ;
	int    rw ;
#endif /* _NO_PROTO */
{
	FileOutputStream * fos;

	int access   = (rw ? O_RDWR : O_WRONLY);
	int mode     = (a ? O_APPEND : O_TRUNC);
	int id      = open( f, access|O_CREAT|mode, 0744);

	if (id == -1)
		return (FileOutputStream*)0;

	fos = (FileOutputStream*) malloc ((unsigned) sizeof(FileOutputStream));

	fos->ostream = newStdOutputStream( id);
	fos->parent  = (OutputStream*)fos->ostream;
	fos->filename = strdup( f);

	fos->write   = _inheritWrite;
	fos->getID   = FileOutputStream_getID;
	fos->close   = _inheritWriteClose;
	fos->flush   = _inheritFlush;
	fos->delete  = deleteFileOutputStream;

	return fos;
}

#ifndef   _NO_PROTO
FileOutputStream * newFileOutputStream( char * f)
#else  /* _NO_PROTO */
FileOutputStream * newFileOutputStream( f)
	char * f ;
#endif /* _NO_PROTO */
{
	return newFileOutputStreamFull( f, 0, 0);
}


/* TempFileOutputStream */

#ifndef   _NO_PROTO
static int  TempFileOutputStream_getContentLength(TempFileOutputStream *);
static void TempFileOutputStream_copy(TempFileOutputStream *, OutputStream *);
#else  /* _NO_PROTO */
static int  TempFileOutputStream_getContentLength();
static void TempFileOutputStream_copy();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
void deleteTempFileOutputStream( TempFileOutputStream * tfos)
#else  /* _NO_PROTO */
void deleteTempFileOutputStream( tfos)
	TempFileOutputStream * tfos ;
#endif /* _NO_PROTO */
{
	if (tfos->ostream) {
		(*tfos->close) ( tfos);
		(*tfos->ostream->delete)( tfos->ostream);
	}
	(void) free ((void*) tfos);
}

#ifndef   _NO_PROTO
TempFileOutputStream * newTempFileOutputStream(void)
#else  /* _NO_PROTO */
TempFileOutputStream * newTempFileOutputStream()
#endif /* _NO_PROTO */
{
	FileOutputStream * fos;

	TempFileOutputStream * tfos;

	fos = newFileOutputStreamFull( tmpnam((char*)0), 0, 1);
	if (!fos)
		return (TempFileOutputStream*)0;

	tfos  = (TempFileOutputStream*) malloc ((unsigned) sizeof(TempFileOutputStream));

	tfos->ostream = fos;
	tfos->parent  = (OutputStream*) fos;
	tfos->copy    = TempFileOutputStream_copy;
	tfos->write   = _inheritWrite;
	tfos->close   = _inheritWriteClose;
	tfos->flush   = _inheritFlush;
	tfos->getContentLength = TempFileOutputStream_getContentLength;
	tfos->delete  = deleteTempFileOutputStream;

	(void) unlink( fos->filename);

	return tfos;
}


/* PrintWriter */

#ifndef   _NO_PROTO
static void PrintWriter_print(PrintWriter *, char *);
static void PrintWriter_println(PrintWriter *, char *);
static void PrintWriter_printlnInt(PrintWriter *, int);
static void PrintWriter_printBoolean(PrintWriter *, int);
static void PrintWriter_printChar(PrintWriter *, char);
static void PrintWriter_printInt(PrintWriter *, int);
static void PrintWriter_printLong(PrintWriter *, long);
static void PrintWriter_printFloat(PrintWriter *, float);
static void PrintWriter_printDouble(PrintWriter *, double);
#else  /* _NO_PROTO */
static void PrintWriter_print();
static void PrintWriter_println();
static void PrintWriter_printlnInt();
static void PrintWriter_printBoolean();
static void PrintWriter_printChar();
static void PrintWriter_printInt();
static void PrintWriter_printLong();
static void PrintWriter_printFloat();
static void PrintWriter_printDouble();
#endif /* _NO_PROTO */


#ifndef   _NO_PROTO
void deletePrintWriter( PrintWriter* pw)
#else  /* _NO_PROTO */
void deletePrintWriter( pw)
	PrintWriter* pw ;
#endif /* _NO_PROTO */
{
	if (pw->convbuf)
		(void) free ((void*) pw->convbuf);
	(void) free ((void*) pw);
}

#ifndef   _NO_PROTO
PrintWriter * newPrintWriter( OutputStream * o)
#else  /* _NO_PROTO */
PrintWriter * newPrintWriter( o)
	OutputStream * o ;
#endif /* _NO_PROTO */
{
	PrintWriter * pw = (PrintWriter*) malloc ((unsigned) sizeof(PrintWriter));

	pw->os = o;
	pw->convbuf = (char*) malloc ((unsigned) 64 * sizeof(char));

	pw->print        = (void_f) PrintWriter_print;
	pw->println      = (void_f) PrintWriter_println;
	pw->printlnInt   = (void_f) PrintWriter_printlnInt;
	pw->printBoolean = (void_f) PrintWriter_printBoolean; /*ab*/
	pw->printChar    = (void_f) PrintWriter_printChar;
	pw->printInt     = (void_f) PrintWriter_printInt;
	pw->printLong    = (void_f) PrintWriter_printLong;
	pw->printFloat   = (void_f) PrintWriter_printFloat;
	pw->printDouble  = (void_f) PrintWriter_printDouble;  /*end*/
	pw->delete       = (void_f) deletePrintWriter;

	return pw;
}

/* SocketInputStream */

#ifndef   _NO_PROTO
void deleteSocketInputStream( SocketInputStream * sid)
#else  /* _NO_PROTO */
void deleteSocketInputStream( sid)
	SocketInputStream * sid ;
#endif /* _NO_PROTO */
{
	if (sid->sid)
		(*sid->sid->delete)( sid->sid);
	(void) free ((void*) sid);
}

#ifndef   _NO_PROTO
static void StdInputStream_setBlocking(StdInputStream *, int);
#else  /* _NO_PROTO */
static void StdInputStream_setBlocking();
#endif /* _NO_PROTO */

#ifdef    WIN32
#ifndef   _NO_PROTO
static int SocketInputStream_read(SocketInputStream *, char *, int);
#else  /* _NO_PROTO */
static int SocketInputStream_read();
#endif /* _NO_PROTO */
#endif /* WIN32 */

#ifndef   _NO_PROTO
SocketInputStream* newSocketInputStream( SocketHandle id)
#else  /* _NO_PROTO */
SocketInputStream* newSocketInputStream( id)
	SocketHandle id ;
#endif /* _NO_PROTO */
{
	SocketInputStream * sid = (SocketInputStream*)malloc ((unsigned) sizeof(SocketInputStream));
#ifdef    WIN32
	sid->handle = id;
	sid->sid    = (StdInputStream*)0;
	sid->read   = SocketInputStream_read;
#else  /* WIN32 */
	sid->sid      =  newStdInputStream( id);
	StdInputStream_setBlocking(sid->sid, 1);

	sid->parent    = (InputStream*) sid->sid;
	sid->available = _inheritAvailable;
	sid->readChar  = _inheritReadChar;
	sid->read      = _inheritRead;
	sid->skip      = _inheritSkip;
	sid->close     = _inheritReadClose;
#endif /* WIN32 */

	sid->delete    = deleteSocketInputStream;

	return sid;
}

#ifdef WIN32

#ifndef   _NO_PROTO
static int SocketInputStream_read( SocketInputStream * sid, char * buf, int len)
#else  /* _NO_PROTO */
static int SocketInputStream_read( sid, buf, len)
	SocketInputStream * sid ;
	char              * buf ;
	int                 len ;
#endif /* _NO_PROTO */
{
		return recv( sid->handle, buf,len, 0);
}

#ifndef   _NO_PROTO
static int SocketOutputStream_write( SocketOutputStream * sod, char * buf, int len)
#else  /* _NO_PROTO */
static int SocketOutputStream_write( sod, buf, len)
	SocketOutputStream * sod ;
	char               * buf ;
	int                  len ;
#endif /* _NO_PROTO */
{
		return send( sod->handle, buf,len, 0);
}


#ifndef   _NO_PROTO
void deleteSocketOutputStream( SocketOutputStream * sid)
#else  /* _NO_PROTO */
void deleteSocketOutputStream( sid)
	SocketOutputStream * sid ;
#endif /* _NO_PROTO */
{
		(void) free ((void*) sid);
}

#ifndef   _NO_PROTO
SocketOutputStream * newSocketOutputStream( SocketHandle id)
#else  /* _NO_PROTO */
SocketOutputStream * newSocketOutputStream( id)
	SocketHandle id ;
#endif /* _NO_PROTO */
{
		SocketOutputStream * sod = (SocketOutputStream*) malloc ((unsigned) sizeof(SocketOutputStream));
		
		sod->handle = id;
		sod->write  = SocketOutputStream_write;
		sod->delete = deleteSocketOutputStream;

		return sod;
}
#else  /* WIN32 */
#ifndef   _NO_PROTO
SocketOutputStream *newSocketOutputStream( SocketHandle id)
#else  /* _NO_PROTO */
SocketOutputStream *newSocketOutputStream( id)
	SocketHandle id ;
#endif /* _NO_PROTO */
{
	return newStdOutputStream( id);
}
#endif /* WIN32 */

/* METHODS */

/* StdInputStream - public */

#ifndef   _NO_PROTO
static int StdInputStream_readChar( StdInputStream * sid)
#else  /* _NO_PROTO */
static int StdInputStream_readChar( sid)
	StdInputStream * sid ;
#endif /* _NO_PROTO */
{
	char buf[1];
	return (*sid->read)( sid, buf, 1);
}

#ifndef   _NO_PROTO
static int StdInputStream_read( StdInputStream * sid, char * buf, int len)
#else  /* _NO_PROTO */
static int StdInputStream_read( sid, buf, len)
	StdInputStream * sid ;
	char           * buf ;
	int              len ;
#endif /* _NO_PROTO */
{
	register int n;
	register char * bptr = buf;
	register int    cnt  = len;
	register int  total  = 0;

	if (sid->ateof)
		return -1;
	do {
		if ((n = read( sid->id, bptr, cnt)) == -1) {
			sid->ateof = 1;
			return -1;
		}
		total += n;
		if (!n) {
			sid->ateof = 1;
			if (!total)
				return -1;
			break;
		}
		cnt  -= n;
		bptr += n;

	} while (total < len && !sid->dontblock);

	buf[total] = '\0';

#ifdef    BAD_PROXY
	/* this is for an error condition where proxies can garbage the data */
	n = strlen(buf);
	while (n < total) {
		buf[n] = ' ';
		n = strlen(buf);
	}
#endif /* BAD_PROXY */

	return total;
}

#ifndef   _NO_PROTO
static int StdInputStream_available( StdInputStream * sid)
#else  /* _NO_PROTO */
static int StdInputStream_available( sid)
	StdInputStream * sid ;
#endif /* _NO_PROTO */
{
	return !sid->ateof;
}

#ifndef   _NO_PROTO
static void StdInputStream_setBlocking( StdInputStream * sid, int i)
#else  /* _NO_PROTO */
static void StdInputStream_setBlocking( sid, i)
	StdInputStream * sid ;
	int              i ;
#endif /* _NO_PROTO */
{
	sid->dontblock = !i;
}

#ifndef   _NO_PROTO
static int StdInputStream_skip( StdInputStream * sid, int i)
#else  /* _NO_PROTO */
static int StdInputStream_skip( sid, i)
	StdInputStream * sid ;
	int              i ;
#endif /* _NO_PROTO */
{ /* not implemented */
	return 0;
}

#ifndef   _NO_PROTO
static void StdInputStream_close( StdInputStream * sid)
#else  /* _NO_PROTO */
static void StdInputStream_close( sid)
	StdInputStream * sid ;
#endif /* _NO_PROTO */
{
	if (sid->id != -1)
		(void) close ( sid->id);
	sid->id = -1;
}


/* CachedInputStream - public */

#ifndef   _NO_PROTO
static void CachedInputStream_setCache( CachedInputStream * ca, char * buf, int len)
#else  /* _NO_PROTO */
static void CachedInputStream_setCache( ca, buf, len)
	CachedInputStream * ca ;
	char              * buf ;
	int                 len ;
#endif /* _NO_PROTO */
{
	ca->cache    = buf;
	ca->cachelen = len;
	ca->cacheptr = ca->cache;
}

#ifndef   _NO_PROTO
static int CachedInputStream_available( CachedInputStream * ca)
#else  /* _NO_PROTO */
static int CachedInputStream_available( ca)
	CachedInputStream * ca ;
#endif /* _NO_PROTO */
{
	if (ca->cachelen > 0) {
		return 1;
	}
	return (*ca->istream->available)( ca->istream);
}

#ifndef   _NO_PROTO
static int CachedInputStream_readChar( CachedInputStream * ca)
#else  /* _NO_PROTO */
static int CachedInputStream_readChar( ca)
	CachedInputStream * ca ;
#endif /* _NO_PROTO */
{ /* not used here */
	char buf[1];
	if ((*ca->read)( ca, buf, 1) == -1)
		return -1;
	return (int)(buf[0]);
}

#ifndef   _NO_PROTO
static int CachedInputStream_read( CachedInputStream * ca, char * buf, int len)
#else  /* _NO_PROTO */
static int CachedInputStream_read( ca, buf, len)
	CachedInputStream * ca ;
	char              * buf ;
	int                 len ;
#endif /* _NO_PROTO */
{
	register int clen   = 0;
	register int bigger = 0;
	char * bptr = buf;

	if (ca->cachelen > 0) {
		clen = ca->cachelen;
		if (clen > len)
			bigger++;

		MEMCPY ( bptr, ca->cacheptr, (bigger ? len : clen));

		if (bigger) {
			ca->cachelen -= len;
			ca->cacheptr += (clen-len);

			return len;
		}
		ca->cacheptr = (char*)0;
		ca->cachelen = 0;
		len  -= clen;
		bptr += clen;
	}

	clen += (*ca->istream->read)( ca->istream, bptr, len);

	return clen;
}

#ifndef   _NO_PROTO
static int CachedInputStream_skip( CachedInputStream * ca, int len)
#else  /* _NO_PROTO */
static int CachedInputStream_skip( ca, len)
	CachedInputStream * ca ;
	int                 len ;
#endif /* _NO_PROTO */
{
	register int clen   = 0;
	register int bigger = 0;

	if (ca->cachelen > 0) {
		clen = ca->cachelen;
		if (clen > len)
			bigger++;
		if (bigger) {
			ca->cachelen -= len;
			ca->cacheptr += (clen-len);

			return len;
		}
		ca->cacheptr = (char*)0;
		ca->cachelen = 0;
		len  -= clen;
	}

	clen += (*ca->istream->skip)( ca->istream, len);
	return clen;
}

#ifndef   _NO_PROTO
static void CachedInputStream_close( CachedInputStream * ca)
#else  /* _NO_PROTO */
static void CachedInputStream_close( ca)
	CachedInputStream * ca ;
#endif /* _NO_PROTO */
{
	(*ca->istream->close)( ca->istream);
}


/* BufferedInputStream - public */

#ifndef   _NO_PROTO
static int BufferedInputStream_available( BufferedInputStream * bs)
#else  /* _NO_PROTO */
static int BufferedInputStream_available( bs)
	BufferedInputStream * bs ;
#endif /* _NO_PROTO */
{
	if (!bs->inputstream)
		return 0;
	return (*bs->inputstream->available)( bs->inputstream);
}

#ifndef   _NO_PROTO
static int BufferedInputStream_readChar( BufferedInputStream * bs)
#else  /* _NO_PROTO */
static int BufferedInputStream_readChar( bs)
	BufferedInputStream * bs ;
#endif /* _NO_PROTO */
{ /* not used here */
	char buf[1];
	return (*bs->read)( bs, buf, 1);
}

#ifndef   _NO_PROTO
static char * BufferedInputStream_currentPosition( BufferedInputStream * bs)
#else  /* _NO_PROTO */
static char * BufferedInputStream_currentPosition( bs)
	BufferedInputStream * bs ;
#endif /* _NO_PROTO */
{
	return bs->dptr;
}

#ifndef   _NO_PROTO
static void BufferedInputStream_doubleBuffer( BufferedInputStream *) ;
#else  /* _NO_PROTO */
static void BufferedInputStream_doubleBuffer();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
static int BufferedInputStream_indexOf( BufferedInputStream * bs, char c)
#else  /* _NO_PROTO */
static int BufferedInputStream_indexOf( bs, c)
	BufferedInputStream * bs ;
	char                  c ;
#endif /* _NO_PROTO */
{

	char * start = (char*)0;
	char * sptr  = (char*)0;
	char * q;

	if (!bs->buflen)
		if (!BufferedInputStream_fillBuffer( bs))
			return -1;

	/* look in the current buffer */
	if ((q = STRCHR( bs->dptr, c)) != (char*)0)
		return q-(bs->dptr);

	while (BufferedInputStream_available( bs) &&
	       BufferedInputStream_fillBuffer( bs)) {
		/* make the buffer as large as it can be */
		if ((q = STRCHR( bs->dptr, c)) != (char*)0)
			return q-(bs->dptr);

		BufferedInputStream_doubleBuffer( bs);
	}
	return -1;
}

#ifndef   _NO_PROTO
static int BufferedInputStream_read( BufferedInputStream * bs, char * buf, int len)
#else  /* _NO_PROTO */
static int BufferedInputStream_read( bs, buf, len)
	BufferedInputStream * bs ;
	char                * buf ;
	int                   len ;
#endif /* _NO_PROTO */
{
	register int clen   = 0;
	register int bigger = 0;
	register char * bptr = buf;
	int tally   = 0;

	do {
		if (!bs->buflen)
			BufferedInputStream_fillBuffer( bs);
		if (!bs->buflen)
			break;
		clen = bs->buflen;
		if (clen > len)
			bigger++;

		MEMCPY ( bptr, bs->dptr, (bigger ? len : clen));

		if (bigger) {
			bs->buflen -= len;
			bs->dptr   += (clen-len);

			return len;
		}
		bs->dptr = (char*)0;
		bs->buflen = 0;
		len   -= clen;
		bptr  += clen;
		tally += clen;

	} while (len > 0);

	return tally;
}

#ifndef   _NO_PROTO
static int BufferedInputStream_skip( BufferedInputStream * bs, int len)
#else  /* _NO_PROTO */
static int BufferedInputStream_skip( bs, len)
	BufferedInputStream * bs ;
	int                   len ;
#endif /* _NO_PROTO */
{
	register int clen   = 0;
	register int bigger = 0;
	int tally   = 0;

	do {
		if (!bs->buflen) {
			BufferedInputStream_fillBuffer( bs);
		}
		if (!bs->buflen)
			break;
		if (bs->buflen > len) {
			bs->buflen -= len;
			bs->dptr   += len;
			return 0;
		}
		bs->dptr   = (char*)0;
		len   -= bs->buflen;
		bs->buflen = 0;

	} while (len > 0);

	return 0;
}

#ifndef   _NO_PROTO
static void BufferedInputStream_close( BufferedInputStream* bs) 
#else  /* _NO_PROTO */
static void BufferedInputStream_close( bs) 
	BufferedInputStream* bs ;
#endif /* _NO_PROTO */
{
}

/* BufferedInputStream - private */

#ifndef   _NO_PROTO
static int BufferedInputStream_fillBuffer( BufferedInputStream * bs)
#else  /* _NO_PROTO */
static int BufferedInputStream_fillBuffer( bs)
	BufferedInputStream * bs ;
#endif /* _NO_PROTO */
{
	int len;
	int n;

	if (!bs->data) {
		bs->data   = (void*) malloc ((unsigned) (bs->bufsize+1) * sizeof(char));
		bs->dptr   = (char*)0;
		bs->buflen = 0;
	}

	if (bs->buflen > 0) {
		MEMCPY ( bs->data, bs->dptr, bs->buflen);
		bs->dptr = (char*)bs->data+bs->buflen;
		len  = bs->bufsize - bs->buflen;
	} else  {
		bs->dptr = (char*)bs->data;
		len  = bs->bufsize;
		bs->buflen = 0;
	}

	n = (*bs->inputstream->read) ( bs->inputstream, bs->dptr, len);

	if ((n == -1) && !bs->buflen)
		return n;
	if (n != -1) {
		bs->dptr[n] = '\0';
		bs->buflen += n;
	}

	bs->dptr    = (char*)bs->data;

	return bs->buflen;
}

#ifndef   _NO_PROTO
static void BufferedInputStream_doubleBuffer( BufferedInputStream * bs)
#else  /* _NO_PROTO */
static void BufferedInputStream_doubleBuffer( bs)
	BufferedInputStream * bs ;
#endif /* _NO_PROTO */
{
	int i = bs->bufsize*2;

	bs->data = (void*)realloc(bs->data, i);

	bs->bufsize = i;
}

#ifndef   _NO_PROTO
void * BufferedInputStream_readAll( BufferedInputStream * bs, int * sizep)
#else  /* _NO_PROTO */
void * BufferedInputStream_readAll( bs, sizep)
	BufferedInputStream * bs ;
	int                 * sizep ;
#endif /* _NO_PROTO */
{
	char * block = (char*)0;
	
	int n;
	int blocklen = 0;

	*sizep = 0;

	while( 1) {
		if ((n = BufferedInputStream_fillBuffer( bs)) == -1) {
			break;
		}
		if (!n) {
			continue;
		}
		if (!block) {
			block    = (char*)bs->data;
			blocklen = n;
			bs->data     = (char*)0;
			bs->dptr     = (char*)0;
			bs->buflen   = 0;
		} else {
			block = (char*)realloc(block, blocklen+n+1);
			MEMCPY(block+blocklen, bs->data, n);
			blocklen += n;
			bs->dptr = (char*)0;
			bs->buflen = 0;
		}
	}


	*sizep = blocklen;

	if (!block)
		return (void*)0;

	block[blocklen] = '\0';

	if (bs->data) {
		if (bs->data != block)
			free(bs->data);
		bs->data   = (char*)0;
		bs->dptr   = (char*)0;
		bs->buflen = 0;
	}

	return (void*)block;
}

/* DataInputStream - public */

#ifndef   _NO_PROTO
static char * DataInputStream_readLine( DataInputStream * ds)
#else  /* _NO_PROTO */
static char * DataInputStream_readLine( ds)
	DataInputStream * ds ;
#endif /* _NO_PROTO */
{
	int  ix  = BufferedInputStream_indexOf( ds->bs, '\n');
	char * p = BufferedInputStream_currentPosition( ds->bs);

	if ( ix == -1)
		return (char*)0;

	p[ix]    = '\0';

	if (p[ix-1] == '\r')
		p[ix-1] = '\0';

	(*ds->bs->skip)( ds->bs, ix+1);

	return p;
}

/* InputData - public */
#ifndef   _NO_PROTO
static void InputData_loadData(InputData *);
#else  /* _NO_PROTO */
static void InputData_loadData();
#endif /* _NO_PROTO */

#ifndef   _NO_PROTO
static void * InputData_getData( InputData * id)
#else  /* _NO_PROTO */
static void * InputData_getData( id)
	InputData * id ;
#endif /* _NO_PROTO */
{
	InputData_loadData( id);

	return id->data;
}

#ifndef   _NO_PROTO
static int InputData_getSize( InputData * id)
#else  /* _NO_PROTO */
static int InputData_getSize( id)
	InputData * id ;
#endif /* _NO_PROTO */
{
	InputData_loadData( id);

	return id->size;
}

/* InputData - private */

#ifndef   _NO_PROTO
static void InputData_loadData( InputData * id)
#else  /* _NO_PROTO */
static void InputData_loadData( id)
	InputData * id ;
#endif /* _NO_PROTO */
{
	if (id->loaded)
		return;

	id->size   = 0;
	id->data   = BufferedInputStream_readAll( id->bs, &id->size);
	id->loaded = 1;
}


/* OutputStream is abstract */


#ifndef   _NO_PROTO
static int BufferedOutputStream_write( BufferedOutputStream * bs, char * buf, int len)
#else  /* _NO_PROTO */
static int BufferedOutputStream_write( bs, buf, len)
	BufferedOutputStream * bs ;
	char                 * buf ;
	int                    len ;
#endif /* _NO_PROTO */
{
	if (!bs->data) {
		bs->data   = (char*) malloc ((unsigned) (bs->bufsize+1) * sizeof(char));
		bs->dptr   = bs->data;
		bs->buflen = 0;
	}
	if (bs->buflen + len > bs->bufsize)
		(*bs->flush)( bs);

	if (len > bs->bufsize)
		return (*bs->outputstream->write)(bs->outputstream, buf, len);

	MEMCPY ( bs->dptr, buf, len);

	bs->buflen += len;
	bs->dptr   += len;

	return len;
}

#ifndef   _NO_PROTO
static void BufferedOutputStream_flush( BufferedOutputStream * bs)
#else  /* _NO_PROTO */
static void BufferedOutputStream_flush( bs)
	BufferedOutputStream * bs ;
#endif /* _NO_PROTO */
{
	if (!bs->data)
		return;
	if (!bs->buflen)
		return;
	(void) (*bs->outputstream->write)(bs->outputstream, bs->data, bs->buflen);
	bs->dptr     = bs->data;
	bs->buflen   = 0;
}

#ifndef   _NO_PROTO
static void BufferedOutputStream_close( BufferedOutputStream * bs)
#else  /* _NO_PROTO */
static void BufferedOutputStream_close( bs)
	BufferedOutputStream * bs ;
#endif /* _NO_PROTO */
{
	BufferedOutputStream_flush( bs);
	(*bs->outputstream->close)( bs->outputstream);
}


/* StdOutputStream - public */

#ifndef   _NO_PROTO
static int StdOutputStream_write( StdOutputStream * sos, char * buf, int len)
#else  /* _NO_PROTO */
static int StdOutputStream_write( sos, buf, len)
	StdOutputStream * sos ;
	char            * buf ;
	int               len ;
#endif /* _NO_PROTO */
{
	int n = write( sos->id, buf, len);
	if (n != -1)
		sos->length += n;

	return n;
}

#ifndef   _NO_PROTO
static void StdOutputStream_flush( StdOutputStream * sos)
#else  /* _NO_PROTO */
static void StdOutputStream_flush( sos)
	StdOutputStream * sos ;
#endif /* _NO_PROTO */
{
}

#ifndef   _NO_PROTO
static void StdOutputStream_close( StdOutputStream * sos)
#else  /* _NO_PROTO */
static void StdOutputStream_close( sos)
	StdOutputStream * sos ;
#endif /* _NO_PROTO */
{
	(void) close( sos->id);
}

#ifndef   _NO_PROTO
static int StdOutputStream_getID( StdOutputStream * sos)
#else  /* _NO_PROTO */
static int StdOutputStream_getID( sos)
	StdOutputStream * sos ;
#endif /* _NO_PROTO */
{
	return sos->id;
}


/* FileOutputStream - private */

#ifndef   _NO_PROTO
static int FileOutputStream_getID( FileOutputStream * fos)
#else  /* _NO_PROTO */
static int FileOutputStream_getID( fos)
	FileOutputStream * fos ;
#endif /* _NO_PROTO */
{
	return (*fos->ostream->getID)( fos->ostream);
}

/* TempFileOutputStream */

#ifndef   _NO_PROTO
static int TempFileOutputStream_getContentLength( TempFileOutputStream * tfos)
#else  /* _NO_PROTO */
static int TempFileOutputStream_getContentLength( tfos)
	TempFileOutputStream * tfos ;
#endif /* _NO_PROTO */
{
	int id = (*tfos->ostream->getID)( tfos->ostream);

	struct stat sb;

	if (fstat( id, &sb) == -1)
		return -1;

	return sb.st_size;
}

#ifndef   _NO_PROTO
static void TempFileOutputStream_copy( TempFileOutputStream * tfos, OutputStream * o)
#else  /* _NO_PROTO */
static void TempFileOutputStream_copy( tfos, o)
	TempFileOutputStream * tfos ;
	OutputStream         * o ;
#endif /* _NO_PROTO */
{
	int id = (*tfos->ostream->getID)( tfos->ostream);
	StdInputStream * i;
	InputData      * idata;
	void *            data;
	int               size;

	(void) lseek( id, 0, SEEK_SET);

	i = newStdInputStream( id);

	idata = newInputData( (InputStream*)i);
	data  = (*idata->getData)( idata);
	size  = (*idata->getSize)( idata);

	(*o->write)( o, (char*)data, size);

	(*o->flush)(o);

	(*idata->delete)( idata);
	(*i->delete)( i);
}


/* PrintWriter */

#ifndef   _NO_PROTO
static void PrintWriter_print( PrintWriter * pw, char * s)
#else  /* _NO_PROTO */
static void PrintWriter_print( pw, s)
	PrintWriter * pw ;
	char        * s ;
#endif /* _NO_PROTO */
{
	if (s)
		(*pw->os->write)( pw->os, s, strlen(s));
}

#ifndef   _NO_PROTO
static void PrintWriter_println( PrintWriter * pw, char * s)
#else  /* _NO_PROTO */
static void PrintWriter_println( pw, s)
	PrintWriter * pw ;
	char        * s ;
#endif /* _NO_PROTO */
{
	if (s)
		(*pw->os->write)( pw->os, s, strlen(s));
	(*pw->os->write)( pw->os, "\n", 1);
}

#ifndef   _NO_PROTO
static void PrintWriter_printInt( PrintWriter * pw, int i)
#else  /* _NO_PROTO */
static void PrintWriter_printInt( pw, i)
	PrintWriter * pw ;
	int           i ;
#endif /* _NO_PROTO */
{
	(void) sprintf( pw->convbuf, "%d", i);
	(*pw->print)( pw, pw->convbuf);
}

#define TRUE_STRING     "true"
#define FALSE_STRING    "false"

#ifndef   _NO_PROTO
static void PrintWriter_printBoolean( PrintWriter * pw, int i)
#else  /* _NO_PROTO */
static void PrintWriter_printBoolean( pw, i)
	PrintWriter * pw ;
	int           i ;
#endif /* _NO_PROTO */
{
	(*pw->print)( pw, (i ? TRUE_STRING : FALSE_STRING));
}

#ifndef   _NO_PROTO
static void PrintWriter_printlnInt( PrintWriter * pw, int i)
#else  /* _NO_PROTO */
static void PrintWriter_printlnInt( pw, i)
	PrintWriter * pw ;
	int           i ;
#endif /* _NO_PROTO */
{
	PrintWriter_printInt( pw,i);
	(*pw->println)( pw, (char*)0);
}

#ifndef   _NO_PROTO
static void PrintWriter_printChar( PrintWriter *pw, char c)
#else  /* _NO_PROTO */
static void PrintWriter_printChar( pw, c)
	PrintWriter *pw ;
	char         c ;
#endif /* _NO_PROTO */
{
	(void) sprintf( pw->convbuf, "%c", c);
	(*pw->print)( pw, pw->convbuf);
}

#ifndef   _NO_PROTO
static void PrintWriter_printLong( PrintWriter * pw, long l)
#else  /* _NO_PROTO */
static void PrintWriter_printLong( pw, l)
	PrintWriter * pw ;
	long          l ;
#endif /* _NO_PROTO */
{
	(void) sprintf( pw->convbuf, "%l", l);
	(*pw->print)( pw, pw->convbuf);
}

#ifndef   _NO_PROTO
static void PrintWriter_printFloat( PrintWriter *pw, float f)
#else  /* _NO_PROTO */
static void PrintWriter_printFloat( pw, f)
	PrintWriter * pw ;
	float         f ;
#endif /* _NO_PROTO */
{
	(void) sprintf( pw->convbuf, "%f", (double)f);
	(*pw->print)( pw, pw->convbuf);
}

#ifndef   _NO_PROTO
static void PrintWriter_printDouble( PrintWriter * pw, double d)
#else  /* _NO_PROTO */
static void PrintWriter_printDouble( pw, d)
	PrintWriter * pw ;
	double        d ;
#endif /* _NO_PROTO */
{
	(void) sprintf( pw->convbuf, "%f", d);
	(*pw->print)( pw, pw->convbuf);
}

#ifndef   _NO_PROTO
static void PrintWriter_close( PrintWriter * pw)
#else  /* _NO_PROTO */
static void PrintWriter_close( pw)
	PrintWriter * pw ;
#endif /* _NO_PROTO */
{
	if (pw->os)
		(*pw->os->close)( pw);
}

/* these routines allow basic functions to be "inherited" */

#ifndef   _NO_PROTO
static int _inheritRead( InputStream * i, char * buf, int len)
#else  /* _NO_PROTO */
static int _inheritRead( i, buf, len)
	InputStream * i ;
	char        * buf ;
	int           len ;
#endif /* _NO_PROTO */
{
	return (*i->parent->read)( i->parent,buf, len);
}

#ifndef   _NO_PROTO
static int _inheritReadChar( InputStream * i)
#else  /* _NO_PROTO */
static int _inheritReadChar( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	return (*i->parent->readChar)( i->parent);
}

#ifndef   _NO_PROTO
static int _inheritAvailable( InputStream * i)
#else  /* _NO_PROTO */
static int _inheritAvailable( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	return (*i->parent->available)( i->parent);
}

#ifndef   _NO_PROTO
static int _inheritSkip( InputStream * i, int ski)
#else  /* _NO_PROTO */
static int _inheritSkip( i, ski)
	InputStream * i ;
	int           ski ;
#endif /* _NO_PROTO */
{
	return (*i->parent->skip)( i->parent, ski);
}

#ifndef   _NO_PROTO
static void _inheritReadClose( InputStream * i)
#else  /* _NO_PROTO */
static void _inheritReadClose( i)
	InputStream * i ;
#endif /* _NO_PROTO */
{
	(*i->parent->close)( i->parent);
}

#ifndef   _NO_PROTO
static int _inheritWrite( OutputStream * i, char * buf, int len)
#else  /* _NO_PROTO */
static int _inheritWrite( i, buf, len)
	OutputStream * i ;
	char         * buf ;
	int           len ;
#endif /* _NO_PROTO */
{
	return (*i->parent->write)( i->parent,buf, len);
}

#ifndef   _NO_PROTO
static void _inheritWriteClose( OutputStream * o)
#else  /* _NO_PROTO */
static void _inheritWriteClose( o)
	OutputStream * o ;
#endif /* _NO_PROTO */
{
	(*o->parent->close)( o->parent);
}

#ifndef   _NO_PROTO
static void _inheritFlush( OutputStream * o)
#else  /* _NO_PROTO */
static void _inheritFlush( o)
	OutputStream * o ;
#endif /* _NO_PROTO */
{
	(*o->parent->flush)( o->parent);
}

#ifndef   _NO_PROTO
static int startsWithDocType( char *buf, int len)
#else  /* _NO_PROTO */
static int startsWithDocType( buf, len)
	char * buf;
	int len;
#endif /* _NO_PROTO */
{
	char * testme = "<!DOCTYPE";
	char * p;
	if (len < 10)
		return 0;

	for (p = buf; testme && *testme; testme++, p++) {
		if (toupper(*p) != toupper(*testme))
			return 0;
	}
	return 1;
}


/* this is a common interface to the SGML parsing engine */

#ifndef   _NO_PROTO
char * scGetTypedDocument( InputStream *i, char *t)
#else  /* _NO_PROTO */
char * scGetTypedDocument( i, t)
	InputStream * i;
	char        * t;
#endif /* _NO_PROTO */
{
	/* temporary file copy.... */
	static char ofile[64];
	extern BufferedInputStream * newBufferedInputStream();
	BufferedInputStream * bi = newBufferedInputStream( i);
	int wid;
	int first = 1;

	(void) sprintf ( ofile, "/tmp/sgis%d._", getpid());
	ofile[strlen(ofile)+1] = '\0';



	wid = open( ofile, O_RDWR|O_CREAT|O_TRUNC, 0x744);
	if (wid == -1) {
		(void) fprintf ( stderr, "Cannot write %s\n", ofile);
		return (char*)0;
	}

	while (1)  {
		if ((*bi->load)( bi) == -1)
			break;
		if (first) {
			if (!startsWithDocType(bi->dptr, bi->buflen))
				write(  wid, t, strlen(t));
			first = 0;
		}
		write( wid, bi->dptr, bi->buflen);
		bi->dptr = (char*)0;
		bi->buflen = 0;
	}


	close( wid);
	(*bi->close)(bi);
	(*bi->delete)(bi);

	return ofile;
}

#ifdef    SIMPLETEST
/* ARGSUSED */
int main(int ac, char **av)
{
	FileInputStream * fs = newFileInputStream("foopydoo");
	DataInputStream * ds;
	char * s;

	if (!fs) {
		printf("Cannot open stream\n");
		return;
	}
	ds = newDataInputStream( (InputStream*)fs);

	while ((s = (*ds->readLine)(ds)) != (char*)0)
		printf("%s\n", s);

	(*ds->delete)( ds);
	(*fs->delete)( fs);

}
#endif /* SIMPLETEST */

#ifdef    SIMPLETEST2
/* ARGSUSED */
int main(int ac, char **av)
{
	FileInputStream * fs = newFileInputStream("foopydoo");
	DataInputStream * ds;
	FileOutputStream * ofs;

	char * s;

	if (!fs) {
		printf("Cannot open stream\n");
		return;
	}
	ofs = newFileOutputStream("piggywiggy");

	ds = newDataInputStream( (InputStream*)fs);

	while ((s = (*ds->readLine)(ds)) != (char*)0) {
		(*ofs->write)( ofs, s, strlen(s));
		(*ofs->write)( ofs, "\n", 1);
	}

	(*ofs->close)(ofs);
	(*ds->delete)( ds);
	(*fs->delete)( fs);
	(*ofs->delete)( ofs);
}
#endif /* SIMPLETEST2 */

#ifdef    SIMPLETEST3
/* ARGSUSED */
int main(int ac, char **av)
{
	FileInputStream * fs = newFileInputStream("foopydoo");
	DataInputStream * ds;
	FileOutputStream * ofs;
	BufferedOutputStream * bos;
	char * s;

	if (!fs) {
		printf("Cannot open stream\n");
		return;
	}
	ofs = newFileOutputStream("piggywiggy");
	bos = newBufferedOutputStream( (OutputStream*)ofs);
	ds  = newDataInputStream( (InputStream*)fs);

	while ((s = (*ds->readLine)(ds)) != (char*)0) {
		(*bos->write)( bos, s, strlen(s));
		(*bos->write)( bos, "\n", 1);
	}
	(*bos->delete)(bos);
	(*ofs->close)(ofs);
	(*ds->delete)( ds);
	(*fs->delete)( fs);
	(*ofs->delete)( ofs);
}

#endif /* SIMPLETEST3 */

#ifdef    TESTSTREAMS
/* ARGSUSED */
int main(int ac, char **av)
{
	FileInputStream * fs = newFileInputStream("foopydoo");
	FileOutputStream * fos;
	DataInputStream * ds;
	TempFileOutputStream * ts;
	BufferedOutputStream * bs;
	PrintWriter * p;

	char * s;

	if (!fs) {
		printf("Cannot open stream\n");
		return;
	}
	ds = newDataInputStream( (InputStream*)fs);
	ts = newTempFileOutputStream();
	if (!ts) {
		printf("Cannot open tempstream\n");
		return;
	}

	bs = newBufferedOutputStream( (OutputStream*)ts);
	p  = newPrintWriter( (OutputStream*)bs);

	while ((s = (*ds->readLine)(ds)) != (char*)0)
		(*p->println)(p, s);

	(*p->delete)( p);
	(*bs->delete)( bs);
	(*ds->delete)( ds);
	(*fs->delete)( fs);

	fos = newFileOutputStream("f2");
	if (!fos) {
		printf("Cannot open stream\n");
		return;
	}

	(*ts->copy)( ts, fos);

	(*ts->delete)( ts);
	(*fos->delete)( fos);
}
#endif /* TESTSTREAMS */
