#include "Ustreams.h"

int
StdInputStream::read( char * buf, int len)
{
	register int n;
	register char * bptr = buf;
	register int    cnt  = len;
	register int  total  = 0;

	if (ateof)
		return -1;
	do {
		if ((n = ::read( id, bptr, cnt)) == -1) {
			ateof = 1;
			return -1;
		}
		total += n;
		if (!n) {
			ateof = 1;
			if (!total)
				return -1;
			break;
		}
		cnt  -= n;
		bptr += n;

	} while (total < len && !dontblock);

	buf[total] = '\0';

#ifdef    BAD_PROXY
	// this is for an error condition where proxies can garbage the data
	n = strlen(buf);
	while (n < total) {
		buf[n] = ' ';
		n = strlen(buf);
	}
#endif /* BAD_PROXY */

	return total;
}

int
CachedInputStream::read(  char *buf, int len)
{
	register int clen   = 0;
	register int bigger = 0;
	char * bptr = buf;

	if (cachelen > 0) {
		clen = cachelen;
		if (clen > len)
			bigger++;

		MEMCPY ( bptr, cacheptr, (bigger ? len : clen));

		if (bigger) {
			cachelen -= len;
			cacheptr += (clen-len);

			return len;
		}
		cacheptr = (char*)0;
		cachelen = 0;
		len  -= clen;
		bptr += clen;
	}

	clen += istream->read( bptr, len);

	return clen;
}

int
CachedInputStream::skip(  int len)
{
	register int clen   = 0;
	register int bigger = 0;

	if (cachelen > 0) {
		clen = cachelen;
		if (clen > len)
			bigger++;
		if (bigger) {
			cachelen -= len;
			cacheptr += (clen-len);

			return len;
		}
		cacheptr = (char*)0;
		cachelen = 0;
		len  -= clen;
	}

	clen += istream->skip( len);
	return clen;
}

int
BufferedInputStream::fillBuffer()
{
	int len;
	int n;

	if (!data) {
		data   = (void*) malloc ((unsigned) (bufsize+1) * sizeof(char));
		dptr   = (char*)0;
		buflen = 0;
	}

	if (buflen > 0) {
		MEMCPY ( data, dptr, buflen);
		dptr = (char*)data+buflen;
		len  = bufsize - buflen;
	} else  {
		dptr = (char*)data;
		len  = bufsize;
		buflen = 0;
	}


	n = inputstream->read ( dptr, len);

	if ((n == -1) && !buflen)
		return n;
	if (n != -1) {
		dptr[n] = '\0';
		buflen += n;
	}

	dptr    = (char*)data;

	return buflen;
}


void
BufferedInputStream::doubleBuffer()
{
	int i = bufsize*2;

	data = (void*)REALLOC(data, i);

	bufsize = i;
}

int
BufferedInputStream::indexOf(char c)
{
	char * start = (char*)0;
	char * sptr  = (char*)0;
	char * q;

	if (!buflen)
		if (!fillBuffer())
			return -1;

	// look in the current buffer
	if ((q = STRCHR( dptr, c)) != (char*)0)
		return q-dptr;

	while (available() && fillBuffer()) {
		// make the buffer as large as it can be
		if ((q = STRCHR( dptr, c)) != (char*)0)
			return q-dptr;

		doubleBuffer();
	}
	return -1;
}

int
BufferedInputStream::read( char * buf, int len)
{
	register int clen   = 0;
	register int bigger = 0;
	register char * bptr = buf;
	int tally   = 0;

	do {
		if (!buflen)
			fillBuffer();
		if (!buflen)
			break;
		clen = buflen;
		if (clen > len)
			bigger++;

		MEMCPY ( bptr, dptr, (bigger ? len : clen));

		if (bigger) {
			buflen -= len;
			dptr   += (clen-len);

			return len;
		}
		dptr = (char*)0;
		buflen = 0;
		len   -= clen;
		bptr  += clen;
		tally += clen;

	} while (len > 0);

	return tally;
}


int
BufferedInputStream::skip( int len)
{
	register int clen   = 0;
	register int bigger = 0;
	int tally   = 0;

	do {
		if (!buflen) {
			fillBuffer();
		}
		if (!buflen)
			break;
		if (buflen > len) {
			buflen -= len;
			dptr   += len;
			return 0;
		}
		dptr   = (char*)0;
		len   -= buflen;
		buflen = 0;

	} while (len > 0);

	return 0;
}


void *
BufferedInputStream::readAll(int * sizep)
{
	char * block = (char*)0;
	int n;
	int blocklen = 0;

	*sizep = 0;

	while( 1) {
		if ((n = fillBuffer()) == -1) {
			break;
		}
		if (!n) {
			continue;
		}
		if (!block) {
			block    = (char*)data;
			blocklen = n;
			data     = (char*)0;
			dptr     = (char*)0;
			buflen   = 0;
		} else {
			block = (char*)REALLOC(block, blocklen+n+1);
			MEMCPY(block+blocklen, data, n);
			blocklen += n;
			dptr = (char*)0;
			buflen = 0;
		}
	}


	*sizep = blocklen;

	if (!block)
		return (void*)0;

	block[blocklen] = '\0';

	if (data) {
		if (data != block)
			FREE(data);
		data   = (char*)0;
		dptr   = (char*)0;
		buflen = 0;
	}

	return (void*)block;
}

char *
DataInputStream::readLine()
{
	int  ix  = indexOf('\n');
	char * p = currentPosition();

	if ( ix == -1)
		return (char*)0;

	p[ix]    = '\0';

	if (p[ix-1] == '\r')
		p[ix-1] = '\0';

	skip(ix+1);

	return p;
}

int
BufferedOutputStream::write( char * buf, int len)
{
	if (!data) {
		data   = new char[bufsize+1];
		dptr   = data;
		buflen = 0;
	}
	if (buflen + len > bufsize)
		flush();

	if (len > bufsize)
		return outputstream->write(buf, len);

	MEMCPY ( dptr, buf, len);

	buflen += len;
	dptr   += len;

	return len;
}

void
BufferedOutputStream::flush()
{
	if (!data)
		return;
	if (!buflen)
		return;
	(void) outputstream->write(data, buflen);
	dptr    = data;
	buflen  = 0;
}

void
TempFileOutputStream::copy( OutputStream * o)
{
	(void) lseek( id, 0, SEEK_SET);
	StdInputStream * i = new StdInputStream( id);
	InputData * idata = new InputData( i);

	o->write( (char*)(idata->getData()), idata->getSize());

	o->flush();

	delete idata;
	delete i;
}

void testStreams()
{
	FileInputStream * fs = FileInputStream::getFileInputStream((char *) "foopydoo");
	if (!fs) {
		printf("Cannot open stream\n");
		return;
	}
	DataInputStream * ds = new DataInputStream( fs);
	char * s;
	TempFileOutputStream * ts = TempFileOutputStream::getTempFileOutputStream();
	if (!ts) {
		printf("Cannot open tempstream\n");
		return;
	}

	BufferedOutputStream * bs = new BufferedOutputStream( ts);
	PrintWriter *           p = new PrintWriter( bs);

	while ((s = ds->readLine()) != (char*)0)
		p->println(s);

	delete p;
	delete bs;
	delete ds;
	delete fs;

	FileOutputStream * fos = FileOutputStream::getFileOutputStream((char *) "f2");
	if (!fos) {
		printf("Cannot open stream\n");
		return;
	}

	ts->copy( fos);

	delete ts;
	delete fos;
}

static char * sc_error_message = (char*)0;

void
setSCError( char * s)
{
	sc_error_message = s;
}

char *
getSCError()
{
	return sc_error_message;
}

#include <ctype.h>

static
int
startsWithDocType( char *buf, int len)
{
	char * testme = (char *) "<!DOCTYPE";
	char * p;
	if (len < 10)
		return 0;

	for (p = buf; testme && *testme; testme++, p++) {
		if (toupper(*p) != toupper(*testme))
			return 0;
	}
	return 1;
}

extern "C" { // for use with the C SGML parser engine

char *
scGetTypedDocument( InputStream *i, char *t)
{
	/* temporary file copy.... */
	static char ofile[64];
	BufferedInputStream * bi = new BufferedInputStream( i);
	int wid;
	int first = 1;

	(void) sprintf ( ofile, "/tmp/sgis%d._", getpid());
	ofile[strlen(ofile)+1] = '\0';

	wid = open( ofile, O_RDWR|O_CREAT|O_TRUNC, 0x744);
	if (wid == -1) {
		(void) fprintf ( stderr, "Cannot write %s\n", ofile);
		return (char*)0;
	}

	while (1)  {
		if (bi->fillBuffer() == -1)
			break;
		if (first) {
			if (!startsWithDocType(bi->dptr, bi->buflen))
				write(  wid, t, strlen(t));
			first = 0;
		}
		write( wid, bi->dptr, bi->buflen);
		bi->dptr = (char*)0;
		bi->buflen = 0;
	}

	close( wid);
	bi->close();
	delete bi;

	return ofile;
}

}
