/*
** A.J.Fountain,
** Imperial Software Technology (IST),
** Berkshire House,
** 252 Kings Road,
** Reading, 
** Berkshire,
** United Kingdom RG1 4HP.
**
** Telephone: +44 189 587055
** Fax:       +44 189 589005
** Email:     af@ist.co.uk
*/

/*
** sccsid = "@(#)ToolTipP.h	1.7" -- 02/02/07
*/

#ifndef   _XmToolTipP_h_
#define   _XmToolTipP_h_

#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <Xm/ToolTip.h>
#include <Xm/XmP.h>
#include <X11/ShellP.h>

/*
** ToolTip Class Structure
*/

typedef struct _XmToolTipClassPart
{
	XtPointer extension ;
} XmToolTipClassPart ;

/*
** Full ToolTip Class Record
*/

typedef struct _XmToolTipClassRec
{
	CoreClassPart          core_class ;
	CompositeClassPart     composite_class ;
	ShellClassPart         shell_class ;
#ifdef    TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL
	OverrideShellClassPart override_shell_class ;
#else  /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
	WMShellClassPart       wm_shell_class ;
	VendorShellClassPart   vendor_shell_class ;
#endif /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
	XmToolTipClassPart     tooltip_class ;
} XmToolTipClassRec ;

/*
** Tooltip Record Structure
*/

typedef struct XmToolTipRecord_s *XmToolTipRecord_p ;
typedef struct XmToolTipRecord_s
{
	Widget            tooltip ;
	Widget            target ;
	XmString          tip ;

#if       (XmVERSION > 1)
	XmRenderTable     font ;
#else  /* (XmVERSION > 1) */
	XmFontList        font ;
#endif /* (XmVERSION > 1) */

	Boolean           enabled ;
	XmToolTipRecord_p prev ;
	XmToolTipRecord_p next ;
} XmToolTipRecord_t ;

/*
** ToolTip Widget Instance Record
*/

typedef struct _XmToolTipPart
{
	/*
	** Public Widget Resources
	*/

	int               popup_delay ;
	int               popdown_delay ;
	Dimension         vertical_offset ;
	Dimension         horizontal_offset ;

	/*
	** Public Mirror Resources
	*/

	Pixel             foreground_pixel ;
	Pixel             top_shadow_pixel ;
	Pixmap            top_shadow_pixmap ;
	Pixel             bottom_shadow_pixel ;
	Pixmap            bottom_shadow_pixmap ;
	XmFontList        font_list ;
	unsigned char     string_direction ;

#if       (XmVERSION > 1)
	XmDirection       layout_direction ;
#endif /* (XmVERSION > 1) */

	/*
	** Public Convenience Resources 
	*/

	Widget            label ;

	/*
	** Private Widget Data
	*/

	Boolean           font_list_set ;

	XtIntervalId      popup_id ;
	XtIntervalId      popdown_id ;
	Boolean           popped_up ;
	XtAppContext      context ;

	/*
	** Set of registered targets
	*/

	XmToolTipRecord_p records ;
} XmToolTipPart ;

/*
** Full ToolTip Instance Record
*/

typedef struct _XmToolTipRec
{
	CorePart          core ;
	CompositePart     composite ;
	ShellPart         shell ;
	OverrideShellPart override ;
#ifdef    TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL
	OverrideShellPart override_shell ;
#else  /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
	WMShellPart       wm_shell ;
	VendorShellPart   vendor_shell ;
#endif /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
	XmToolTipPart     tooltip ;
} XmToolTipRec ;

externalref XmToolTipClassRec xmToolTipClassRec ;

/*
** Private Internal Interface
*/

#ifndef   _NO_PROTO
extern XmToolTipRecord_p _XmToolTipFindRecord(Widget, Widget) ;
extern XmToolTipRecord_p _XmToolTipAddRecord(Widget, Widget, XmString, XmFontList) ;
extern void              _XmToolTipRemoveRecord(Widget, Widget) ;
extern void              _XmToolTipRemoveAllRecords(Widget) ;
extern void              _XmToolTipSetEnabled(Widget, Widget, Boolean) ;
extern void              _XmToolTipSetTip(Widget, Widget, XmString, XmFontList) ;
#else  /* _NO_PROTO */
extern XmToolTipRecord_p _XmToolTipFindRecord() ;
extern XmToolTipRecord_p _XmToolTipAddRecord() ;
extern void              _XmToolTipRemoveRecord() ;
extern void              _XmToolTipRemoveAllRecords() ;
extern void              _XmToolTipSetEnabled() ;
extern void              _XmToolTipSetTip() ;
#endif /* _NO_PROTO */

#endif /* _XmToolTipP_h_ */
/* DON'T ADD ANYTHING AFTER THIS #endif */
