/*@ cc -g -c @I @F @L -o @R.o
*/

/*
** A.J.Fountain,
** Imperial Software Technology (IST),
** Berkshire House,
** 252 Kings Road,
** Reading, 
** Berkshire,
** United Kingdom RG1 4HP.
**
** Telephone: +44 189 587055
** Fax:       +44 189 589005
** Email:     af@ist.co.uk
*/

/*
** A very simple popup override shell with a label child,
** with some event handlers and timers to popup/down the whole.
** A small mirror is implemented to copy down font/rendition data.
*/

#ifndef   lint
static char *sccsid[] = {"@(#)ToolTip.c	1.22"} ; /* 02/02/07 */
#endif /* lint */

#include <X11/IntrinsicP.h> 
#include <X11/StringDefs.h>
#include <Xm/Label.h>
#include <Xm/Xm.h>
#include <Xm/XmP.h>

#include <Xm/ToolTipP.h>

#if       (XmVERSION == 1)
#define   XmNrenderTable              XmNfontList
#define   XmRenderTable               XmFontList
#endif /* (XmVERSION == 1) */

#ifndef   TOOLTIP_DEFAULT_FONT
#define   TOOLTIP_DEFAULT_FONT        "fixed"
#endif /* TOOLTIP_DEFAULT_FONT */

/*
** PR 9272:
**
** Offsets must be bounded to be much less than the size of the Screen.
** These are excessively large, but at least prevent complete layout destruction.
*/

#ifndef   MAX_VERTICAL_OFFSET
#define   MAX_VERTICAL_OFFSET         100
#endif /* MAX_VERTICAL_OFFSET */

#ifndef   MAX_HORIZONTAL_OFFSET
#define   MAX_HORIZONTAL_OFFSET       100
#endif /* MAX_HORIZONTAL_OFFSET */

/*
** Patches
**
** 1.1.1: 
**	PR 9721: Overlay left edge of screen
**	PR 9722: Bound checking on offsets
**	PR 9723: Initialize Mirror Resources
*/


/*
** External Declarations
*/

/*
** Defined in ColorI.h, but not all vendors ship this.
*/

#ifndef   _NO_PROTO
extern void _XmBackgroundColorDefault(Widget, int, XrmValue *) ;
#else  /* _NO_PROTO */
extern void _XmBackgroundColorDefault() ;
#endif /* _NO_PROTO */

/*
** Static Widget Class Methods
*/

#ifndef   _NO_PROTO
static void             ClassInitialize(void) ;
static void             Initialize(Widget, Widget, ArgList, Cardinal *) ;
static Boolean          SetValues(Widget, Widget, Widget, ArgList, Cardinal *) ;
static void             Destroy(Widget) ;
#else  /* _NO_PROTO */
static void             ClassInitialize() ;
static void             Initialize() ;
static Boolean          SetValues() ;
static void             Destroy() ;
#endif /* _NO_PROTO */

/*
** Private Methods
*/

#ifndef   _NO_PROTO
static void       PopupToolTip(XtPointer, XtIntervalId *) ;
static void       PopdownToolTip(XtPointer, XtIntervalId *) ;
static XmFontList GetDefaultFontList(Widget) ;
#if       (XmVERSION > 1)
static void       CheckRenderTable(Widget, int, XrmValue *) ;
#endif /* (XmVERSION > 1) */
#else  /* _NO_PROTO */
static void       PopupToolTip() ;
static void       PopdownToolTip() ;
static XmFontList GetDefaultFontList() ;
#if       (XmVERSION > 1)
static void       CheckRenderTable() ;
#endif /* (XmVERSION > 1) */
#endif /* _NO_PROTO */

/*
** Resource Table
*/

#define Offset(field) XtOffsetOf(XmToolTipRec, field)

#ifndef   XmUNSPECIFIED_PIXEL
#define   XmUNSPECIFIED_PIXEL   ((Pixel) (~0))
#endif /* XmUNSPECIFIED_PIXEL */

static XtResource resources[] =
{
	/*
	** ToolTip Specific Resources
	*/

	{
		XmNpopupDelay, XmCPopupDelay, XmRInt, 
		sizeof(int), Offset(tooltip.popup_delay),
		XmRImmediate, (XtPointer) 500
	},
	{
		XmNpopdownDelay, XmCPopdownDelay, XmRInt, 
		sizeof(int), Offset(tooltip.popdown_delay),
		XmRImmediate, (XtPointer) 0
	},
	{
		XmNverticalOffset, XmCVerticalOffset, XmRDimension,
		sizeof(Dimension), Offset(tooltip.vertical_offset),
		XmRImmediate, (XtPointer) 2
	},
	{
		XmNhorizontalOffset, XmCHorizontalOffset, XmRDimension,
		sizeof(Dimension), Offset(tooltip.horizontal_offset),
		XmRImmediate, (XtPointer) 2
	},
	{
		XmNlabelWidget, XmCLabelWidget, XmRWidget,
		sizeof(Widget), Offset(tooltip.label),
		XmRImmediate, (XtPointer) 0
	},

	/*
	** Mirrored Resources
	**
	** Problem: some Motif 2.1.0 have the _Xm*ColorDefault routines in XmP.h,
	** some platforms have only foreground and select color routines,
	** yet more define them in the private ColorI.h, others not at all.
	**
	** The reference 2.1.30 has them all in XmP.h, but operating
	** system vendors tend to mangle this to suit their desktop.
	*/

	{	/* Core Override */
		XmNbackground, XmCBackground, XmRPixel,
		sizeof (Pixel), Offset(core.background_pixel),
#if       (XmVERSION > 1)
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXEL
#else  /* (XmVERSION > 1) */
		XmRCallProc, (XtPointer) _XmBackgroundColorDefault
#endif /* (XmVERSION > 1) */
	},
	{	/* Core Override */
		XmNborderColor, XmCBorderColor, XmRPixel,
		sizeof (Pixel), Offset(core.border_pixel),
#if       (XmVERSION > 1)
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXEL
#else  /* (XmVERSION > 1) */
		XmRCallProc, (XtPointer) _XmBackgroundColorDefault
#endif /* (XmVERSION > 1) */
	},
	{	/* Mirrored */
		XmNbottomShadowColor, XmCBottomShadowColor, XmRPixel,
		sizeof (Pixel), Offset(tooltip.bottom_shadow_pixel),
#if       (XmVERSION > 1)
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXEL
#else  /* (XmVERSION > 1) */
		XmRImmediate, (XtPointer) _XmBottomShadowColorDefault
#endif /* (XmVERSION > 1) */
	},
	{	/* Mirrored */
		XmNbottomShadowPixmap, XmCBottomShadowPixmap, XmRPrimBottomShadowPixmap,
		sizeof (Pixmap), Offset(tooltip.bottom_shadow_pixmap),
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXMAP
	},
	{	/* Mirrored */
		XmNfontList, XmCFontList, XmRFontList,
		sizeof(XmFontList), Offset(tooltip.font_list),
#if       (XmVERSION > 1)
		XmRCallProc, (XtPointer) CheckRenderTable
#else  /* (XmVERSION > 1) */
		XmRImmediate, (XtPointer) NULL
#endif /* (XmVERSION > 1) */
	},
	{	/* Mirrored */
		XmNforeground, XmCForeground, XmRPixel,
		sizeof (Pixel), Offset(tooltip.foreground_pixel),
#if       (XmVERSION > 1)
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXEL
#else  /* (XmVERSION > 1) */
		XmRImmediate, (XtPointer) _XmForegroundColorDefault
#endif /* (XmVERSION > 1) */
	},
#if       (XmVERSION > 1)
	{	/* Mirrored */
		XmNlayoutDirection, XmCLayoutDirection, XmRDirection,
		sizeof (XmDirection), Offset(tooltip.layout_direction),
		XmRImmediate, (XtPointer) XmDEFAULT_DIRECTION
	},
#endif /* (XmVERSION > 1) */
#if       (XmVERSION > 1)
	{
		"pri.vate", "Pri.vate", XmRBoolean,
		sizeof(Boolean), Offset(tooltip.font_list_set),
		XmRImmediate, (XtPointer) FALSE
	},
	{	/* Mirrored */
		XmNrenderTable, XmCRenderTable, XmRRenderTable,
		sizeof(XmRenderTable), Offset(tooltip.font_list),
		XmRCallProc, (XtPointer) CheckRenderTable
	},
	{	/* Mirrored */
		XmNstringDirection, XmCStringDirection, XmRStringDirection,
		sizeof (unsigned char), Offset(tooltip.string_direction),
		XmRImmediate, (XtPointer) XmDEFAULT_DIRECTION
	},
#endif /* (XmVERSION > 1) */
	{	/* Mirrored */
		XmNtopShadowColor, XmCTopShadowColor, XmRPixel,
		sizeof (Pixel), Offset(tooltip.top_shadow_pixel),
#if       (XmVERSION > 1)
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXEL
#else  /* (XmVERSION > 1) */
		XmRImmediate, (XtPointer) _XmTopShadowColorDefault
#endif /* (XmVERSION > 1) */
	},
	{	/* Mirrored */
		XmNtopShadowPixmap, XmCTopShadowPixmap, XmRPrimTopShadowPixmap,
		sizeof (Pixmap), Offset(tooltip.top_shadow_pixmap),
		XmRImmediate, (XtPointer) XmUNSPECIFIED_PIXMAP
	},

	/*
	** Core Overrides
	*/

	{	
		XmNborderWidth, XmCBorderWidth, XmRDimension,
		sizeof (Dimension), Offset(core.border_width),   /* rectangle.border_width */
		XmRImmediate, (XtPointer) 1
	},

	/*
	** Shell Overrides
	*/

	{
		XmNallowShellResize, XmCAllowShellResize, XmRBoolean,
		sizeof(Boolean), Offset(shell.allow_shell_resize),
		XtRImmediate, (XtPointer) TRUE
	},
	{
		XmNoverrideRedirect, XmCOverrideRedirect, XmRBoolean,
		sizeof(Boolean), Offset(shell.override_redirect),
		XtRImmediate, (XtPointer) TRUE
	},
	{
		XmNsaveUnder, XmCSaveUnder, XmRBoolean,
		sizeof(Boolean), Offset(shell.save_under),
		XtRImmediate, (XtPointer) FALSE
	}
} ;

#undef Offset

/*
** Actions (None)
*/

/*
** Translations (None)
*/

/*
** Full Widget Class Record
*/

XmToolTipClassRec xmToolTipClassRec = 
{
	/* Core fields */
	{
#ifdef    TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL
		(WidgetClass) &overrideShellClassRec,	/* superclass		*/
#else  /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
		(WidgetClass) &vendorShellClassRec,	/* superclass		*/
#endif /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
		"XmToolTip",				/* class_name		*/
		sizeof(XmToolTipRec),			/* widget size		*/
		ClassInitialize, 			/* class_init		*/
		NULL,					/* class_part_init	*/
		FALSE,					/* class_inited		*/
		Initialize,				/* initialize		*/
		NULL,					/* init_hook		*/
		XtInheritRealize,			/* realize		*/
		NULL,					/* actions		*/
		0,					/* num_actions		*/
		(XtResourceList) resources,		/* resources		*/
		XtNumber(resources),			/* num_resources	*/
		NULLQUARK,				/* xrm_class		*/
		TRUE,					/* compress_motion	*/
		(XtEnum) False,				/* compress_exposure	*/
		TRUE,					/* compress enterleave	*/
		FALSE,					/* visibility_interest	*/
		Destroy,				/* destroy		*/
		XtInheritResize,			/* resize		*/
		XtInheritExpose,			/* expose		*/
		SetValues,				/* set_values		*/
		(XtArgsFunc) 0,				/* set_values_hook	*/
		XtInheritSetValuesAlmost,		/* set_values_almost	*/
		0,					/* get_values_hook	*/
		XtInheritAcceptFocus,			/* accept_focus		*/
		XtVersion,				/* version		*/
		NULL,					/* callback_private	*/
		XtInheritTranslations,			/* tm_table		*/
		XtInheritQueryGeometry,			/* query_geometry	*/
		XtInheritDisplayAccelerator,		/* display_accelerator	*/
		NULL					/* extension		*/
	},
	/* Composite fields */
	{
		XtInheritGeometryManager,		/* geometry_manager   	*/    	
		XtInheritChangeManaged,			/* change_managed	*/	
		XtInheritInsertChild,			/* insert_child		*/	
		XtInheritDeleteChild,			/* delete_child		*/	
		NULL					/* extension		*/	
	}, 
	/* Shell fields */
	{
		NULL					/* extension		*/	
	},
#ifdef    TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL
	/* Override shell fields */
	{
		NULL					/* extension		*/	
	},
#else  /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
	/* WM shell fields */
	{
		NULL					/* extension		*/	
	},
	/* Vendor shell fields */
	{
		NULL					/* extension		*/	
	},
#endif /* TOOLTIP_INHERITS_FROM_OVERRIDE_SHELL */
	/* Tooltip fields */
	{
		NULL					/* extension		*/	
	}
} ;

externaldef(xmtooltipwidgetclass) WidgetClass xmToolTipWidgetClass = (WidgetClass) &xmToolTipClassRec ;

/*
** Widget Class Methods
*/

#ifndef   _NO_PROTO
static void ClassInitialize(void)
#else  /* _NO_PROTO */
static void ClassInitialize()
#endif /* _NO_PROTO */
{
	/*
	** Safety. If Motif has already been
	** initialized, these are harmless.
	*/

	_XmRegisterConverters() ;
	_XmRegisterPixmapConverters() ;
}

/* ARGSUSED */
#ifndef   _NO_PROTO
static void Initialize(Widget request, Widget new_widget, ArgList argv, Cardinal *argc)
#else  /* _NO_PROTO */
static void Initialize(request, new_widget, argv, argc)
	Widget    request ;
	Widget    new_widget ;
	ArgList   argv ;
	Cardinal *argc ;
#endif /* _NO_PROTO */
{
	XmToolTipWidget tooltip    = (XmToolTipWidget) new_widget ;
	XmString        xms        = (XmString) 0 ;
	Cardinal        label_argc = (Cardinal) 0 ;
	Arg             label_argv[20] ;

	/*
	** Initialize Instance Variables
	*/

	tooltip->core.x      = 0 ;
	tooltip->core.y      = 0 ;
	tooltip->core.width  = 1 ;
	tooltip->core.height = 1 ;
	
	tooltip->tooltip.popup_id   = (XtIntervalId) 0 ;
	tooltip->tooltip.popdown_id = (XtIntervalId) 0 ;
	tooltip->tooltip.popped_up  = False ;
	tooltip->tooltip.context    = XtWidgetToApplicationContext(XtParent(request)) ;
	tooltip->tooltip.records    = (XmToolTipRecord_p) 0 ;

	/*
	** Create Internal Widget
	*/

	if (tooltip->tooltip.font_list == (XmFontList) 0) {
		tooltip->tooltip.font_list = GetDefaultFontList(new_widget) ;
		
		if (tooltip->tooltip.font_list != (XmFontList) 0) {
			tooltip->tooltip.font_list = XmFontListCopy(tooltip->tooltip.font_list) ;
		}
	}

	if (tooltip->tooltip.font_list != (XmFontList) 0) {
		XtSetArg(label_argv[label_argc], XmNrenderTable, tooltip->tooltip.font_list) ; label_argc++ ;
	}

	/*
	** Hard code a few basics, and a default label
	*/
		
	xms = XmStringCreateLocalized("") ;

	XtSetArg(label_argv[label_argc], XmNlabelType,     XmSTRING) ; label_argc++ ;
	XtSetArg(label_argv[label_argc], XmNlabelString,   xms) ;      label_argc++ ;
	XtSetArg(label_argv[label_argc], XmNrecomputeSize, True) ;     label_argc++ ;

	/*
	** Copy down Specified Mirror Resources
	*/

	if (tooltip->core.background_pixel != XmUNSPECIFIED_PIXEL) {
		XtSetArg(label_argv[label_argc], XmNbackground, tooltip->core.background_pixel) ; label_argc++ ;
	}
	
	if (tooltip->tooltip.foreground_pixel != XmUNSPECIFIED_PIXEL) {
		XtSetArg(label_argv[label_argc], XmNforeground, tooltip->tooltip.foreground_pixel) ; label_argc++ ;
	}
	
	if (tooltip->core.border_pixel != XmUNSPECIFIED_PIXEL) {
		XtSetArg(label_argv[label_argc], XmNborderColor, tooltip->core.border_pixel) ; label_argc++ ;
	}
	
	if (tooltip->tooltip.bottom_shadow_pixel != XmUNSPECIFIED_PIXEL) {
		XtSetArg(label_argv[label_argc], XmNbottomShadowColor, tooltip->tooltip.bottom_shadow_pixel) ; label_argc++ ;
	}
	
	if (tooltip->tooltip.top_shadow_pixel != XmUNSPECIFIED_PIXEL) {
		XtSetArg(label_argv[label_argc], XmNtopShadowColor, tooltip->tooltip.top_shadow_pixel) ; label_argc++ ;
	}
	
	if (tooltip->core.background_pixmap != XmUNSPECIFIED_PIXMAP) {
		XtSetArg(label_argv[label_argc], XmNbackgroundPixmap, tooltip->core.background_pixmap) ; label_argc++ ;
	}
	
	if (tooltip->core.border_pixmap != XmUNSPECIFIED_PIXMAP) {
		XtSetArg(label_argv[label_argc], XmNborderPixmap, tooltip->core.border_pixmap) ; label_argc++ ;
	}
	
	if (tooltip->tooltip.bottom_shadow_pixmap != XmUNSPECIFIED_PIXMAP) {
		XtSetArg(label_argv[label_argc], XmNbottomShadowPixmap, tooltip->tooltip.bottom_shadow_pixmap) ; label_argc++ ;
	}
	
	if (tooltip->tooltip.top_shadow_pixmap != XmUNSPECIFIED_PIXMAP) {
		XtSetArg(label_argv[label_argc], XmNtopShadowPixmap, tooltip->tooltip.top_shadow_pixmap) ; label_argc++ ;
	}
	
	if (tooltip->tooltip.font_list != (XmFontList) 0) {
		XtSetArg(label_argv[label_argc], XmNrenderTable, tooltip->tooltip.font_list) ; label_argc++ ;
	}

#if       (XmVERSION > 1)
	if (tooltip->tooltip.layout_direction != XmDEFAULT_DIRECTION) {
		XtSetArg(label_argv[label_argc], XmNlayoutDirection, tooltip->tooltip.layout_direction) ; label_argc++ ;
	}
#endif /* (XmVERSION > 1) */
	
	if (tooltip->tooltip.string_direction != XmDEFAULT_DIRECTION) {
		XtSetArg(label_argv[label_argc], XmNstringDirection, tooltip->tooltip.string_direction) ; label_argc++ ;
	}

	tooltip->tooltip.label = XmCreateLabel(new_widget, "tooltip", label_argv, label_argc) ;

	XmStringFree(xms) ;

	XtManageChild(tooltip->tooltip.label) ;
}

/* ARGSUSED */
#ifndef   _NO_PROTO
static Boolean SetValues(Widget current, Widget request, Widget new_widget, ArgList argv, Cardinal *argc)
#else  /* _NO_PROTO */
static Boolean SetValues(current, request, new_widget, argv, argc)
	Widget    current ;
	Widget    request ;
	Widget    new_widget ;
	ArgList   argv ;
	Cardinal *argc ;
#endif /* _NO_PROTO */
{
	XmToolTipWidget new_tooltip  = (XmToolTipWidget) new_widget ;
	XmToolTipWidget old_tooltip  = (XmToolTipWidget) current ;
	XmFontList      old_fontList = (XmFontList) 0;
	Cardinal        label_argc   = (Cardinal) 0 ;
	Arg             label_argv[20] ;

	/*
	** Prevent Read-only Resource Modification
	**
	** Note that no warnings are issued: this is an override popup so
	** any interactive windowed warnings would be out of place.
	*/

	if (new_tooltip->tooltip.label != old_tooltip->tooltip.label) {
		new_tooltip->tooltip.label = old_tooltip->tooltip.label ;
	}

	/*
	** Sanity Checking. PR 9272: upper bound limitations
	** otherwise the widget has unreasonable layout behavior.
	*/

	if (new_tooltip->tooltip.vertical_offset != old_tooltip->tooltip.vertical_offset) {
		if (new_tooltip->tooltip.vertical_offset > MAX_VERTICAL_OFFSET) {
			new_tooltip->tooltip.vertical_offset = MAX_VERTICAL_OFFSET ;
		}
	}

	if (new_tooltip->tooltip.horizontal_offset != old_tooltip->tooltip.horizontal_offset) {
		if (new_tooltip->tooltip.horizontal_offset > MAX_HORIZONTAL_OFFSET) {
			new_tooltip->tooltip.horizontal_offset = MAX_HORIZONTAL_OFFSET ;
		}
	}

	/*
	** The Mirror
	*/
	
	if ((new_tooltip->core.background_pixel != old_tooltip->core.background_pixel) &&
	    (new_tooltip->core.background_pixel != XmUNSPECIFIED_PIXEL)) {
		XtSetArg(label_argv[label_argc], XmNbackground, new_tooltip->core.background_pixel) ; label_argc++ ;
	}
	else if (new_tooltip->core.background_pixel == XmUNSPECIFIED_PIXEL) {
		new_tooltip->core.background_pixel = WhitePixelOfScreen(XtScreen(new_widget)) ;
		
		XtSetArg(label_argv[label_argc], XmNbackground, new_tooltip->core.background_pixel) ; label_argc++ ;
	}

	if (new_tooltip->core.background_pixmap != old_tooltip->core.background_pixmap) {
		XtSetArg(label_argv[label_argc], XmNbackgroundPixmap, new_tooltip->core.background_pixmap) ; label_argc++ ;
	}
	
	if ((new_tooltip->core.border_pixel != old_tooltip->core.border_pixel) &&
	    (new_tooltip->core.border_pixel != XmUNSPECIFIED_PIXEL)) {
		XtSetArg(label_argv[label_argc], XmNborderColor, new_tooltip->core.border_pixel) ; label_argc++ ;
	}
	else if (new_tooltip->core.border_pixel == XmUNSPECIFIED_PIXEL) {
		new_tooltip->core.border_pixel = BlackPixelOfScreen(XtScreen(new_widget)) ;

		XtSetArg(label_argv[label_argc], XmNborderColor, new_tooltip->core.border_pixel) ; label_argc++ ;
	}
	
	if (new_tooltip->core.border_pixmap != old_tooltip->core.border_pixmap) {
		XtSetArg(label_argv[label_argc], XmNborderPixmap, new_tooltip->core.border_pixmap) ; label_argc++ ;
	}
	
	if ((new_tooltip->tooltip.bottom_shadow_pixel != old_tooltip->tooltip.bottom_shadow_pixel) &&
	    (new_tooltip->tooltip.bottom_shadow_pixel != XmUNSPECIFIED_PIXEL)) {
		XtSetArg(label_argv[label_argc], XmNbottomShadowColor, new_tooltip->tooltip.bottom_shadow_pixel) ; label_argc++ ;
	}
	
	if (new_tooltip->tooltip.bottom_shadow_pixmap != old_tooltip->tooltip.bottom_shadow_pixmap) {
		XtSetArg(label_argv[label_argc], XmNbottomShadowPixmap, new_tooltip->tooltip.bottom_shadow_pixmap) ; label_argc++ ;
	}
	
	if (new_tooltip->tooltip.font_list != old_tooltip->tooltip.font_list) {
		if (new_tooltip->tooltip.font_list == (XmFontList) 0) {
			new_tooltip->tooltip.font_list = GetDefaultFontList(new_widget) ;
		}

		if (new_tooltip->tooltip.font_list != (XmFontList) 0) {
			new_tooltip->tooltip.font_list = XmFontListCopy(new_tooltip->tooltip.font_list) ;
		}
	}

	if (new_tooltip->tooltip.font_list != (XmFontList) 0) {
		XtSetArg(label_argv[label_argc], XmNrenderTable, new_tooltip->tooltip.font_list) ; label_argc++ ;

		old_fontList = old_tooltip->tooltip.font_list ;
	}
	
	if ((new_tooltip->tooltip.foreground_pixel != old_tooltip->tooltip.foreground_pixel) &&
	    (new_tooltip->tooltip.foreground_pixel != XmUNSPECIFIED_PIXEL)) {
		XtSetArg(label_argv[label_argc], XmNforeground, new_tooltip->tooltip.foreground_pixel) ; label_argc++ ;
	}
	else if (new_tooltip->tooltip.foreground_pixel == XmUNSPECIFIED_PIXEL) {
		new_tooltip->tooltip.foreground_pixel = BlackPixelOfScreen(XtScreen(new_widget)) ;
		
		XtSetArg(label_argv[label_argc], XmNforeground, new_tooltip->tooltip.foreground_pixel) ; label_argc++ ;
	}
	
#if       (XmVERSION > 1)
	if (new_tooltip->tooltip.layout_direction != old_tooltip->tooltip.layout_direction) {
		XtSetArg(label_argv[label_argc], XmNlayoutDirection, new_tooltip->tooltip.layout_direction) ; label_argc++ ;
	}
#endif /* (XmVERSION > 1) */
	
	if (new_tooltip->tooltip.string_direction != old_tooltip->tooltip.string_direction) {
		XtSetArg(label_argv[label_argc], XmNstringDirection, new_tooltip->tooltip.string_direction) ; label_argc++ ;
	}

	if ((new_tooltip->tooltip.top_shadow_pixel != old_tooltip->tooltip.top_shadow_pixel) &&
	    (new_tooltip->tooltip.top_shadow_pixel != XmUNSPECIFIED_PIXEL)) {
		XtSetArg(label_argv[label_argc], XmNtopShadowColor, new_tooltip->tooltip.top_shadow_pixel) ; label_argc++ ;
	}
	
	if (new_tooltip->tooltip.top_shadow_pixmap != old_tooltip->tooltip.top_shadow_pixmap) {
		XtSetArg(label_argv[label_argc], XmNtopShadowPixmap, new_tooltip->tooltip.top_shadow_pixmap) ; label_argc++ ;
	}

	/*
	** Apply the Collected Mirror Resources 
	*/

	if (label_argc > 0) {
		XtSetValues(new_tooltip->tooltip.label, label_argv, label_argc) ;
	}

	/*
	** Free up
	*/

	if (old_fontList != (XmFontList) 0) {
		XmFontListFree(old_fontList) ;
	}

	return False ;
}

#ifndef   _NO_PROTO
static void Destroy(Widget w) 
#else  /* _NO_PROTO */
static void Destroy(w)
	Widget w ;
#endif /* _NO_PROTO */
{
	XmToolTipWidget tooltip = (XmToolTipWidget) w ;
		
	if (tooltip->tooltip.popup_id != (XtIntervalId) 0) {
		XtRemoveTimeOut(tooltip->tooltip.popup_id) ;
	}
	
	if (tooltip->tooltip.popdown_id != (XtIntervalId) 0) {
		XtRemoveTimeOut(tooltip->tooltip.popdown_id) ;
	}

	if (tooltip->tooltip.font_list != (XmFontList) 0) {
		XmFontListFree(tooltip->tooltip.font_list) ;
	}
	
	_XmToolTipRemoveAllRecords(w) ;
}

/*
** Private Methods
*/

#ifndef   _NO_PROTO
static XmFontList GetDefaultFontList(Widget w)
#else  /* _NO_PROTO */
static XmFontList GetDefaultFontList(w)
	Widget w ;
#endif /* _NO_PROTO */
{
	XrmValue from ;
	XrmValue to ;

	from.addr = (XtPointer) TOOLTIP_DEFAULT_FONT ;
	from.size = strlen(TOOLTIP_DEFAULT_FONT) + 1 ;
	to.addr   = (XtPointer) 0 ;

	XtConvertAndStore(w, XmRString, &from, XmRFontList, &to) ;

	if (to.addr != (XtPointer) 0) {
		return *((XmFontList *) to.addr) ;
	}
		
	return (XmFontList) 0 ;
}

#if       (XmVERSION > 1)
/* ARGSUSED */
#ifndef   _NO_PROTO
static void CheckRenderTable(Widget w, int offset, XrmValue *value)
#else  /* _NO_PROTO */
static void CheckRenderTable(w, offset, value)
	Widget    w ;
	int       offset ;
	XrmValue *value ;
#endif /* _NO_PROTO */ 
{
	XmToolTipWidget tooltip = (XmToolTipWidget) w ;
	
	if (tooltip->tooltip.font_list_set) {
		value->addr = (XtPointer) 0 ;
	}
	else {
		tooltip->tooltip.font_list_set = True ;
		
		value->addr = (XtPointer) &(tooltip->tooltip.font_list) ;
	}
}
#endif /* (XmVERSION > 1) */

#ifndef   _NO_PROTO
static void PopupToolTip(XtPointer client_data, XtIntervalId *id)
#else  /* _NO_PROTO */
static void PopupToolTip(client_data, id)
	XtPointer     client_data ;
	XtIntervalId *id ;
#endif /* _NO_PROTO */
{
	Widget          w       = (Widget)          client_data ;
	XmToolTipWidget tooltip = (XmToolTipWidget) w ;

	if (id == (XtIntervalId *) 0) {
		/* Programmatic Popup */
		
		if (tooltip->tooltip.popdown_id != (XtIntervalId) 0) {
			XtRemoveTimeOut(tooltip->tooltip.popdown_id) ;

			tooltip->tooltip.popdown_id = (XtIntervalId) 0 ;
		}
	}

	if (tooltip->tooltip.popup_id == (XtIntervalId) 0) {
		if (tooltip->tooltip.popup_delay > 0) {
			tooltip->tooltip.popup_id = XtAppAddTimeOut(tooltip->tooltip.context,
								    (unsigned long) tooltip->tooltip.popup_delay,
								    PopupToolTip,
								    (XtPointer) tooltip) ;
		}
	}
		
	if ((id != (XtIntervalId *) 0) || (tooltip->tooltip.popup_delay == 0)) {
		/* 
		** Either immediate programmatic popup, or delay timer has fired
		*/
	
		XtPopup((Widget) tooltip, XtGrabNone) ;

		tooltip->tooltip.popup_id  = (XtIntervalId) 0 ;
		tooltip->tooltip.popped_up = True ;
	}
}

#ifndef   _NO_PROTO
static void PopdownToolTip(XtPointer client_data, XtIntervalId *id)
#else  /* _NO_PROTO */
static void PopdownToolTip(client_data, id)
	XtPointer     client_data ;
	XtIntervalId *id ;
#endif /* _NO_PROTO */
{
	Widget          w       = (Widget)          client_data ;
	XmToolTipWidget tooltip = (XmToolTipWidget) w ;
	
	if (id == (XtIntervalId *) 0) {
		/* Programmatic Popdown */

		if (tooltip->tooltip.popup_id != (XtIntervalId) 0) {
			XtRemoveTimeOut(tooltip->tooltip.popup_id) ;

			tooltip->tooltip.popup_id = (XtIntervalId) 0 ;
		}
	}
		
	if (tooltip->tooltip.popdown_id == (XtIntervalId) 0) {
		if (tooltip->tooltip.popdown_delay > 0) {
			tooltip->tooltip.popdown_id = XtAppAddTimeOut(tooltip->tooltip.context,
								     (unsigned long) tooltip->tooltip.popdown_delay,
								     PopdownToolTip,
								     (XtPointer) tooltip) ;
		}
	}
	
	if ((id != (XtIntervalId *) 0) || (tooltip->tooltip.popdown_delay == 0)) {
		/* 
		** Either immediate programmatic popdown, or delay timer has fired
		*/

		XtPopdown((Widget) tooltip) ;

		tooltip->tooltip.popdown_id = (XtIntervalId) 0 ;
		tooltip->tooltip.popped_up  = False ;
	}
}

/*
** Private Interface for Registered ToolTip Objects
*/

/* ARGSUSED */
#ifndef   _NO_PROTO
static void EnterTargetHandler(Widget target, XtPointer client_data, XEvent *event, Boolean *unused) 
#else  /* _NO_PROTO */
static void EnterTargetHandler(target, client_data, event, unused) 
	Widget    target ;
	XtPointer client_data ;
	XEvent   *event ;
	Boolean  *unused ;
#endif /* _NO_PROTO */
{
	Position          root_x = (Position) 0 ;
	Position          root_y = (Position) 0 ;
	Position          tool_x = (Position) 0 ;
	Position          tool_y = (Position) 0 ;
	Screen           *screen = (Screen *) 0 ;
	XmToolTipWidget   tooltip ;

	XmToolTipRecord_p tp = (XmToolTipRecord_p) client_data ;
		
	tooltip = (XmToolTipWidget) tp->tooltip ;

	if (tooltip->tooltip.popdown_id != (XtIntervalId) 0) {
		XtRemoveTimeOut(tooltip->tooltip.popdown_id) ;

		tooltip->tooltip.popdown_id = (XtIntervalId) 0 ;
	}

	if ((tp->enabled == False) || (tp->tip == (XmString) 0)) {
		return ;
	}

	if (XtIsSensitive(target) && !(target->core.being_destroyed)) {
		/*
		** Work out the bottom left corner of the target
		*/

		XtTranslateCoords(target, (Position) 0, (Position) target->core.height, &root_x, &root_y) ;

		/*
		** Set the new help tip label string and font
		*/

		XtVaSetValues(tooltip->tooltip.label, 
			      XmNlabelString,  tp->tip, 
			      XmNrenderTable,  tp->font,
			      NULL) ;

		/*
		** Move the Tooltip under the target, allowing for some
		** minor screen overflow considerations. 
		**
		** This does not take into perfect account occlusion of the target,
		** althouugh it will try and place on top if possible when the
		** target is low on the screen. This goes wrong where the width/height
		** of the target plus tooltip offset exceeds the screen width/height.
		*/

		tool_x = root_x + tooltip->tooltip.horizontal_offset ;
		tool_y = root_y + tooltip->tooltip.vertical_offset ;

		screen = XtScreen((Widget) tooltip) ;

		if ((tool_x + tooltip->core.width) >= screen->width) {
			int pos = screen->width - tooltip->core.width - tooltip->tooltip.horizontal_offset ;

			if (pos < 0) {
				pos = tooltip->tooltip.horizontal_offset ;
			}

			tool_x = (Position) pos ;
		}
				
		if ((tool_y + tooltip->core.height) >= screen->height) {
			/*
			** Try and position above the target.
			*/

			int pos = root_y - target->core.height - tooltip->core.height - tooltip->tooltip.vertical_offset ;

			if (pos < 0) {
				pos = tooltip->tooltip.vertical_offset ;
			}
				
			tool_y = (Position) pos ;
		}

		/*
		** PR 9271.
		**
		** Place a target which is half overlapping the left/top edges,
		** and the tooltip popup is truncated.
		*/

		if (tool_x < 0) {
			tool_x = tooltip->tooltip.horizontal_offset ;
		}
		
		if (tool_y < 0) {
			tool_y = tooltip->tooltip.vertical_offset ;
		}

		XtMoveWidget((Widget) tp->tooltip, tool_x, tool_y) ;

		/*
		** And pop it up (possibly delayed by timer).
		*/

		PopupToolTip((XtPointer) tp->tooltip, (XtIntervalId *) 0) ;
	}
}

/* ARGSUSED */
#ifndef   _NO_PROTO
static void LeaveTargetHandler(Widget target, XtPointer client_data, XEvent *event, Boolean *unused) 
#else  /* _NO_PROTO */
static void LeaveTargetHandler(target, client_data, event, unused) 
	Widget    target ;
	XtPointer client_data ;
	XEvent   *event ;
	Boolean  *unused ;
#endif /* _NO_PROTO */
{
	XmToolTipRecord_p tp = (XmToolTipRecord_p) client_data ;

	PopdownToolTip((XtPointer) tp->tooltip, (XtIntervalId *) 0) ;
}

#ifndef   _NO_PROTO
static void _XmToolTipDeleteRecord(XmToolTipRecord_p) ;
#else  /* _NO_PROTO */
static void _XmToolTipDeleteRecord() ;
#endif /* _NO_PROTO */

/* ARGSUSED */
#ifndef   _NO_PROTO
static void DestroyTargetCallback(Widget target, XtPointer client_data, XtPointer call_data)
#else  /* _NO_PROTO */
static void DestroyTargetCallback(target, client_data, call_data)
	Widget    target ;
	XtPointer client_data ;
	XtPointer call_data ;
#endif /* _NO_PROTO */
{
	XmToolTipRecord_p tp = (XmToolTipRecord_p) client_data ;

	_XmToolTipDeleteRecord(tp) ;
}

#ifndef   _NO_PROTO
static void _XmToolTipDeleteRecord(XmToolTipRecord_p tp)
#else  /* _NO_PROTO */
static void _XmToolTipDeleteRecord(tp)
	XmToolTipRecord_p tp ;
#endif /* _NO_PROTO */
{
	XmToolTipWidget tooltip = (XmToolTipWidget) 0 ;

	if (tp != (XmToolTipRecord_p) 0) {
		tooltip = (XmToolTipWidget) tp->tooltip ;

		if (tp->target != (Widget) 0) {
			XtRemoveEventHandler(tp->target,
					     EnterWindowMask,
					     False,
					     EnterTargetHandler,
					     (XtPointer) tp) ;
			
			XtRemoveEventHandler(tp->target,
					     LeaveWindowMask | ButtonPressMask,
					     False,
					     LeaveTargetHandler,
					     (XtPointer) tp) ;

			XtRemoveCallback(tp->target,
					 XmNdestroyCallback,
					 DestroyTargetCallback,
					 (XtPointer) tp) ;
		}

		if (tp->prev != (XmToolTipRecord_p) 0) {
			tp->prev->next = tp->next ;
		}
			
		if (tp->next != (XmToolTipRecord_p) 0) {
			tp->next->prev = tp->prev ;
		}

		if (tp == tooltip->tooltip.records) {
			tooltip->tooltip.records = tp->next ;
		}

		if (tp->tip != (XmString) 0) {
			XmStringFree(tp->tip) ;
		}

		if (tp->font != (XmRenderTable) 0) {
			XmFontListFree(tp->font) ;
		}

		XtFree((char *) tp) ;
	}
}

#ifndef   _NO_PROTO
XmToolTipRecord_p _XmToolTipFindRecord(Widget tooltip, Widget target)
#else  /* _NO_PROTO */
XmToolTipRecord_p _XmToolTipFindRecord(tooltip, target)
	Widget tooltip ;
	Widget target ;
#endif /* _NO_PROTO */
{
	XmToolTipWidget   tip = (XmToolTipWidget) tooltip ;
	XmToolTipRecord_p rp  = (tip ? tip->tooltip.records : (XmToolTipRecord_p) 0) ;

	while (rp != (XmToolTipRecord_p) 0) {
		if (rp->tooltip == tooltip) {
			if (target != (Widget) 0) {
				if (rp->target == target) {
					return rp ;
				}
			}
		}

		rp = rp->next ;
	}

	return (XmToolTipRecord_p) 0 ;
}

#ifndef   _NO_PROTO
XmToolTipRecord_p _XmToolTipAddRecord(Widget tooltip, Widget target, XmString tip, XmFontList font)
#else  /* _NO_PROTO */
XmToolTipRecord_p _XmToolTipAddRecord(tooltip, target, tip, font)
	Widget     tooltip ;
	Widget     target ;
	XmString   tip ;
	XmFontList font ;
#endif /* _NO_PROTO */
{
	XmToolTipWidget   tp = (XmToolTipWidget) tooltip ;
	XmToolTipRecord_p rp = _XmToolTipFindRecord(tooltip, target) ;

	if (rp == (XmToolTipRecord_p) 0) {
		rp = (XmToolTipRecord_p) XtMalloc((unsigned) sizeof(XmToolTipRecord_t)) ;

		rp->tooltip = tooltip ;
		rp->target  = target ;
		rp->enabled = True ;

		if (tip != (XmString) 0) {
			rp->tip = XmStringCopy(tip) ;
		}
		else {
			rp->tip = (XmString) 0 ;
		}

		if (font != (XmRenderTable) 0) {
			rp->font = XmFontListCopy(font) ;
		}
		else {
			rp->font = XmFontListCopy(tp->tooltip.font_list) ;
		}

		rp->prev = (XmToolTipRecord_p) 0 ;
		rp->next = tp->tooltip.records ;

		if (tp->tooltip.records != (XmToolTipRecord_p) 0) {
			(tp->tooltip.records)->prev = rp ;
		}

		tp->tooltip.records = rp ;
			
		XtAddEventHandler(rp->target,
				  EnterWindowMask,
				  False,
				  EnterTargetHandler,
				  (XtPointer) rp) ;
			
		XtAddEventHandler(rp->target,
				  LeaveWindowMask | ButtonPressMask,
				  False,
				  LeaveTargetHandler,
				  (XtPointer) rp) ;

		XtAddCallback(rp->target,
			      XmNdestroyCallback,
			      DestroyTargetCallback,
			      (XtPointer) rp) ;
	}
	
	return rp ;
}

#ifndef   _NO_PROTO
void _XmToolTipSetEnabled(Widget tooltip, Widget target, Boolean enabled)
#else  /* _NO_PROTO */
void _XmToolTipSetEnabled(tooltip, target, enabled)
	Widget  tooltip ;
	Widget  target ;
	Boolean enabled ;
#endif /* _NO_PROTO */
{
	register XmToolTipRecord_p rp ;

	if ((rp = _XmToolTipFindRecord(tooltip, target)) != (XmToolTipRecord_p) 0) {
		rp->enabled = enabled ;
	}
}

#ifndef   _NO_PROTO
void _XmToolTipSetTip(Widget tooltip, Widget target, XmString tip, XmFontList font)
#else  /* _NO_PROTO */
void _XmToolTipSetTip(tooltip, target, tip, font)
	Widget     tooltip ;
	Widget     target ;
	XmString   tip ;
	XmFontList font ;
#endif /* _NO_PROTO */
{
	register XmToolTipRecord_p rp ;
	         XmToolTipWidget   tp = (XmToolTipWidget) tooltip ;

	if ((rp = _XmToolTipFindRecord(tooltip, target)) != (XmToolTipRecord_p) 0) {
		if (rp->tip != (XmString) 0) {
			XmStringFree(rp->tip) ;
			
			rp->tip = (XmString) 0 ;
		}

		if (tip != (XmString) 0) {
			rp->tip = XmStringCopy(tip) ;
		}

		if (font != (XmFontList) 0) {
			if (rp->font != (XmFontList) 0) {
				XmFontListFree(rp->font) ;
			}

			rp->font = XmFontListCopy(font) ;
		}
		else {
			if (rp->font == (XmFontList) 0) {
				rp->font = XmFontListCopy(tp->tooltip.font_list) ;
			}
		}
	}
}

#ifndef   _NO_PROTO
void _XmToolTipRemoveRecord(Widget tooltip, Widget target)
#else  /* _NO_PROTO */
void _XmToolTipRemoveRecord(tooltip, target)
	Widget tooltip ;
	Widget target ;
#endif /* _NO_PROTO */
{
	register XmToolTipRecord_p rp ;

	if ((rp = _XmToolTipFindRecord(tooltip, target)) != (XmToolTipRecord_p) 0) {
		_XmToolTipDeleteRecord(rp) ;
	}
}

#ifndef   _NO_PROTO
void _XmToolTipRemoveAllRecords(Widget tooltip)
#else  /* _NO_PROTO */
void _XmToolTipRemoveAllRecords(tooltip)
	Widget tooltip ;
#endif /* _NO_PROTO */
{
	register XmToolTipRecord_p rp ;

	while ((rp = _XmToolTipFindRecord(tooltip, (Widget) 0)) != (XmToolTipRecord_p) 0) {
		_XmToolTipDeleteRecord(rp) ;
	}
}

typedef struct ToolTipResources_s
{
	XmString      tooltip_string ;
	XmFontList    tooltip_font ;
#if       (XmVERSION > 1)
	XmRenderTable tooltip_render_table ;
#endif /* (XmVERSION > 1) */
} ToolTipResources_t, *ToolTipResources_p ;

#define Offset(field) XtOffsetOf(ToolTipResources_t, field)

#ifndef    _NO_PROTO
static XmString _XmToolTipGetExternal(Widget widget, XmFontList *font)
#else   /* _NO_PROTO */
static XmString _XmToolTipGetExternal(widget, font)
	Widget      widget ;
	XmFontList *font ;
#endif  /* _NO_PROTO */
{
	static XtResource  resources[] =
	{
		{
			XmNtooltip,
			XmCTooltip,
			XmRXmString,
			sizeof(XmString),
			Offset(tooltip_string),
			XtRImmediate,
			(XtPointer) 0
		},
#if       (XmVERSION > 1)
		{
			XmNtooltipRenderTable,
			XmCTooltipRenderTable,
			XmRRenderTable,
			sizeof(XmRenderTable),
			Offset(tooltip_render_table),
			XtRImmediate,
			(XtPointer) 0
		},
#endif /* (XmVERSION > 1) */
		{
			XmNtooltipFont,
			XmCTooltipFont,
			XmRFontList,
			sizeof(XmFontList),
			Offset(tooltip_font),
			XtRImmediate,
			(XtPointer) 0
		}
	} ;

	ToolTipResources_t base ;

	base.tooltip_string       = (XmString)   0 ;
	base.tooltip_font         = (XmFontList) 0 ;

#if       (XmVERSION > 1)
	base.tooltip_render_table = (XmRenderTable) 0 ;
#endif /* (XmVERSION > 1) */

	XtGetSubresources(widget, 
		         (XtPointer)      &base, 
			  NULL,
			  NULL,
		         (XtResourceList) &resources,
		          XtNumber(resources),
		         (ArgList) 0,
		         (Cardinal) 0) ;

	*font = base.tooltip_font ;

#if       (XmVERSION > 1)
	if (base.tooltip_render_table != (XmRenderTable) 0) {
		*font = base.tooltip_render_table ;
	}
#endif /* (XmVERSION > 1) */

	return base.tooltip_string ;
}

#undef Offset

/*
** Public Interface
*/

#ifndef   _NO_PROTO
Widget XmCreateToolTip(Widget parent, char *name, ArgList argv, Cardinal argc)
#else  /* _NO_PROTO */
Widget XmCreateToolTip(parent, name, argv, argc)
	Widget   parent ;
	char    *name ;
	ArgList  argv ;
	Cardinal argc ;
#endif /* _NO_PROTO */
{
	return XtCreateWidget(name, xmToolTipWidgetClass, parent, argv, argc) ;
}

#ifndef   _NO_PROTO
void XmToolTipRegister(Widget tooltip, Widget widget, XmString help_tip, XmFontList help_font)
#else  /* _NO_PROTO */
void XmToolTipRegister(tooltip, widget, help_tip, help_font)
	Widget     tooltip ;
	Widget     widget ;
	XmString   help_tip ;
	XmFontList help_font ;
#endif /* _NO_PROTO */
{
#ifdef    XTHREADS
	XtAppContext app_context = XtWidgetToApplicationContext(tooltip) ;
	
	XtAppLock(app_context) ;
#endif /* XTHREADS */

	if (XmIsToolTip(tooltip)) {
		if ((widget != (Widget) 0) && !(XmIsGadget(widget))) {
			XmString   help_external = (XmString) 0 ;
			XmFontList font_external = (XmFontList) 0 ;

			if ((help_tip == (XmString) 0) || (help_font == (XmFontList) 0)) {
				help_external = _XmToolTipGetExternal(widget, &font_external) ;
			}

			(void) _XmToolTipAddRecord(tooltip, 
						   widget, 
						   (help_tip  ? help_tip  : help_external),
						   (help_font ? help_font : font_external)) ;
		}
	}
#ifdef    XTHREADS
	XtAppUnlock(app_context) ;
#endif /* XTHREADS */
}

#ifndef   _NO_PROTO
void XmToolTipUnregister(Widget tooltip, Widget widget)
#else  /* _NO_PROTO */
void XmToolTipUnregister(tooltip, widget)
	Widget   tooltip ;
	Widget   widget ;
#endif /* _NO_PROTO */
{
#ifdef    XTHREADS
	XtAppContext app_context = XtWidgetToApplicationContext(tooltip) ;

	XtAppLock(app_context) ;
#endif /* XTHREADS */
	
	if (XmIsToolTip(tooltip)) {
		if (widget != (Widget) 0) {
			_XmToolTipRemoveRecord(tooltip, widget) ;
		}
	}

#ifdef    XTHREADS
	XtAppUnlock(app_context) ;
#endif /* XTHREADS */
}

#ifndef   _NO_PROTO
void XmToolTipSetEnabled(Widget tooltip, Widget widget, Boolean enabled)
#else  /* _NO_PROTO */
void XmToolTipSetEnabled(tooltip, widget, enabled)
	Widget   tooltip ;
	Widget   widget ;
	Boolean  enabled ;
#endif /* _NO_PROTO */
{
#ifdef    XTHREADS
	XtAppContext app_context = XtWidgetToApplicationContext(tooltip) ;

	XtAppLock(app_context) ;
#endif /* XTHREADS */

	if (XmIsToolTip(tooltip)) {
		if (widget != (Widget) 0) {
			_XmToolTipSetEnabled(tooltip, widget, enabled) ;
		}
	}

#ifdef    XTHREADS
	XtAppUnlock(app_context) ;
#endif /* XTHREADS */
}

#ifndef   _NO_PROTO
void XmToolTipSetTip(Widget tooltip, Widget widget, XmString tip, XmFontList font)
#else  /* _NO_PROTO */
void XmToolTipSetTip(tooltip, widget, tip, font)
	Widget     tooltip ;
	Widget     widget ;
	XmString   tip ;
	XmFontList font ;
#endif /* _NO_PROTO */
{
#ifdef    XTHREADS
	XtAppContext app_context = XtWidgetToApplicationContext(tooltip) ;

	XtAppLock(app_context) ;
#endif /* XTHREADS */

	if (XmIsToolTip(tooltip)) {
		if (widget != (Widget) 0) {
			_XmToolTipSetTip(tooltip, widget, tip, font) ;
		}
	}

#ifdef    XTHREADS
	XtAppUnlock(app_context) ;
#endif /* XTHREADS */
}
