#ifndef _GRAPHICS_H
#define _GRAPHICS_H

#include <afxwin.h>
#include "Image.h"

typedef enum {
	SOLID_LINE,
	DASHED_LINE,
	DOTTED_LINE
} GLineStyle;

typedef enum {
	SOLID_FILL,
	STIPPLE_FILL
} FillStyle;

typedef enum {
	SOLID,
	HOLLOW,
	HASHED
} BrushFillStyle;

typedef enum {
	WINDING_RULE
} FillRule;

class PolygonRegion;

class Graphics
{
	protected:
		FontObj * currentFont;

	public:
		Graphics()
		{
			currentFont = (FontObj*)0;
		}

		virtual void drawLine( int x, int y, int x2, int y2) = 0;
		virtual void drawRect( int x, int y, int width, int height) = 0;
		virtual void drawArc(  int x, int y, int width, int height, float startAngle, float arcAngle) = 0;
		virtual void fillArc(  int x, int y, int width, int height, float startAngle, float arcAngle) = 0;
		virtual void fillPolygon( GPoint * p, int size) = 0;
		virtual void fillRect( int x, int y, int width, int height) = 0;
		virtual void fill3DRect(int x, int y, int w, int h, bool tf) = 0;
		virtual void setClip( PolygonRegion * p) = 0;
		virtual void drawString( char * s, int x, int y) = 0;
		virtual Color *getForegroundColor(CWnd *w) = 0;
		virtual void setColor( Color* c) = 0;
		virtual void setBackgroundColor( Color* c) = 0;
		virtual void setLineStyle( GLineStyle l) = 0;
		virtual FillStyle getFillStyle() = 0;
		virtual void setFillStyle( FillStyle f) = 0;
		virtual void setAnyFillStyle( BrushFillStyle f) = 0;
		virtual void setFillRule( FillRule f) = 0;
		virtual int getFontWidth(char *s) = 0;
		virtual int getFontHeight() = 0;

#ifdef WIN32
		virtual Canvas * getGC() = 0;
#endif
		virtual FontObj * getFont()
		{
			return currentFont;
		}

		virtual void setFont( FontObj * fo) = 0;
		virtual int isSameSizeAs( CWnd * w) = 0;
		virtual int getWidth()  = 0;
		virtual int getHeight() = 0;
		virtual int usesCache() = 0;
		virtual void flushCache() = 0;
		virtual ISTImage *getImage() = 0;
		virtual void setXOR()     = 0;
		virtual void invertDisplay(Boolean yn) = 0;
		virtual void setGridSet(Boolean yn) = 0;
		virtual Boolean getGridSet() = 0;
};
#endif








