#ifndef _MFCGRAPHICS_H
#define _MFCGRAPHICS_H

#include "Toolkit.h"
#include "MFCImage.h"
#include "math.h"

class MFCGraphics : public Graphics
{
public: 
	GImage *temp;
	private:
		CDC			*gc;
		Toolkit		*tk;
		CPen*		oldPen;
		CBrush*		oldBrush;
		CRgn*		oldRgn;
		CFont*		oldFont;
		CBitmap*	oldBitmap;
		CPen		drawPen;
		CBrush		drawBrush;
		CFont		drawFont;
		CDC *		winDC;
		CBitmap		drawBitmap;
		LOGPEN		currentPenStyle;
		LOGBRUSH	currentBrushStyle;
		LOGFONT		currentFontStyle;
		BOOL		invertedDisplay;
		
		CWnd *		parent;
		int			width      ;  
		int			height     ;  
		bool		doubleBuffered;
		FillStyle	fill_style ;
		Boolean		gridSet;
		
		void resetPen(){
			currentPenStyle.lopnStyle = PS_SOLID;	      //SolidLine
			currentPenStyle.lopnWidth = CPoint(0,0);	  //1 Pixel wide
			currentPenStyle.lopnColor = RGB(255,255,255); //White	
			
			cleanupGDI(oldPen);
			gc->SelectObject(&currentPenStyle);
		}

		void cleanupGDI (CGdiObject *originalObject) {
		
		//The need for this function comes from using the MFC CDC wrapper.
		//We actually need to pull the last assigned GDI object CPen/CBrush/CRgn
		//out of the device Context and delete it or we get resource problems.
			
			CGdiObject *killMe = (CGdiObject *) 0;
			if (originalObject != (CGdiObject *) 0) {
				killMe = gc->SelectObject(originalObject);
				if (killMe != (CGdiObject *)0) {
					killMe->DeleteObject();
					originalObject = (CGdiObject *)0;
				}
			}
		}

		void resetBrush(){
			currentBrushStyle.lbStyle = BS_SOLID;		  //SolidFill
			if (invertedDisplay)
				currentBrushStyle.lbColor = RGB(255,255,255);		  //WHITE
			else
				currentBrushStyle.lbColor = RGB(0,0,0 );		  //BLACK
		
			currentBrushStyle.lbHatch = (long) 0;		  //No Hatching
			
			cleanupGDI(oldBrush);
			drawBrush.CreateBrushIndirect(&currentBrushStyle);
			oldBrush = gc->SelectObject(&drawBrush);			
			
			cleanupGDI(oldBrush);
		}
		
		void setup(CWnd *winHandle)
		{
			gridSet = false;
			gc = (CDC*) 0;
			tk        = Toolkit::getDefaultToolkit();
			invertedDisplay = FALSE;
			oldPen    = (CPen *) 0;
			oldBrush  = (CBrush *) 0;
			oldFont   = (CFont *) 0;
			oldBitmap = (CBitmap *) 0;
			doubleBuffered = false;
		}

		void freeGC() {
		}

		typedef struct{
				POINT upperLeft;
				POINT lowerRight;
				POINT startPoint;
				POINT endPoint;
		} ArcPlot;


		ArcPlot CalcArc(int X, int Y, int Width, int Height,
						float fStartDegrees, float fSweepDegrees)		
		{
			int     iXStart, iYStart;	// End point of starting radial line.
			int     iXEnd, iYEnd;       // End point of ending radial line.
			float   fStartRadians;		// Start angle in radians.
		    float   fEndRadians;		// End angle in radians.
			int     XCenter, YCenter;	// GDI X/Y for Arc.
			float   fTwoPi = 2.0f * 3.141592f;
			ArcPlot plotResult;
			DWORD   WidthCenter, HeightCenter;
		
			//Convert Coordinates to Center of Bounding box for MFC.
			XCenter = X + (int)(Width/2);
			YCenter = Y + (int)(Height/2);
			WidthCenter = Width / 2;
			HeightCenter = Height /2;
		
			if (fSweepDegrees > 0.0f) {
					fStartRadians = ((fStartDegrees / 360.0f) * fTwoPi);
					fEndRadians = (((fStartDegrees + fSweepDegrees) / 360.0f) * fTwoPi);
			}   
			else {
					fStartRadians = (((fStartDegrees + fSweepDegrees)  / 360.0f) * fTwoPi);
					fEndRadians =  ((fStartDegrees / 360.0f) * fTwoPi);
			}	
      
			iXStart = X + (int)((float)WidthCenter  * (float)cos(fStartRadians));
			iYStart = Y - (int)((float)HeightCenter * (float)sin(fStartRadians));
		    iXEnd   = X + (int)((float)WidthCenter  * (float)cos(fEndRadians));
			iYEnd   = Y - (int)((float)HeightCenter * (float)sin(fEndRadians));
      
			plotResult.upperLeft.x  = X ;
			plotResult.upperLeft.y  = Y ;
			plotResult.lowerRight.x = XCenter + WidthCenter;
			plotResult.lowerRight.y = YCenter + HeightCenter;
			plotResult.startPoint.x = iXStart;
			plotResult.startPoint.y = iYStart;
			plotResult.endPoint.x   = iXEnd;
			plotResult.endPoint.y   = iYEnd;

			return plotResult;
		}

		

		void moveCursorTo(POINT here)
		{
			//ArcTo and AngleArc will not work on 95/98 so we need to manually
			//move the "pen" and calculate the Arc
			MoveToEx(gc->m_hDC, here.x, here.y, NULL);
		}
  

		BOOL AngleArc(int X, int Y, int Width, int Height,
					   float fStartDegrees, float fSweepDegrees)
		{
				BOOL bResult;
				ArcPlot plotResult = CalcArc (X, Y, Width, Height, fStartDegrees, fSweepDegrees);
				
				/*
					AD_COUNTERCLOCKWISE   Figures drawn counterclockwise.
					AD_CLOCKWISE   Figures drawn clockwise.
				*/

				bResult = gc->Arc( CRect( plotResult.upperLeft, plotResult.lowerRight),
								   plotResult.startPoint,
								   plotResult.endPoint);

				if (fSweepDegrees < 0.0f)
					 moveCursorTo(plotResult.startPoint);
				else
					 moveCursorTo(plotResult.endPoint);

				return bResult;
		}

		BOOL FillAngleArc(int X, int Y, int Width, int Height,
						  float fStartDegrees, float fSweepDegrees)
		{
				BOOL bResult = true;
				ArcPlot plotResult = CalcArc (X, Y, Width, Height, fStartDegrees, fSweepDegrees);
				
				cleanupGDI(oldBrush);
				drawBrush.DeleteObject();
				drawBrush.CreateBrushIndirect(&currentBrushStyle);
				oldBrush = gc->SelectObject(&drawBrush);			
				
				gc->Pie(CRect( plotResult.upperLeft, plotResult.lowerRight),
								   plotResult.startPoint,
								   plotResult.endPoint);
				
				return bResult;
		}
		
		void setPenLineStyle(UINT lStyle)
		{
			
			cleanupGDI(oldPen);
			
			currentPenStyle.lopnStyle = lStyle;
			drawPen.DeleteObject();
			drawPen.CreatePenIndirect(&currentPenStyle);
			oldPen= gc->SelectObject(&drawPen);
		
			oldPen->DeleteObject();
			cleanupGDI(oldPen);
			
			//NB We May need to SetBkMode to TRANSPARENT on Device Context
			//to see the gaps in the line if not default BK colour. Also, if
			//the Pen Width is modified to be > 1 Pixel the PenStyle will not 
			//take effect.
	
		}
		
		void setPenForeColor (Color *c) { 
			cleanupGDI(oldPen);
			//Loads up an RGB value and assigns it to the Device Context.
			currentPenStyle.lopnColor = (COLORREF) c; 
			drawPen.DeleteObject();
			drawPen.CreatePenIndirect(&currentPenStyle);
			oldPen= gc->SelectObject(&drawPen);
			oldPen->DeleteObject();
			cleanupGDI(oldPen);
		}

		void setBrushColor(Color *c) {
			cleanupGDI(oldBrush);		
			drawBrush.DeleteObject();
			currentBrushStyle.lbColor = (COLORREF) c; 
			
			drawBrush.CreateBrushIndirect(&currentBrushStyle);
			oldBrush = gc->SelectObject(&drawBrush);
			oldBrush->DeleteObject();
			cleanupGDI(oldBrush);		
		}

		virtual Canvas *getGC() {
			return (Canvas*)gc;
		}

		void setFontGDI(FontObj *f) {
			cleanupGDI(oldFont);		
			drawFont.DeleteObject();
			drawFont.CreateStockObject(ANSI_VAR_FONT);
			currentFont = (FontObj*) drawFont;
			oldFont = gc->SelectObject(&drawFont);
			oldFont->DeleteObject();
			cleanupGDI(oldFont);		
		}

		void setBrushFillStyle(UINT bStyle)
		{
			cleanupGDI(oldBrush);		
			drawBrush.DeleteObject();
			currentBrushStyle.lbStyle = bStyle;
			
			drawBrush.CreateBrushIndirect(&currentBrushStyle);
			oldBrush = gc->SelectObject(&drawBrush);
			oldBrush->DeleteObject();
			cleanupGDI(oldBrush);		
			
//			BS_HOLLOW Hollow brush. 
//			BS_HATCHED
//			BS_SOLID Solid brush. 

		}


		void setBrushStyle(UINT bStyle)
		{
			cleanupGDI(oldBrush);		
			drawBrush.DeleteObject();
			currentBrushStyle.lbStyle = bStyle;
			
			drawBrush.CreateBrushIndirect(&currentBrushStyle);
			//gc->SetBkColor((COLORREF) c);
			
			oldBrush = gc->SelectObject(&drawBrush);
			oldBrush->DeleteObject();
			cleanupGDI(oldBrush);		
		}
		
		void setWidth ( int w) { width  = w; }
		void setHeight( int h) { height = h; }

		void setupBkDrawing(CWnd *w, int width, int height, BOOL invDisplay = NULL)
		{
			//Double buffering.
			winDC = w->GetDC();

			parent = w;
			
			if (gc == (CDC *)0) {
				gc = new CDC;
				gc->CreateCompatibleDC(winDC); //compatible with sys display	
			}
			
			
			setWidth(width);
			setHeight(height);
			
			if (invDisplay) {
				invertedDisplay = true;
				gc->SetBkColor(RGB(255,255,255));
				gc->SetTextColor(RGB(0,0,0));
				
			}
			else
			{
				invertedDisplay = false;
				gc->SetBkColor(RGB(0,0,0));
				
				gc->SetTextColor(RGB(255,255,255));
			}

			resetPen();
			resetBrush();
			
			if (!drawBitmap.m_hObject) {
				drawBitmap.CreateCompatibleBitmap(winDC,width,height);
			}
			doubleBuffered = true;
			oldBitmap = gc->SelectObject(&drawBitmap);
		}

		void setupStdDrawing (CWnd *w)
		{
			parent = w;
			gc = w->GetDC();
		}

		CBitmap *getMFCImage(){
			CBitmap *temp = gc->SelectObject(oldBitmap);
			oldBitmap = gc->SelectObject(temp);
			return temp;
		}

	public:
			
			FILE *name;

			void log(char *msg){
				name = fopen("C:\\roo.txt","a+");
				fprintf(name,"%s\n",msg);
				fclose (name);
			}

		void setNewContext(CWnd *w)
		{
			if (gc != (CDC *) 0) {
				w->ReleaseDC(gc);
			}
			parent = w;
			gc = w->GetDC();
			
		}

		void setNewContext(CWnd *w, int width, int height, BOOL invDisplay = NULL)
		{
			setup(w);
			setupBkDrawing(w,width,height,invDisplay);
		}

		void init()
		{
		}

		MFCGraphics(CWnd *winHandle) {
			
			setup(winHandle);
			setupStdDrawing (winHandle);
		}

		MFCGraphics( CWnd *w, int ww, int wh, Boolean invDisplay = NULL)
		{
			setup(w);
			setupBkDrawing(w,ww,wh,invDisplay);
		}
		
		~MFCGraphics()
		{
			gc->SelectObject(oldBitmap);
			cleanupGDI(oldPen);
			if (oldPen)
				oldPen->DeleteObject();
			cleanupGDI(oldBrush);
			if (oldBrush)
				oldBrush->DeleteObject();
			cleanupGDI(oldFont);
			if (oldFont)
				oldFont->DeleteObject();
			cleanupGDI(oldBitmap);
			if (oldBitmap)
				oldBitmap->DeleteObject();
			drawPen.DeleteObject();
			drawBrush.DeleteObject();
			drawFont.DeleteObject();
			drawBitmap.DeleteObject();
			
		
			if (gc)
				delete gc;
		
		}

		virtual void drawLine(int x, int y, int x2, int y2)
		{
			gc->MoveTo(x, y);
		    gc->LineTo(x2,y2);
		}

		virtual void
		drawRect(int x, int y, int w, int h)
		{
			gc->Rectangle( x, y, x+w, y+h );
		}

		virtual void
		drawArc(int x, int y, int w, int h,
					float startAngle, float arcAngle)
		{
			AngleArc(x,y,w,h,startAngle,arcAngle);
		}

		virtual void fillArc(int x, int y, int w, int h,
					float startAngle, float arcAngle)
		{
			FillAngleArc(x,y,w,h,startAngle,arcAngle);
		}

		virtual void fillPolygon(GPoint * p,int size)
		{
			gc->Polygon(p, size);
		}

		virtual void fillRect(int x, int y, int w, int h)
		{
			gc->Rectangle(x,y,w+x,h+y);
		}

		virtual void fill3DRect(int x, int y, int w, int h, bool tf)
		{
			CRect r(x, y, x+w, y+h);
			gc->FillRect(r,&drawBrush);
			gc->Draw3dRect(r,RGB(230, 230, 230), RGB(100, 100, 100));
		}

		
		virtual void setClip( PolygonRegion * p)
		{
			gc->SelectClipRgn((CRgn *) p->getData());
		}

		
		virtual void drawString( char * s, int x, int y)
		{
			gc->ExtTextOut(  x,y, ETO_OPAQUE, NULL, CString(s),NULL);
		}

		virtual void setColor( Color * c)
		{
			setPenForeColor(c);
		}

		virtual void setBackgroundColor( Color * c)
		{
			setBrushColor(c); 
		}

		void
		setSolidLineStyle()
		{
			setPenLineStyle(PS_SOLID);
		}

		void
		setDashedLineStyle()
		{
			setPenLineStyle(PS_DASH);
		}

		void
		setDottedLineStyle()
		{
			setPenLineStyle(PS_DOT);
		}

		virtual void setLineStyle( GLineStyle l)
		{
			switch (l) {
				case SOLID_LINE:
					setSolidLineStyle();
					break;
				case DASHED_LINE:
					setDashedLineStyle();
					break;
				case DOTTED_LINE:
					setDottedLineStyle();
					break;
				default:
					return;
			}
		}

		virtual void setFillRule( FillRule f)
		{
			return;
		}

		virtual FillStyle getFillStyle()
		{
			return fill_style;
		}

		virtual void setFillStyle( FillStyle f)
		{
			fill_style = f ;
			switch (f) {
				case SOLID_FILL:
					setSolidFillStyle();
					break;
				case STIPPLE_FILL:
					setStippledFillStyle();
					break;
				default:
					printf("unsupported fill style %d\n", f);
					return;
			}
		}

		void setStippledFillStyle()
		{
			setBrushStyle(BS_HATCHED);
		}

		virtual void setAnyFillStyle (BrushFillStyle f)
		{
			if (f == SOLID)
				setBrushFillStyle(BS_SOLID);
			if (f == HOLLOW)
				setBrushFillStyle(BS_HOLLOW);			
			if (f == HASHED)
				setBrushFillStyle(BS_HATCHED);			
		}

		void setSolidFillStyle()
		{
			setBrushStyle(BS_SOLID);
		}

		virtual int getFontWidth(char *s)
		{
			//Return the Width of the font currently being used
			//in the Graphics Context.
			//gc->GetTextExtent( &textMetrics );
			CSize boundingBox = gc->GetTextExtent( s, strlen (s) );
			return boundingBox.cx;
		}
		
		virtual int getFontHeight()
		{
			TEXTMETRIC fontMetrics;

			if(gc->GetTextMetrics( &fontMetrics ))
				return fontMetrics.tmAscent+fontMetrics.tmDescent;
			else
				return -1;
			
		}

		virtual void setFont( FontObj * fo)
		{
			setFontGDI(fo); //functional.

		}

		virtual void *getFont( )
		{
			return (void*) gc;
		}

		void getDims( CWnd* w, int * dw, int * dh)
		{
			RECT rect;
			
			w->GetClientRect(&rect);
			*dw =(int)rect.right;
			*dh =(int)rect.bottom;
		}
		
		virtual int getWidth () { return width; }
		virtual int getHeight() { return height; }
		

		virtual int isSameSizeAs( CWnd * cw)
		{
			int draw_height = 0;
			int draw_width  = 0;
			getDims( cw, &draw_width, &draw_height);

			return (getWidth()  == draw_width &&
			       getHeight() == draw_height);
		}

		virtual int usesCache() { 
			return doubleBuffered;
			return (oldBitmap != (CBitmap *) 0); 
		}

		virtual void invertDisplay(Boolean yn)
		{
			invertedDisplay = yn;
		}

		Boolean getInverted()
		{
			return invertedDisplay;
		}
		
		virtual Boolean getGridSet() {
			return gridSet;
		}

		virtual void setGridSet(Boolean yn) {
			gridSet = yn;
		}
		virtual void setXOR()
		{
			gc->SetROP2( R2_NOTXORPEN );
		}

		virtual void flushCache(){	
			BITMAP bmInfo;
			drawBitmap.GetObject(sizeof(bmInfo),&bmInfo);
			winDC = parent->GetDC();
			
			if (winDC->BitBlt(0,0,bmInfo.bmWidth,bmInfo.bmHeight,gc,0,0,SRCCOPY) == 0)
				AfxMessageBox("Zero Blit");
		
		}

		virtual Color *getForegroundColor(CWnd *w){
			LOGPEN colorPen;
			drawPen.GetObject(sizeof(colorPen),&colorPen );
			return (Color*)colorPen.lopnColor; 
		}

		virtual ISTImage* getImage(){
			temp = (GImage *) getMFCImage();
			return (ISTImage *) new MFCImage (this, temp);
		}

};
#endif
