#ifndef _MFCTOOLKIT_H
#define _MFCTOOLKIT_H

#include <afxwin.h>
#include "Toolkit.h"
#include "MFCGraphics.h"

class MFCPolygonRegion;
class MFCToolkit;

class MFCFontMetrics: public FontMetrics
{
	private:
		TEXTMETRIC cs;
		
		void
		setFontMetrics( char * s)
		{
		}
	
	public:
		void setBrushFillStyle (FillStyle f); 
		virtual int getStringWidth( char * s)
		{
			CSize boundingBox = ((CDC *) font)->GetTextExtent( s, strlen (s) );
			return boundingBox.cx;
		}

		MFCFontMetrics( FontObj * f)
			:FontMetrics( f)
		{}

};

class MFCPolygonRegion: public PolygonRegion
{
	private:
		CRgn r;

	virtual void createRegion( GPoint * p, int sz)
	{
		//If we can get a handle on the Region delete it.
		if (r.m_hObject != (HRGN) 0) 
			r.DeleteObject();
		r.CreatePolygonRgn( p, sz, WINDING );
	}

	public:
		MFCPolygonRegion( GPoint * ps, int size)
		{
			createRegion (ps,size);
		}

		MFCPolygonRegion(CRgn *myr)
		{
			if (r.m_hObject != (HRGN) 0) {
				r.DeleteObject();
			}
			r.CreateRectRgn(0,0,0,0);
			r.CopyRgn(myr);
		}

		~MFCPolygonRegion()
		{
			r.DeleteObject();
		}

		virtual void * getData() { 
			return (void*)&r; 
		}

		virtual PolygonRegion* 
		getIntersection( PolygonRegion * p)
		{
			CRgn IntersectRgn;
			IntersectRgn.CreateRectRgn(0,0,0,0);
			if (r.m_hObject != (HRGN) 0) {
				IntersectRgn.CombineRgn(&r,(CRgn *) p->getData(),RGN_AND);
				return new MFCPolygonRegion( &IntersectRgn );
			}
			else {

				return p;
			}
		}
};

class MFCToolkit: public Toolkit
{
	private:
		CWnd    *     mainWidget;  
		FontObj *     defaultFont; 
		HCURSOR oldCursor;
		COLORREF *colorBlack;
		COLORREF *colorWhite;
		COLORREF *colorGray;
		COLORREF *colorOrange;
		Graphics * g;

	public:
		MFCToolkit() {
			g = (Graphics *) 0;	
		}

		virtual Graphics * getGraphics(CWnd *w)
		{
		
				g = new MFCGraphics(w);
		
			return g;
		}

		virtual Graphics * getGraphics( CWnd * c,int width, int height, Boolean invDisplay)
		{
			mainWidget = c;
			return new MFCGraphics( mainWidget, width, height, invDisplay);
		}

		virtual int init( CWnd * top, char * pcm)
		{
			mainWidget = top; 
			return 1;
		}
	
		virtual int setIcon( char * iconname, char * iconlabel)
		{
			return 1;
		}

		virtual void setIconFromData( char * bits, int width, int height)
		{
		}

		void useIcon( HICON icon)
		{
		}

		void initDialog( CWnd * d)
		{
		}

		PolygonRegion * getPolygonRegion( GPoint * points, int size)
		{
			return new MFCPolygonRegion( points,size);
		}

		void setContext( CWnd* w)
		{
		}

		
		virtual void* getDisplay()
		{
			return mainWidget->GetDC();
		}

		CWnd* getMainWidget()
		{
			return mainWidget;
		}

		virtual FontMetrics * getFontMetrics( FontObj * f)
		{
			return new MFCFontMetrics( f);
		}

		virtual Color* loadColor( char * rgb)
		{
			int bRed,bGreen,bBlue;
			CString parseRGB (rgb);

			if (!parseRGB.IsEmpty()){
				sscanf(((LPCTSTR) parseRGB.Mid(1,2)),"%x",&bRed);		
				sscanf(((LPCTSTR) parseRGB.Mid(3,2)),"%x",&bGreen);
				sscanf(((LPCTSTR) parseRGB.Right(2)),"%x",&bBlue);
			}
			else
				return NULL;

			return (Color *)PALETTERGB((BYTE)bRed,(BYTE)bGreen,(BYTE)bBlue);
		}

		virtual int getSelectionByIndex (CWnd *radioBox)
		{
			int     firstID        = 0;
			int     lastID         = 0;
			int     checkedID      = 0;

			CWnd    *currentWindow = radioBox->GetWindow(GW_CHILD);
			CWnd    *cwn;

			lastID=currentWindow->GetDlgCtrlID ();

			while (cwn = currentWindow->GetNextWindow(GW_HWNDNEXT))
				currentWindow = cwn;

			firstID = currentWindow->GetDlgCtrlID ();

			//Strangely resource values are inverted according to creation
			//technique i.e resources/code so flip them if necessary
			if (firstID > lastID){
				int buffer = lastID;
				lastID = firstID;
				firstID = buffer;
			}

			checkedID = radioBox->GetCheckedRadioButton( firstID, lastID);

			if (checkedID == 0)
				  return -1;

			checkedID = checkedID - firstID;

			return checkedID;
		  }

		  virtual int getRBChildren (CWnd *radioBox)
		  {
			CWnd    *currentWindow  = radioBox->GetWindow(GW_CHILD);
			CWnd    * cwn;
			int Counter = 0;
			
			while (cwn = currentWindow->GetNextWindow(GW_HWNDNEXT)){
		   				currentWindow = cwn;
					Counter++;
			}

			return Counter;
		  }

		  void setSelectionByIndex (CWnd *radioBox, int val)
		  {
			int     firstID         = 0;
			int     lastID          = 0;
			int     checkID         = 0;
			CWnd    *currentWindow  = radioBox->GetWindow(GW_CHILD);
			CWnd    * cwn;

			lastID = currentWindow->GetDlgCtrlID ();

			while (cwn = currentWindow->GetNextWindow(GW_HWNDNEXT))
		   		currentWindow = cwn;

			firstID=currentWindow->GetDlgCtrlID ();

			if (firstID > lastID) {
				int buf = firstID;
				firstID=lastID;
				lastID=buf;
			}
	checkID = firstID + val;
			radioBox->CheckRadioButton(firstID, lastID, checkID);

			return;
		  }

		char * getCursorFor( GCursor c)
		{
			switch (c) {
				case WATCH_CURSOR:
					return IDC_WAIT;   
				case EXCHANGE_CURSOR:
					return IDC_SIZEALL;
				case FLEUR_CURSOR:
					return IDC_WAIT;
				case PENCIL_CURSOR:
					return IDC_UPARROW;
				case CROSS_CURSOR:
					return IDC_CROSS;
				default:
					break;
			}
			
			return IDC_ARROW;

		}

		virtual void defineCursor( CWnd * w, GCursor cursor)
		{
			if (!w->IsWindowVisible())
				return;  
			HCURSOR hCursor = AfxGetApp()->LoadStandardCursor(getCursorFor(cursor));
			SetClassLong(w->m_hWnd,GCL_HCURSOR,(LONG) hCursor);   // new cursor 
			oldCursor = SetCursor( hCursor );
			
		}

		virtual void undefineCursor( CWnd * w)
		{
				SetCursor(oldCursor);
				SetClassLong(w->m_hWnd,GCL_HCURSOR,(LONG) oldCursor);   // new cursor 
		}

		virtual int getWidth(CWnd * w)
		{
			RECT rect;
			w->GetClientRect(&rect);
			return (int)rect.right; 
		}

		virtual int getHeight(CWnd * w)
		{
			RECT rect;
			w->GetClientRect(&rect);
			return (int)rect.bottom; 
	
		}

		virtual FontObj * getDefaultFont() { return NULL; }
		
		
		virtual Color * getColor( GColor color)
		{
			switch( color) {
				case COLOR_BLACK:
				
					return (Color*) RGB(0,0,0);
				case COLOR_WHITE:
				
					return (Color*) RGB(255,255,255);
				case COLOR_GRAY:
				
					return (Color*) RGB(122,0,0);
				case COLOR_ORANGE:
				
					return (Color*) RGB (255,0,0);
				default:
					break;
			}
			return (Color*)0;
		}
		
		
		GButton getButton( UINT cFlags)
		{
			//Modifiers if necessary.
			//MK_CONTROL   Set if the CTRL key is down.
			//MK_SHIFT   Set if the SHIFT key is down.

			if ((cFlags & MK_LBUTTON) == MK_LBUTTON)
				return BUTTON_1;
			
			//Other buttons if necessary.
			//MK_MBUTTON   Set if the middle mouse button is down.
			//MK_RBUTTON   Set if the right mouse button is down.
			
			return NOT_A_BUTTON; // actually not interested at the moment
		}

		
		virtual GEvent * newEvent( void * ev)
		{
			return (GEvent*)0;
		}

		GEvent * newMouseMoveEvent( UINT nFlags, CPoint point )
		{
			GMotionEvent * gme = new GMotionEvent( point.x, point.y);
			return (GEvent*)gme;
		}	
		
		GEvent * newKeyPressEvent (UINT nChar, UINT nRepCnt, UINT nFlags, int isapress )
		{
				GKeyEvent * gke = new GKeyEvent(0, 0);
				gke->setOnsetEvent(isapress);
				return (GEvent*)gke;
		}
		
		GEvent * newButtonClickEvent( UINT nFlags, CPoint point , int isapress)
		{
				GButtonEvent * gbe = new GButtonEvent(point.x, point.y);
				gbe->setOnsetEvent(isapress);
				gbe->setButton( getButton( nFlags ));
				return (GEvent*)gbe;
			
		}	
		
		virtual void addWorkProcedure( void * proc, void * data) 
		{
		}

		virtual void updateDisplay()
		{
		}
};

#endif
