#ifndef _XGRAPHICS_H
#define _XGRAPHICS_H

#ifndef _XtIntrinsic_h
#include <X11/Intrinsic.h>
#endif

#include "Toolkit.h"
#include "XmImage.h"

class XGraphics: public Graphics
{
	private:
		GC        gc         ;
		Drawable  drawable   ;
		Display * display    ;
		Toolkit * tk         ;
		FillStyle fill_style ;
		Pixmap    pixmap     ;
		int       width      ;
		int       height     ;
		Drawable realdrawable;
                Boolean invertedDisplay;
                GImage *temp;
		Boolean gridSet;

		void setup()
		{
			realdrawable = (Drawable)0;
                        invertedDisplay = FALSE;
			stipmap  = (Pixel) 0;
			pixmap   = (Pixmap)0;
			tk       = Toolkit::getDefaultToolkit();
			display  = (Display *)tk->getDisplay();
			width    = -1;
			height   = -1;
		}

		void init()
		{
			gc  = XCreateGC( display, drawable, 0, NULL);
                        XFlush(display);
		}

		void freeGC() {
			if (gc != (GC)0) {
				XFreeGC( display, gc);
				gc = (GC)0;
			}
		}

		void setWidth ( int w) { width  = w; }
		void setHeight( int h) { height = h; }
		void setRealDrawable( Drawable d) { realdrawable = d; }

		void
		setPixmapDrawable( Widget w, int ww, int wh, Boolean invDisplay = 0)
		{
			setWidth(ww);
			setHeight(wh);
                        
			pixmap=XCreatePixmap(display,
				RootWindowOfScreen(XtScreen(w)),
					(unsigned int) width,
					(unsigned int) height,
		    (unsigned int) XDefaultDepth(display,DefaultScreen(display)));
		       drawable = pixmap;


                            
		}


		Pixmap stipmap;
		int    stip_xorigin;

		void getDims( Widget w, Dimension * dw, Dimension * dh)
		{
			XtVaGetValues(w,
				XmNheight,&dw,
				XmNwidth, &dh,
					NULL);
		}

		void setDimensions( Widget w)
		{
			Dimension draw_height = 0;
			Dimension draw_width  = 0;

			getDims( w, &draw_width, &draw_height);
			setWidth( draw_width);
			setHeight(draw_height);
		}

                virtual GImage *getXImage() {
		                return (GImage *) XGetImage(display,
		                                  drawable,
		                                  0,0,
		                                  width,
		                                  height,
		                                  (unsigned long) AllPlanes,
		                                  ZPixmap);
		}

		virtual ISTImage* getImage(){
			temp = (GImage *) getXImage();
			ISTImage *im = (ISTImage *) new XMImage (this,temp);
			return  im;
		}

	public:
		XGraphics()
		{
			setup();
		}

		XGraphics( Widget w)
		{
			setup();
			setDrawable(XtWindow(w));
			setDimensions( w);
			init();
		}

		XGraphics( Widget w,int ww, int wh, Boolean invDisplay = 0)
		{
			setup();
			setPixmapDrawable( w, ww, wh);
			setRealDrawable( XtWindow(w));
			init();
		}

		XGraphics( Drawable win)
		{
			setup();
			setDrawable(win);
			init();
		}

		~XGraphics()
		{
			XFreePixmap(display,stipmap);
			if (pixmap)
				XFreePixmap(display, pixmap);
			freeGC();
		}

		void setDrawable( Drawable d)
		{
			drawable = d;
		}

		virtual void setXOR()
		{
			XSetFunction(display,gc,GXxor);
		}

		virtual void      setFont( FontObj * fo)
		{
			currentFont = fo;
			XFontStruct * font = (XFontStruct*)currentFont;
			XSetFont(display, gc, font->fid);
		}

		virtual int getWidth () { return width; }
		virtual int getHeight() { return height; }
		virtual int isSameSizeAs( CWnd * cw)
		{
			Dimension draw_height = 0;
			Dimension draw_width  = 0;

			Widget w = cw->xd_rootwidget();
			getDims( w, &draw_width, &draw_height);

			return (getWidth()  == draw_width &&
			       getHeight() == draw_height);
		}

		virtual int usesCache() { return (pixmap != (Pixmap)0); }

		virtual void flushCache()
		{
			XCopyArea(display, pixmap, realdrawable, gc,
				0,0,
			(unsigned int) getWidth(),
			(unsigned int) getHeight(),0,0);
		}

		virtual void
		drawLine(int x, int y, int x2, int y2)
		{
			XDrawLine(display, drawable, gc, x, y, x2, y2);
		}

		virtual void
		drawRect(int x, int y, int w, int h)
		{
			XDrawRectangle(display, drawable, gc, x, y, w, h);
		}

		virtual void
		drawArc(int x, int y, int w, int h, float startAngle, float arcAngle)
		{
			XDrawArc(display, drawable, gc, x, y, w, h, (int)(startAngle*64), (int)(arcAngle*64));
		}

		virtual void fillArc(int x, int y, int w, int h, float startAngle, float arcAngle)
		{
			XFillArc(display, drawable, gc, x, y, w, h, (int)(startAngle*64), (int)(arcAngle*64));
		}

		// the java way is a Polygon data structure...

		virtual void fillPolygon(GPoint * p,int size)
		{
			XFillPolygon( display, drawable, gc,
				p, size,
					Nonconvex,CoordModeOrigin);
		}

		virtual void fillRect(int x, int y, int w, int h)
		{
			XFillRectangle( display,drawable, gc,
				x, y, w, h);
		}

		virtual void fill3DRect(int x, int y, int w, int h, bool a)
		{
			XFillRectangle( display,drawable, gc,
				x, y, w, h);
		}


		virtual void drawString( char * s, int x, int y)
		{
			XDrawString(display, drawable, gc, x, y, s, strlen(s));
		}

		// note that only here do we know about the Pixel

		virtual void setColor( Color * c)
		{
			XColor *xc = (XColor*)c;
			Pixel  p  = xc->pixel;


			XSetForeground(display,gc,p);
		}

		virtual void setBackgroundColor( Color * c)
		{
			XColor *xc = (XColor*)c;
			Pixel  p  = xc->pixel;

			XSetBackground(display,gc,p);
		}

		void
		setSolidLineStyle()
		{
			XSetLineAttributes(display,gc,
			       1,LineSolid,CapRound,JoinRound);
		}

		void
		setDashedLineStyle()
		{
			static char dashed[]  = {4,7};
			XSetLineAttributes( display,gc,
			       1,LineOnOffDash,CapRound,JoinRound);
			XSetDashes(display,gc,0,dashed,2);
		}

		void
		setDottedLineStyle()
		{
			static char dotted[]  = {1,3};
			XSetLineAttributes( display,gc,
			       1,LineOnOffDash,CapRound,JoinRound);
			XSetDashes(display,gc,0,dotted,2);
		}

		virtual void setLineStyle( GLineStyle l)
		{
			switch (l) {
				case SOLID_LINE:
					setSolidLineStyle();
					break;
				case DASHED_LINE:
					setDashedLineStyle();
					break;
				case DOTTED_LINE:
					setDottedLineStyle();
					break;
				default:
					return;
			}
		}


		virtual void setFillRule( FillRule f)
		{
			if (f != WINDING_RULE)
				return;
			XSetFillRule(display,gc,WindingRule);
		}

		virtual void setFillStyle( FillStyle f)
		{
			fill_style = f;
			switch (f) {
				case SOLID_FILL:
					setSolidFillStyle();
					break;
				case STIPPLE_FILL:
					setStippledFillStyle();
					break;
				default:
					printf("unsupported fill style %d\n", f);
					return;
			}
		}

		virtual FillStyle getFillStyle()
		{
			return fill_style;
		}

		void setStippledFillStyle()
		{
		     
		       if (stipmap == (Pixel) 0) {
				static unsigned char stipple[] = {
					0x44, 0x44, 0x22, 0x22, 0x11, 0x11, 0x88, 0x88, 0x44, 0x44, 0x22, 0x22, 0x11, 0x11, 0x88, 0x88, 0x44, 0x44, 0x22, 0x22, 0x11, 0x11, 0x88, 0x88, 0x44, 0x44, 0x22, 0x22, 0x11, 0x11, 0x88, 0x88
				};
			      
				stipmap=XCreateBitmapFromData(display,
				    drawable,//XDefaultRootWindow(display),
				    (char *) stipple,
				    16,
				    16);
			}
			
			XSetStipple(display,gc,stipmap);
			XSetTSOrigin(display,gc,stip_xorigin,0);
			stip_xorigin++; 
			XSetFillStyle( display, gc, FillStippled);
		}

		void setSolidFillStyle()
		{
			XSetFillStyle( display, gc, FillSolid);
		}

		virtual int getFontWidth(char *s)
		{
			return 200; //no handling, fontwidths need to be acquired.
		}

		virtual int getFontHeight()
		{
			FontObj * f = getFont();
			if (f == (FontObj*)0)
				f = tk->getDefaultFont();
			XFontStruct * font = (XFontStruct*)f;
			return font->ascent+font->descent;
		}

		virtual Color *getForegroundColor(CWnd * w){
                        XColor *foregroundColour;
                        XtVaGetValues(w->xd_rootwidget(),
                                      XmNforeground, &foregroundColour,
                                      NULL);
                        return (Color*) foregroundColour;
   		}


		virtual void invertDisplay(Boolean yn) {
                        invertedDisplay = yn;
		}

		virtual void setAnyFillStyle( BrushFillStyle f) {
		}

		void getXColors(XColor *colors) {
			XQueryColors(display,NULL,colors,256);
		}

		virtual void setClip( PolygonRegion * p)
		{
		        return;
		}
		virtual Boolean getGridSet() {
			return gridSet;
		}

		virtual void setGridSet(Boolean yn) {
			gridSet = yn;
		}
};

#endif















