#ifndef _XTOOLKIT_H
#define _XTOOLKIT_H
#include "XGraphics.h"
#include <X11/cursorfont.h>

class XFontMetrics: public FontMetrics
{
	private:
		XCharStruct  cs;

		void
		setFontMetrics( char * s)
		{
			XTextExtents((XFontStruct*) font,
					s,(int) strlen(s),
					&isLeftToRight,
					&fontAscent,
					&fontDescent,
						&cs);
		}

	public:
		XFontMetrics( FontObj * f)
			:FontMetrics( f)
		{}

		virtual int getStringWidth( char * s)
		{
			setFontMetrics( s);
			return cs.width;
		}

};

class XTPolygonRegion: public PolygonRegion
{
	private:
		Region r;

		void createRegion( GPoint * p, int sz)
		{
			printf("Region with %d points\n", sz);
			r = XPolygonRegion( p, sz, WINDING_RULE);
		}

	public:
		XTPolygonRegion( GPoint * ps, int size)
		{
		       createRegion( ps, size);
		}

		XTPolygonRegion( Region myr)
		{
			r = myr;
		}

		~XTPolygonRegion()
		{
			if (r != (Region)0)
				XDestroyRegion( r);
		}

		virtual void * getData() { return (void*)r; }

		virtual PolygonRegion *
		getIntersection( PolygonRegion * p)
		{
			Region nr = XCreateRegion();
			Region pr = (Region)p->getData();
			XIntersectRegion( r, pr, nr);

			return new XTPolygonRegion( nr);
		}
};

class XToolkit: public Toolkit
{
	private:
		Widget        mainWidget;
		XtAppContext  app_context;
		int           num_of_planes;
		Colormap      color_map;
		Pixel         Black;
		Pixel         White;
		FontObj *     defaultFont;

		FontObj * createDefaultFont()
		{
			XFontStruct *the_font = (XFontStruct *) 0 ;
			char  *def = (char*)XtDefaultFont;
			XrmValue    fromVal;
			XrmValue    toVal;
			Boolean     converted;

			toVal.addr = (caddr_t) 0 ;
			toVal.size = 0 ;

			fromVal.addr=(caddr_t) def;
			fromVal.size=(unsigned int) strlen(def);
			converted=XtConvertAndStore(mainWidget,
			       XtRString,     &fromVal,
			       XtRFontStruct, &toVal);
			if ( ! converted )
				return (FontObj*)0;

			if (toVal.addr != (XtPointer) 0)
				the_font=(XFontStruct *) ( *((XFontStruct **) toVal.addr));

			return (the_font);
		}

		unsigned int getCursorFor( GCursor c)
		{
			switch (c) {
				case WATCH_CURSOR:
					return XC_watch;
				case EXCHANGE_CURSOR:
					return XC_exchange;
				case FLEUR_CURSOR:
					return XC_fleur;
				case PENCIL_CURSOR:
					return XC_pencil;
				case CROSS_CURSOR:
					return XC_cross;
				default:
					break;
			}
			return XC_left_ptr;
		}

	public:

		virtual void defineCursor( CWnd * w, GCursor cursor)
		{
			Widget wid  = w->xd_rootwidget();
			if (!XtIsRealized(wid))
				return;
			Cursor c    = XCreateFontCursor(XtDisplay(wid),getCursorFor(cursor));
			XDefineCursor(XtDisplay(wid), XtWindow(wid), c);
		}

		virtual void undefineCursor( CWnd * w)
		{
			Widget wid  = w->xd_rootwidget();
			if (!XtIsRealized(wid))
				return;
			XUndefineCursor(XtDisplay(wid), XtWindow(wid));
		}

		virtual int getWidth( CWnd * w)
		{
			Dimension d = (Dimension)0;
			XtVaGetValues(w->xd_rootwidget(), XmNwidth, &d, NULL);
			return (int)d;
		}

		virtual int getHeight( CWnd * w)
		{
			Dimension d = (Dimension)0;
			XtVaGetValues(w->xd_rootwidget(), XmNheight, &d, NULL);
			return (int)d;
		}

		virtual Graphics * getGraphics( CWnd * c)
		{
			mainWidget = c->xd_rootwidget();
			return (Graphics*) new XGraphics( mainWidget);
		}


		virtual Graphics * getGraphics( CWnd * c,int width, int height, Boolean invDisplay = NULL)
		{
			mainWidget = c->xd_rootwidget();
			return (Graphics *) new XGraphics( mainWidget, width, height);
		}

		virtual void * getDisplay()
		{
			return XtDisplay(mainWidget);
		}

		virtual int init( CWnd * top, char * pcm)
		{
                        mainWidget = top->xd_rootwidget();
			Display *d = (Display *)getDisplay();
			app_context=XtWidgetToApplicationContext(mainWidget);
			num_of_planes=XDefaultDepth( d,	DefaultScreen(d));
			/* black and white switched for inverse */
			Black=BlackPixel(d,DefaultScreen(d));
			White=WhitePixel(d,DefaultScreen(d));

			XtVaSetValues(mainWidget, XmNallowShellResize, TRUE, NULL);

			XtVaGetValues(mainWidget,XmNcolormap, &color_map, NULL);

			initDialog( top);

			if (!color_map)
				color_map=DefaultColormap(d,DefaultScreen(d));
#if PRIVATE_COLORMAP
   
   if (private_color_map != NULL) {

      color_map=XCopyColormapAndFree(getDisplay(),color_map);
      XParseColor(getDisplay(),color_map,"white", &rgb);
      XAllocColor(getDisplay(),color_map, &rgb);
      Black=rgb.pixel;
      XParseColor(getDisplay(),color_map,"black", &rgb);
      XAllocColor(getDisplay(),color_map, &rgb);
      White=rgb.pixel;
   }
#endif
			defaultFont = createDefaultFont();
			return 1;
		}


		virtual FontObj * getDefaultFont() { return defaultFont; }

		virtual int setIcon( char * iconname, char * iconlabel)
		{
			char image_file[255];
			XImage  *image;

			if (num_of_planes < 8)
				return 0;
			char * esov = getenv("ESOVSYSTEM");
			if (!esov)
				esov = (char  *)".";

			sprintf( image_file, "%s/xresources/EsovIcon.xwd",
							esov);


			Pixmap icon_esov = XmGetPixmap(
				XDefaultScreenOfDisplay((Display *)getDisplay()),
					iconlabel,
						White,
							Black);
			if (!icon_esov)
				return 0;

			useIcon( icon_esov);
			return 1;
		}

		virtual void setIconFromData( char * bits, int width, int height)
		{
			Pixmap icon_esov = XCreatePixmapFromBitmapData(
						(Display *)getDisplay(),
					XDefaultRootWindow((Display *)getDisplay()),
					bits,
					width,
					height,
					White,
					Black,
				(unsigned int) XDefaultDepth((Display *)getDisplay(),DefaultScreen((Display *)getDisplay())));
			useIcon( icon_esov);
		}

		void useIcon( Pixmap icon)
		{
			XtVaSetValues(mainWidget, XmNiconPixmap,icon, NULL);
		}

		void initDialog( CWnd * d)
		{
			XtVaSetValues(d->xd_rootwidget(), XmNcolormap, color_map, NULL);
		}

		PolygonRegion * getPolygonRegion( GPoint * points, int size)
		{
			return new XTPolygonRegion( points,size);
		}

		void setContext( Widget w)
		{
			mainWidget = w;
		}


		virtual FontMetrics * getFontMetrics( FontObj * f)
		{
			return new XFontMetrics( f);
		}

		Colormap getColormap()
		{
			return color_map;
		}

		virtual Color* loadColor( char * rgb)
		{
			XColor *xc = new XColor;

			int status = XParseColor( (Display *)getDisplay(),
			    getColormap(),
			    rgb,
			    xc);

			if (status == BadColor){
				delete xc;
				return (Color*)0;
			}

			status = XAllocColor( (Display *)getDisplay(),
						getColormap(),
						  xc);
			if (status == BadColor){
				delete xc;
				return (Color*)0;
			};

			return (Color*)xc;
		}

		virtual int getSelectionByIndex (CWnd *radioBox)
		{
			Widget menuWidget;
			WidgetList children=NULL;
			int numberOfChildren = 0;
			int counter = 0;
			
			XtVaGetValues(radioBox->xd_rootwidget(),
				      XmNnumChildren,  &numberOfChildren,
				      XmNchildren,     &children,
				      NULL);
			counter=0;
			while (counter < numberOfChildren && !XmToggleButtonGetState(children[counter])) {
				counter++;
			} 
			
			if(counter==numberOfChildren) {
				return(-1);
			}
			else {
				return( counter );
			} 
			
		} 

		virtual int getRBChildren (CWnd *radioBox)
		{
			Widget menuWidget;
			WidgetList children=NULL;
			int numberOfChildren = 0;
			
			XtVaGetValues(radioBox->xd_rootwidget(),
				      XmNnumChildren,  &numberOfChildren,
				      XmNchildren,     &children,
				      NULL);
						
			return( numberOfChildren );
			
		} 

		void setSelectionByIndex (CWnd *radioBox, int ix)
		{
			int counter = 0;
			int numberOfChildren = 0;
			Widget selectedWidget;
			WidgetList children = NULL;
			
			XtVaGetValues(radioBox->xd_rootwidget(),
				      XmNnumChildren,  &numberOfChildren,
				      XmNchildren,     &children,
				      NULL);
			
			XmToggleButtonSetState( children[ix], TRUE, TRUE );
			
		}

		XColor colorBlack;
		XColor colorWhite;

		virtual Color * getColor( GColor color)
		{
			Display * d = (Display *) 0;
                        d = (Display*)getDisplay();
                        if (d != (Display*) 0) {
                                switch( color) {
                                      case COLOR_BLACK:
					colorBlack.pixel = BlackPixel(d, DefaultScreen(d));
					return (Color*)&colorBlack;
                                      case COLOR_WHITE:
					colorWhite.pixel = WhitePixel(d, DefaultScreen(d));
					return (Color*)&colorWhite;
                                      default:
					break;
                                }
                        }
			return (Color*)0;
		}

		GButton getButton( XButtonEvent * xbe)
		{
			if (xbe->button == Button1)
				return BUTTON_1;
			return NOT_A_BUTTON; // actually not interested at the moment
		}

		virtual GEvent * newEvent( void * ev)
		{
			int isapress = 0;
			XEvent *xe = (XEvent*)ev;
			switch(xe->type) {
				case MotionNotify:
				       {
						printf("MotionEvent\n");
						XMotionEvent * xme = (XMotionEvent*)xe;
						GMotionEvent * gme = new GMotionEvent( xme->x, xme->y);
						return (GEvent*)gme;
					}
				case KeyPress:
						isapress = 1;
				case KeyRelease:
					{
						printf("KeyEvent\n");
						XKeyEvent * xke = (XKeyEvent*)xe;
						GKeyEvent * gke = new GKeyEvent(xke->x, xke->y);
						gke->setOnsetEvent(isapress);
						return (GEvent*)gke;
					}
				case ButtonPress:
						isapress = 1;
				case ButtonRelease:
					{
						printf("ButtonEvent\n");
						XButtonEvent * xbe = (XButtonEvent*)xe;
						GButtonEvent * gbe = new GButtonEvent(xbe->x, xbe->y);
						gbe->setOnsetEvent(isapress);
						gbe->setButton( getButton( xbe));
						return (GEvent*)gbe;
					}
				default:
					break;
			}
			
			return (GEvent*)0;
		}

		virtual void     addWorkProcedure( void * proc, void * data) {
			XtAppAddWorkProc( app_context, (XtWorkProc) proc, (XtPointer)data);
		}

		virtual void    updateDisplay()
		{
			XEvent event;
			Display * display = (Display*) getDisplay();
			XFlush(display);
			XmUpdateDisplay(mainWidget);

			XSync(display,0);

			while (XtPending())
			{
				XtNextEvent(&event);
				XtDispatchEvent(&event);
			}
		}
};
#endif
