/*
** A.J.Fountain,
** Imperial Software Technology (IST),
** Berkshire House,
** 252 Kings Road,
** Reading, 
** Berkshire,
** United Kingdom RG1 4HP.
**
** Telephone: +44 189 587055
** Fax:       +44 189 589005
** Email:     af@ist.co.uk
*/

#ifndef   lint
static char *sccsid[] = {"@(#)Converters.c(1.4)"} ; /* 98/06/15 */
#endif /* lint */

/*
** General Purpose Converters to enable resources to be specified in resource files
** and in X-Designer resource panels
*/

#include <stdio.h>
#include <X11/X.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/cursorfont.h>

#ifdef    INTEGRATE_EDITTABLE
#include <Xint/IntDefs.h>
#include <Xint/EditTable.h>
#endif /* INTEGRATE_EDITTABLE */

#include <math.h>

/*
**  Simple structure for holding a name and value pair - used throughout
*/

typedef struct NameValueMap_s *NameValueMap_p ;
typedef struct NameValueMap_s
{
	char *name ;
	int   value ;
} NameValueMap_t ;

/*
** Some structures for holding common EPak/INT data 
*/

typedef struct TwoInteger_s
{
	int i1 ;
	int i2 ;
} TwoInteger_t, *TwoInteger_p ;

typedef struct ThreeInteger_s
{
	int i1 ;
	int i2 ;
	int i3 ;
}  ThreeInteger_t, *ThreeInteger_p ;

typedef struct FourInteger_s
{
	int i1 ;
	int i2 ;
	int i3 ;
	int i4 ;
} FourInteger_t, *FourInteger_p ;

typedef struct TwoFloat_s
{
        float f1 ;
        float f2 ;
} TwoFloat_t, *TwoFloat_p ;
 
typedef struct ThreeFloat_s
{
        float f1 ;
        float f2 ;
        float f3 ;
}  ThreeFloat_t, *ThreeFloat_p ;
 
typedef struct FourFloat_s
{
        float f1 ;
        float f2 ;
        float f3 ;
        float f4 ;
} FourFloat_t, *FourFloat_p ;

typedef struct TwoDouble_s
{
        double d1 ;
        double d2 ;
} TwoDouble_t, *TwoDouble_p ;
 
typedef struct ThreeDouble_s
{
        double d1 ;
        double d2 ;
        double d3 ;
}  ThreeDouble_t, *ThreeDouble_p ;
 
typedef struct FourDouble_s
{
        double d1 ;
        double d2 ;
        double d3 ;
        double d4 ;
} FourDouble_t, *FourDouble_p ;

#ifndef   MAX_TOKENISER_STRING
#define   MAX_TOKENISER_STRING        16
#endif /* MAX_TOKENISER_STRING */

#ifndef   OUT_OF_BOUNDS_MARKER
#define   OUT_OF_BOUNDS_MARKER       500
#endif /* OUT_OF_BOUNDS_MARKER */

#define   ConverterReturn(retVal, retType, toVal)            \
{                                                            \
	 if ((toVal)->addr == (XtPointer) 0) {               \
		 (toVal)->addr = (XtPointer) &(retVal) ;     \
		 (toVal)->size = sizeof(retType) ;           \
                                                             \
		 return True ;                               \
	} else if ((toVal)->size >= sizeof(retType)) {       \
		*((retType *) (toVal)->addr) = (retVal) ;    \
		(toVal)->size = sizeof(retType) ;            \
		                                             \
		return True ;                                \
	} else {                                             \
		(toVal)->size = sizeof(retType) ;            \
		                                             \
		return False ;                               \
	}                                                    \
}

/*
** Just in case you dont have strcasecmp() or strncasecmp()
*/
 
#ifndef   _NO_PROTO
static int STRNCASECMP(char *s1, char *s2, int n)
#else  /* _NO_PROTO */
static int STRNCASECMP(s1, s2, n)
        char *s1 ;
        char *s2 ;
        int   n ;
#endif /* _NO_PROTO */
{
        register int c1, c2 ;
 
        while (*s1 && *s2 && n--) {
                c1 = (isupper(*s1) ? tolower(*s1) : *s1) ;
                c2 = (isupper(*s2) ? tolower(*s2) : *s2) ;
 
                if (c1 != c2) {
                        return 1 ;
                }
 
                s1++ ;
                s2++ ;
        }
 
        if ((n <= 0) && (c1 == c2)) {
                return 0 ;
        }
 
        if (*s1 || *s2) {
                return 1 ;
        }
 
        return 0 ;
}
 
#ifndef   _NO_PROTO
static int STRCASECMP(char *s1, char *s2)
#else  /* _NO_PROTO */
static int STRCASECMP(s1, s2)
        char *s1 ;
        char *s2 ;
#endif /* _NO_PROTO */
{
        register int c1, c2 ;
 
        while (*s1 && *s2) {
                c1 = (isupper(*s1) ? tolower(*s1) : *s1) ;
                c2 = (isupper(*s2) ? tolower(*s2) : *s2) ;
 
                if (c1 != c2) {
                        return 1 ;
                }
 
                s1++ ;
                s2++ ;
        }
 
        if (*s1 || *s2) {
                return 1 ;
        }
 
        return 0 ;
}

/*
** _XdCvtStringToEnum:
**
** Generic Mapper using the above structure for String onto an Enumerated Type
**
** Not really a converter, but a back end for one
**
** ASSUMPTION: The names are sorted alphabetically for efficiency
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
static Boolean _XdCvtStringToEnum(Display       *display,
                                  XrmValuePtr    argv,
				  Cardinal      *argc,
				  XrmValuePtr    from_address,
				  XrmValuePtr    to_address,
				  XtPointer     *closure,
				  NameValueMap_p converter_map,
				  char          *skip_family,   /* Any XmN we can ignore                            */
				  char          *skip_prefix,   /* Any prefix for the map (ALIGNMENT) we can ignore */
				  char          *converter_type)
#else /* _NO_PROTO */
static Boolean _XdCvtStringToEnum(display,
                                  argv,
				  argc,
				  from_address,
				  to_address,
				  closure,
				  converter_map,
				  skip_family,
				  skip_prefix,
				  converter_type)
        Display       *display ;
        XrmValuePtr    argv ;
        Cardinal      *argc ;
        XrmValuePtr    from_address ;
        XrmValuePtr    to_address ;
        XtPointer     *closure ;
        NameValueMap_p converter_map ;
        char          *skip_family ;
        char          *skip_prefix ;
        char          *converter_type ;
#endif/* _NO_PROTO */
{
        static int            output     = 0 ;
               String         input      = (from_address ? (String) from_address->addr : (String) 0) ;
               String         temp       = (String) 0 ;
               String         token      = (String) 0 ;
               String         tptr       = (String) 0 ;
               char           tokeniser[MAX_TOKENISER_STRING + 1] ;
               NameValueMap_p mptr       = (NameValueMap_p) 0 ;
               int            fam_len    = 0 ;
               int            pre_len    = 0 ;
               int            cmp ;
               Boolean        retcode    = True ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        /* Deduce the input string to be converted */
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        /* Convert using the supplied map */
 
 
        output           = 0 ;
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(int) ;
 
        if (isdigit(token[0])) {
                output = atoi(token) ;
        }
        else {
                mptr = converter_map ;
 
                /* Skip any family prefix */
 
                tptr = token ;
 
                /* Skip any XmN style prefix */
 
                if ((fam_len = (skip_family ? strlen(skip_family) : 0)) > 0) {
                        if (STRNCASECMP(tptr, skip_family, fam_len) == 0) {
                                tptr += fam_len ;
                        }
                }
 
                /* Skip any common resource prefix (Eg all the ALIGNMENT_ in XmNALIGNMENT_BEGINNING...) */
 
                if ((pre_len = (skip_prefix ? strlen(skip_prefix) : 0)) > 0) {
                        if (STRNCASECMP(tptr, skip_prefix, pre_len) == 0) {
                                tptr += pre_len ;
                        }
                }
 
                /* Now look up in the table of values provided */
 
 
                for (;;) {
                        if (mptr->name == (char *) 0) {
                                /* Fallen off the end of the Map      */
                                /* Therefore the input does not parse */
 
                                to_address->addr = (XtPointer) 0 ;
                                to_address->size = 0 ;
                                retcode          = False ;
 
                                break ;
                        }
 
                        if ((cmp = STRCASECMP(mptr->name, tptr)) == 0) {
                                output = mptr->value ;

				if (closure != (XtPointer *) 0) {
					*closure = (XtPointer) output ;
				}
 
                                break ;
                        }
 
                        if (cmp < 0) {
                                /* Sorted Names: Passed the point of no return */
 
                                to_address->addr = (XtPointer) 0 ;
                                to_address->size = 0 ;
 
                                retcode          = False ;
 
                                break ;
                        }
 
                        mptr++ ;
                }
        }
 
        XtFree((char *) temp) ;
 
        if (retcode == False) {
                XtStringConversionWarning(input, converter_type) ;
        }
 
        return retcode ;
}

#if       defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT) || defined(INTEGRATE_EPAK) || defined(INTEGRATE_GRAPHPAK)
/*
** XdCvtStringToCursorType():
**
** Converts a string representing a cursor type into the integral value 
**
** Supported input:
**
** (a) The full formal name:        "XC_watch"
** (b) Truncated name:              "watch"
** (c) Integral value:              "2"
*/
/* ARGSUSED */
#ifndef  _NO_PROTO
Boolean XdCvtStringToCursorType(Display    *display,
                                XrmValuePtr argv,
                                Cardinal   *argc,
                                XrmValuePtr from_address,
                                XrmValuePtr to_address,
                                XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToCursorType(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
	static NameValueMap_t cursor_map[] =
	{
	        { "X_cursor",            XC_X_cursor            },
	        { "arrow",               XC_arrow               },
	        { "based_arrow_down",    XC_based_arrow_down    },
	        { "based_arrow_up",      XC_based_arrow_up      },
	        { "boat",                XC_boat                },
	        { "bogosity",            XC_bogosity            },
	        { "bottom_left_corner",  XC_bottom_left_corner  },
	        { "bottom_right_corner", XC_bottom_right_corner },
	        { "bottom_side",         XC_bottom_side         },
	        { "bottom_tee",          XC_bottom_tee          },
	        { "box_spiral",          XC_box_spiral          },
	        { "center_ptr",          XC_center_ptr          },
	        { "circle",              XC_circle              },
	        { "clock",               XC_clock               },
	        { "coffee_mug",          XC_coffee_mug          },
	        { "cross",               XC_cross               },
	        { "cross_reverse",       XC_cross_reverse       },
	        { "crosshair",           XC_crosshair           },
	        { "diamond_cross",       XC_diamond_cross       },
	        { "dot",                 XC_dot                 },
	        { "dotbox",              XC_dotbox              },
	        { "double_arrow",        XC_double_arrow        },
	        { "draft_large",         XC_draft_large         },
	        { "draft_small",         XC_draft_small         },
	        { "draped_box",          XC_draped_box          },
	        { "exchange",            XC_exchange            },
	        { "fleur",               XC_fleur               },
	        { "gobbler",             XC_gobbler             },
	        { "gumby",               XC_gumby               },
	        { "hand1",               XC_hand1               },
	        { "hand2",               XC_hand2               },
	        { "heart",               XC_heart               },
	        { "icon",                XC_icon                },
	        { "iron_cross",          XC_iron_cross          },
	        { "left_ptr",            XC_left_ptr            },
	        { "left_side",           XC_left_side           },
	        { "left_tee",            XC_left_tee            },
	        { "leftbutton",          XC_leftbutton          },
	        { "ll_angle",            XC_ll_angle            },
	        { "lr_angle",            XC_lr_angle            },
	        { "man",                 XC_man                 },
	        { "middlebutton",        XC_middlebutton        },
	        { "mouse",               XC_mouse               },
	        { "pencil",              XC_pencil              },
	        { "pirate",              XC_pirate              },
	        { "plus",                XC_plus                },
	        { "question_arrow",      XC_question_arrow      },
	        { "right_ptr",           XC_right_ptr           },
	        { "right_side",          XC_right_side          },
	        { "right_tee",           XC_right_tee           },
	        { "rightbutton",         XC_rightbutton         },
	        { "rtl_logo",            XC_rtl_logo            },
	        { "sailboat",            XC_sailboat            },
	        { "sb_down_arrow",       XC_sb_down_arrow       },
	        { "sb_h_double_arrow",   XC_sb_h_double_arrow   },
	        { "sb_left_arrow",       XC_sb_left_arrow       },
	        { "sb_right_arrow",      XC_sb_right_arrow      },
	        { "sb_up_arrow",         XC_sb_up_arrow         },
	        { "sb_v_double_arrow",   XC_sb_v_double_arrow   },
	        { "shuttle",             XC_shuttle             },
	        { "sizing",              XC_sizing              },
	        { "spider",              XC_spider              },
	        { "spraycan",            XC_spraycan            },
	        { "star",                XC_star                },
	        { "target",              XC_target              },
	        { "tcross",              XC_tcross              },
	        { "top_left_arrow",      XC_top_left_arrow      },
	        { "top_left_corner",     XC_top_left_corner     },
	        { "top_right_corner",    XC_top_right_corner    },
	        { "top_side",            XC_top_side            },
	        { "top_tee",             XC_top_tee             },
	        { "trek",                XC_trek                },
	        { "ul_angle",            XC_ul_angle            },
	        { "umbrella",            XC_umbrella            },
	        { "ur_angle",            XC_ur_angle            },
	        { "watch",               XC_watch               },
	        { "xterm",               XC_xterm               },
	        { (char *) 0,            -1                     }
	} ;

	return _XdCvtStringToEnum(display,
                                  argv,
                                  argc,
                                  from_address,
                                  to_address,
                                  closure,
                                  cursor_map,
                                  "X",         /* All cursor types are prefixed thus            */
                                  "C_",        /* Ignore any supplied C_ prefix for convenience */
				  "CursorType") ;
}
#endif /* defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT) || defined(INTEGRATE_EPAK) || defined(INTEGRATE_GRAPHPAK) */

#if       defined(INTEGRATE_EPAK) || defined(INTEGRATE_GRAPHPAK)
/*
** XdCvtStringToCursor: 
**
** takes cursors as named in X11/cursortype.h and turns into the real thing
*/
/* ARGSUSED */
#ifndef  _NO_PROTO
Boolean XdCvtStringToCursor(Display    *display,
                            XrmValuePtr argv,
                            Cardinal   *argc,
                            XrmValuePtr from_address,
                            XrmValuePtr to_address,
                            XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToCursor(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
#ifndef   _NO_PROTO
	extern Cursor XCreateFontCursor(Display *, int) ;
#else  /* _NO_PROTO */
	extern Cursor XCreateFontCursor() ;
#endif /* _NO_PROTO */

	/* Well now, this is the same as the above, except we convert into a Cursor not a CursorType */
	/* So we'll just use it to convert the string to CursorType, then construct a cursor off it  */

	static Cursor   output      = (Cursor) 0 ;
	       Boolean  retcode     = True ;
	       int      cursor_type = 0 ;
	       XrmValue to_cursor_type ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

	/* Just change the output address to hold the cursor type ... */

	to_cursor_type.size   = sizeof(int) ;
	to_cursor_type.addr   = (XtPointer) &cursor_type ;

	retcode = XdCvtStringToCursorType(display, argv, argc, from_address, &to_cursor_type, closure) ;

	/* Now create a font cursor with it */

	if (retcode == True) {
		to_address->addr = (XtPointer) 0 ;
		to_address->size = 0 ;

		if (to_cursor_type.size > 0) {
			if (((cursor_type = *((int *) to_cursor_type.addr)) < XC_X_cursor) || 
			     (cursor_type > XC_num_glyphs)                                 ||
			     ((output = XCreateFontCursor(display, cursor_type)) == (Cursor) 0)) {
				XtStringConversionWarning((String) from_address->addr, "Cursor") ;
				
				return False ;
			}

			to_address->addr = (XtPointer) &output ;
			to_address->size = sizeof(Cursor) ;

			if (closure != (XtPointer *) 0) {
				*closure = (XtPointer) output ;
			}

			return True ;
		}
		/* Else... XdCvtStringToCursorType() will have already raised a warning for XmRCursorType */
	}
	/* Else... XdCvtStringToCursorType() will have already raised a warning for XmRCursorType */

	return False ;
}
#endif /* defined(INTEGRATE_EPAK) || defined(INTEGRATE_GRAPHPAK) */

 
#ifndef   INTEGRATE_XDESIGNER
#if       defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_VIEW3D) || defined(INTEGRATE_CHARTOBJECT)
/*
** X-Designer already has this converter registered, but your application may not have
**
** EPak/GraphPak
*/
 
/*
** XdCvtStringToPixmap:
**
** A converter from strings to pixmaps.
*/
 
#include "xpm.h"
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToPixmap(Display    *display,
                            XrmValuePtr argv,
                            Cardinal   *argc,
                            XrmValuePtr from_address,
                            XrmValuePtr to_address,
                            XtPointer  *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToPixmap(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif /* _NO_PROTO */
{
        static   Pixmap  output  = (Pixmap) 0 ;
                 String  input   = (from_address ? (String) from_address->addr : (String) 0) ;
                 String  path    = (String) 0 ;
                 Boolean retcode = False ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        output           = None ;
        to_address->size = sizeof(Pixmap) ;
        to_address->addr = (XtPointer) &output ;
 
        if ((input == (String ) 0) || (strcmp(input, "None") == 0)) {
                retcode = True ;
		
		if (closure != (XtPointer *) 0) {
			*closure = output ;
		}
        }
        else if (strcmp(input, "ParentRelative") == 0) {
                retcode = True ;
                output  = ParentRelative ;
			
		if (closure != (XtPointer *) 0) {
			*closure = output ;
		}
        }
        else {
                if ((path = XtResolvePathname(display, "bitmaps", input, ".xpm", NULL, NULL, 0, NULL)) != (String) 0) {
                        if (XpmReadFileToPixmap(display,
                                                DefaultRootWindow(display),
                                                path,
                                               &output,
                                               (Pixmap *) 0,
                                               (XpmAttributes *) 0) != XpmSuccess) {
                                output = None ;
                        }
                }
 
                if (output == None) {
                        XtStringConversionWarning((path ? path : input), "Pixmap") ;
 
                        retcode = False ;
                }
                else {
			if (closure != (XtPointer *) 0) {
				*closure = (XtPointer) output ;
			}
                        
			retcode = True ;
                }
        }
 
        /*
        ** Some systems (VMS) XtResolvePathname() does NOT always return heap copy
        */
 
        if ((path != (String) 0) && (path != input)) {
                XtFree(path) ;
        }
 
        return retcode ;
}
#endif /* defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_VIEW3D) || defined(INTEGRATE_CHARTOBJECT) */
#endif /* INTEGRATE_XDESIGNER */

/*
** _XdCvtMapTokenNewlines:
**
** utility function: maps soft quoted \n to real newline in a string
**                   returns heap allocated copy
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
static String _XdCvtMapTokenNewlines(String input)
#else  /* _NO_PROTO */
static String _XdCvtMapTokenNewlines(input)
        String input ;
#endif /* _NO_PROTO */
{
        String output = (String) 0 ;
        int    ii     = 0 ;
        int    oi     = 0 ;
 
        if (input == (String) 0) {
                return (String) 0 ;
        }
 
        output = (String) XtMalloc((strlen(input) + 1) * sizeof(char)) ;
 
        for (ii = 0, oi = 0 ; input[ii] != '\0' ; ii++, oi++) {
                if (input[ii] == '\\') {
                        if (input[ii + 1] == 'n') {
                                output[oi] = '\n' ;
                                ii++ ;
                        }
                        else {
                                output[oi++] = input[ii++] ;
                                output[oi]   = input[ii] ;
                        }
                } else {
                        output[oi] = input[ii] ;
                }
        }
 
        output[oi] = '\0' ;
 
        return output ;
}

#if       defined(INTEGRATE_EPAK) || defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT)
/*
** XdCvtStringToTranslationTable:
**
** takes standard Xt Translations string turns into an XtTranslations record
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToTranslationTable(Display    *display,
                                      XrmValuePtr argv,
                                      Cardinal   *argc,
                                      XrmValuePtr from_address,
                                      XrmValuePtr to_address,
                                      XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToTranslationTable(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static  XtTranslations output = (XtTranslations) 0 ;
                String         input  = (from_address ? (String) from_address->addr : (String) 0) ;
                String         temp   = (String) 0 ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if ((temp = _XdCvtMapTokenNewlines(input)) == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                return True ;
        }
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        if ((output = XtParseTranslationTable(temp)) == (XtTranslations) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = 0 ;
 
                XtFree(temp) ;
 
                XtStringConversionWarning(input, "TranslationTable") ;
 
                return False ;
        }
 
        XtFree(temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}
#endif /* defined(INTEGRATE_EPAK) || defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT) */


#ifdef    INTEGRATE_EPAK 
/*
** XdCvtStringToPixelList:
**
** takes comma separated list of color names (or pixel integers) and turns into an array of pixels
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToPixelList(Display    *display,
                               XrmValuePtr argv,
                               Cardinal   *argc,
                               XrmValuePtr from_address,
                               XrmValuePtr to_address,
                               XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToPixelList(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static Pixel        *output   = (Pixel *) 0 ;
               String        input    = (from_address ? (String) from_address->addr : (String) 0) ;
               String        token    = (String) 0 ;
               String        temp     = (String) 0 ;
               int           count    = 0 ;
               Colormap      colormap = (Colormap) 0 ;
               Pixel         pixel ;
               XColor        color ;
               XColor        exact ;
               Status        status ;
               char          tokeniser[MAX_TOKENISER_STRING + 1] ;
               unsigned long pixel_array[1] ;
               unsigned long plane_array[1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        /* We need a Colormap for this */
 
        if ((colormap = XDefaultColormap(display, DefaultScreen(display))) == (Colormap) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = 0 ;
 
                XtStringConversionWarning(input, "PixelList") ;
 
                return False ;
        }
 
        /* Deduce the input string to be converted */
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree(temp) ;
 
                return True ;
        }
 
        output = (Pixel *) XtMalloc((unsigned) (count + 1) * sizeof(Pixel)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
                if (isdigit(token[0])) {
                        pixel = (Pixel) atoi(token) ;
                }
                else {
                        if ((status = XAllocColorCells(display, colormap, False, plane_array, 0, pixel_array, 1)) == 0) {
                                to_address->addr = (XtPointer) 0 ;
                                to_address->size = 0 ;
 
                                XtFree(temp) ;
                                XtFree((char *) output) ;
 
                                XtStringConversionWarning(input, "PixelList") ;
 
                                return False ;
                        }
 
                        color.pixel = pixel_array[0] ;
                        color.flags = DoRed | DoGreen | DoBlue ;
 
                        if (token[0] == '#') {
                                XParseColor(display, colormap, token, &color) ;
                        }
                        else {
                                XLookupColor(display, colormap, token, &color, &exact) ;
 
                                /*
                                ** Porting Note
                                **
                                ** Some X libraries (IBM) mangle the pixel value of colour in the XLookupColor
                                */
 
                                color.pixel = pixel_array[0] ;
                        }
 
                        pixel = color.pixel ;
                }
 
                output[count] = pixel ;
 
                output = (Pixel *) XtRealloc((char *) output, ((++count) + 1) * sizeof(Pixel)) ;
                token  = strtok((String) 0, tokeniser) ;
        }
 
        output[count] = (Pixel) 0 ;
 
        XtFree(temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}
#endif /* INTEGRATE_EPAK */
 
/*
** _XdCvtStringToIntegerArray():
**
** Generic Mapper for String onto an array of integers of some kind
**
** Not really a converter, but a back end for one
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
static Boolean _XdCvtStringToIntegerArray(Display     *display,
                                          XrmValuePtr  argv,
					  Cardinal    *argc,
					  XrmValuePtr  from_address,
					  XrmValuePtr  to_address,
					  XtPointer   *closure,
					  String       converter_type,
					  int          last_value)
#else  /* _NO_PROTO */
static Boolean _XdCvtStringToIntegerArray(display, argv, argc, from_address, to_address, closure, converter_type, last_value)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
        String       converter_type ;
        int          last_value ;
#endif /* _NO_PROTO */
{
        static int   *output = (int *) 0 ;
               String input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String temp   = (String) 0 ;
               String token  = (String) 0 ;
               int    count  = 0 ;
               char   tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (int *) XtMalloc((unsigned) (count + 1) * sizeof(int)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
                if (!isdigit(token[0]) && (token[0] != '-')) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, converter_type) ;
 
                        return False ;
                }
 
                /* Negative values map onto -1 throughout */
 
                if (token[0] == '-') {
                        output[0] = -1 ;
                }
                else {
                        output[0] = atoi(token) ;
                }
 
                output = (int *) XtRealloc((char *) output, (++count) * sizeof(int)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }
 
        output[count] = last_value ;
 
        XtFree((char *) temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}

/*
** _XdCvtStringToStringArray():
**
** Generic Mapper for String onto an array of strings of some kind
**
** Not really a converter, but a back end for one
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
static Boolean _XdCvtStringToStringArray(Display     *display,
                                         XrmValuePtr  argv,
					 Cardinal    *argc,
					 XrmValuePtr  from_address,
					 XrmValuePtr  to_address,
					 XtPointer   *closure,
					 String       converter_type,
					 int          last_value)
#else  /* _NO_PROTO */
static Boolean _XdCvtStringToStringArray(display, argv, argc, from_address, to_address, closure, converter_type, last_value)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
        String       converter_type ;
        int          last_value ;
#endif /* _NO_PROTO */
{
        static String *output = (String *) 0 ;
               String  input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String  temp   = (String) 0 ;
               String  token  = (String) 0 ;
               int     count  = 0 ;
               char    tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (String *) XtMalloc((unsigned) (count + 1) * sizeof(String)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
                output[count] = _XdCvtMapTokenNewlines(token) ;
 
                output = (String *) XtRealloc((char *) output, (++count) * sizeof(String)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }

        output[count] = (String) last_value ;
 
        XtFree((char *) temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToStringArray():
**
** Mapper for String onto an array of simple strings
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToStringArray(Display     *display,
                                 XrmValuePtr  argv,
                                 Cardinal    *argc,
                                 XrmValuePtr  from_address,
                                 XrmValuePtr  to_address,
                                 XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToStringArray(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
	return _XdCvtStringToStringArray(display, argv, argc, from_address, to_address, closure, "StringArray", 0) ;
}
#endif /* INTEGRATE_CHARTOBJECT */

/*
** _XdCvtStringToEnumeratedArray():
**
** Generic Mapper for String onto an array of enumerated integers of some kind
**
** Not really a converter, but a back end for one
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
static Boolean _XdCvtStringToEnumeratedArray(Display       *display,
                                             XrmValuePtr    argv,
					     Cardinal      *argc,
					     XrmValuePtr    from_address,
					     XrmValuePtr    to_address,
					     XtPointer     *closure,
					     String         converter_type,
					     int            last_value,
					     NameValueMap_p map,
					     char          *skip_family,   /* Any XmN we can ignore                            */
					     char          *skip_prefix)   /* Any prefix for the map (ALIGNMENT) we can ignore */
#else  /* _NO_PROTO */
static Boolean _XdCvtStringToEnumeratedArray(display, 
                                             argv,
					     argc, 
					     from_address, 
					     to_address, 
					     closure, 
					     converter_type, 
					     last_value,
					     map,
					     skip_family, 
					     skip_prefix)
        Display       *display ;
        XrmValuePtr    argv ;
        Cardinal      *argc ;
        XrmValuePtr    from_address ;
        XrmValuePtr    to_address ;
        XtPointer     *closure ;
        String         converter_type ;
        int            last_value ;
	NameValueMap_p map ;
	char          *skip_family ;
	char          *skip_prefix ;
#endif /* _NO_PROTO */
{
        static int           *output = (int *) 0 ;
               String         input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String         temp   = (String) 0 ;
               String         token  = (String) 0 ;
               int            count  = 0 ;
               char           tokeniser[MAX_TOKENISER_STRING + 1] ;
	       Boolean        found  = False ;
	       NameValueMap_p mptr ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (int *) XtMalloc((unsigned) (count + 1) * sizeof(int)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
                if (!isdigit(token[0])) {
			XrmValue from ;
			XrmValue to ;

			from.addr = (XtPointer) token ;
			from.size = strlen(token) ;

			to.addr   = (XtPointer) 0 ;
			to.size   = 0 ;

			if ((_XdCvtStringToEnum(display, 
					        argv,
					        argc,
					       &from,
					       &to,
					        closure,
					        map,
					        skip_family,
					        skip_prefix,
					        converter_type) == False) || (to.size == 0)) {
				to_address->addr = (XtPointer) 0 ;
				to_address->size = sizeof(XtPointer) ;

				XtFree((char *) output) ;
				XtFree(temp) ;

				return False ;
			}

			output[count] = *((int *) to.addr) ;
                }
                else {
                        output[count] = atoi(token) ;

			for (mptr = map ; mptr && mptr->name ; mptr++) {
				if (mptr->value == output[0]) {
					found = True ;
					break ;
				}
			}

			if (found == False) {
				to_address->addr = (XtPointer) 0 ;
				to_address->size = sizeof(XtPointer) ;
				
				XtFree((char *) output) ;
				XtFree(temp) ;

				return False ;
			}
                }
 
                output = (int *) XtRealloc((char *) output, (++count) * sizeof(int)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }
 
        output[count] = last_value ;
 
        XtFree((char *) temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}

/*
** _XdCvtStringToDoubleArray():
**
** Generic Mapper for String onto an array of doubles of some kind
**
** Not really a converter, but a back end for one
*/

#if       defined (DOUBLE_PRECISION) || defined (USE_DOUBLE)
#define   DOUBLE_CONVERSION_FORMAT     "%lg"
#else  /* defined (DOUBLE_PRECISION) || defined (USE_DOUBLE) */
#define   DOUBLE_CONVERSION_FORMAT     "%g"
#endif /* defined (DOUBLE_PRECISION) || defined (USE_DOUBLE) */
/* ARGSUSED */
#ifndef   _NO_PROTO
static Boolean _XdCvtStringToDoubleArray(Display     *display,
                                         XrmValuePtr  argv,
					 Cardinal    *argc,
					 XrmValuePtr  from_address,
					 XrmValuePtr  to_address,
					 XtPointer   *closure,
					 String       converter_type,
					 double       last_value)
#else  /* _NO_PROTO */
static Boolean _XdCvtStringToDoubleArray(display, argv, argc, from_address, to_address, closure, converter_type, last_value)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
        String       converter_type ;
        double       last_value ;
#endif /* _NO_PROTO */
{
        static double *output = (double *) 0 ;
               String  input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String  temp   = (String) 0 ;
               String  token  = (String) 0 ;
	       double  value ;
               int     count  = 0 ;
               char    tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (double *) XtMalloc((unsigned) (count + 1) * sizeof(double)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
		value = (double) 0 ;

		if (sscanf(token, DOUBLE_CONVERSION_FORMAT, &value) != 1) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, converter_type) ;
 
                        return False ;
		}

		output[count] = value ;
 
                output = (double *) XtRealloc((char *) output, (++count) * sizeof(double)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }
 
        output[count] = last_value ;
 
        XtFree((char *) temp) ;
	
	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDoubleArray():
**
** Mapper for String onto a simple array of doubles
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToDoubleArray(Display     *display,
                                 XrmValuePtr  argv,
                                 Cardinal    *argc,
                                 XrmValuePtr  from_address,
                                 XrmValuePtr  to_address,
                                 XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToDoubleArray(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
	return _XdCvtStringToDoubleArray(display, argv, argc, from_address, to_address, closure, "DoubleArray", (double) 0) ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToCellHeightData():
**
** Mapper for String onto EditTable CellHeightData integer list
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToCellHeightData(Display     *display,
                                    XrmValuePtr  argv,
				    Cardinal    *argc,
				    XrmValuePtr  from_address,
				    XrmValuePtr  to_address,
				    XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToCellHeightData(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
	extern Boolean xintEditTableCvtCellHeightData ;

	Boolean retcode ;

	retcode = _XdCvtStringToIntegerArray(display, 
					     argv,
					     argc, 
					     from_address, 
					     to_address, 
					     closure,
					     "CellHeightData", 
					     OUT_OF_BOUNDS_MARKER) ;

	if ((retcode == True) && (to_address->size > 0)) {
		xintEditTableCvtCellHeightData = True ;
	}

	return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE 
/*
** XdCvtStringToCellWidthData():
**
** Mapper for String onto EditTable CellWidthData integer list
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToCellWidthData(Display     *display,
                                   XrmValuePtr  argv,
                                   Cardinal    *argc,
                                   XrmValuePtr  from_address,
                                   XrmValuePtr  to_address,
                                   XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToCellWidthData(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
        extern Boolean xintEditTableCvtCellWidthData ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToIntegerArray(display, 
					     argv, 
					     argc, 
					     from_address, 
					     to_address, 
					     closure,
					     "CellWidthData", 
					     OUT_OF_BOUNDS_MARKER) ;
 
        if ((retcode == True) && (to_address->size > 0)) {
                xintEditTableCvtCellWidthData = True ;
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToColumnAlignmentData:
**
** converts an XintALIGNMENT_BEGINNING_TOP, XintALIGNMENT_BEGINNING_END,.. string list into an array of integers
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColumnAlignmentData(Display    *display,
                                         XrmValuePtr argv,
                                         Cardinal   *argc,
                                         XrmValuePtr from_address,
                                         XrmValuePtr to_address,
                                         XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColumnAlignmentData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtColumnAlignmentData ;
 
        static NameValueMap_t column_alignment_map[] =
        {
                { "BEGINNING_TOP",    XintALIGNMENT_BEGINNING_TOP    },
                { "CENTER_TOP",       XintALIGNMENT_CENTER_TOP       },
                { "END_TOP",          XintALIGNMENT_END_TOP          },
                { "BEGINNING_MIDDLE", XintALIGNMENT_BEGINNING_MIDDLE },
                { "CENTER_MIDDLE",    XintALIGNMENT_CENTER_MIDDLE    },
                { "END_MIDDLE",       XintALIGNMENT_END_MIDDLE       },
                { "BEGINNING_BOTTOM", XintALIGNMENT_BEGINNING_BOTTOM },
                { "CENTER_BOTTOM",    XintALIGNMENT_CENTER_BOTTOM    },
                { "END_BOTTOM",       XintALIGNMENT_END_BOTTOM       },
                { (char *) 0,         0                              }
        } ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToEnumeratedArray(display,
                                                argv,
                                                argc,
                                                from_address,
                                                to_address,
                                                closure,
                                                "ColumnAlignmentData",
                                                OUT_OF_BOUNDS_MARKER,
                                                column_alignment_map,
                                                "Xint",
                                                "ALIGNMENT_") ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtColumnAlignmentData = True ;
                }
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToColumnAnnotationData:
**
** converts a string containing annotation specifiers into a StringArray
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColumnAnnotationData(Display    *display,
                                          XrmValuePtr argv,
                                          Cardinal   *argc,
                                          XrmValuePtr from_address,
                                          XrmValuePtr to_address,
                                          XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColumnAnnotationData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtColumnAnnotationData ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToStringArray(display,
                                            argv,
                                            argc,
                                            from_address,
                                            to_address,
                                            closure,
                                            "ColumnAnnotationData",
                                            OUT_OF_BOUNDS_MARKER) ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtColumnAnnotationData = True ;
                }
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToColumnDataFormatData:
**
** converts a string containing format specifiers into a StringArray
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColumnDataFormatData(Display    *display,
                                          XrmValuePtr argv,
                                          Cardinal   *argc,
                                          XrmValuePtr from_address,
                                          XrmValuePtr to_address,
                                          XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColumnDataFormatData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtColumnDataFormatData ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToStringArray(display,
                                            argv,
                                            argc,
                                            from_address,
                                            to_address,
                                            closure,
                                            "ColumnDataFormatData",
                                            OUT_OF_BOUNDS_MARKER) ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtColumnDataFormatData = True ;
                }
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToColumnDataTypeData:
**
** converts an XintTYPE_SHORT, XintTYPE_LONG,.. string list into an array of integers
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColumnDataTypeData(Display    *display,
                                        XrmValuePtr argv,
                                        Cardinal   *argc,
                                        XrmValuePtr from_address,
                                        XrmValuePtr to_address,
                                        XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColumnDataTypeData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtColumnDataTypeData ;
 
        static NameValueMap_t column_data_type_map[] =
        {
                { "SHORT",    XintTYPE_SHORT   },
                { "INTEGER",  XintTYPE_INTEGER },
                { "LONG",     XintTYPE_LONG    },
                { "FLOAT",    XintTYPE_FLOAT   },
                { "DOUBLE",   XintTYPE_DOUBLE  },
                { "STRING",   XintTYPE_STRING  },
                { "POINTER",  XintTYPE_POINTER },
                { (char *) 0, 0                }
        } ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToEnumeratedArray(display,
                                                argv,
                                                argc,
                                                from_address,
                                                to_address,
                                                closure,
                                                "ColumnDataTypeData",
                                                OUT_OF_BOUNDS_MARKER,
                                                column_data_type_map,
                                                "Xint",
                                                "TYPE_") ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtColumnDataTypeData = True ;
                }
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToColumnEditModeData:
**
** converts an XintCOLUMN_EDITABLE, XintCOLUMN_NON_EDITABLE,.. string list into an array of integers
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColumnEditModeData(Display    *display,
                                        XrmValuePtr argv,
                                        Cardinal   *argc,
                                        XrmValuePtr from_address,
                                        XrmValuePtr to_address,
                                        XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColumnEditModeData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtColumnEditModeData ;
 
        static NameValueMap_t column_edit_mode_map[] =
        {
                { "EDITABLE",     XintCOLUMN_EDITABLE     },
                { "NON_EDITABLE", XintCOLUMN_NON_EDITABLE },
                { (char *) 0,     0                       }
        } ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToEnumeratedArray(display,
                                                argv,
                                                argc,
                                                from_address,
                                                to_address,
                                                closure,
                                                "ColumnEditModeData",
                                                OUT_OF_BOUNDS_MARKER,
                                                column_edit_mode_map,
                                                "Xint",
                                                "COLUMN_") ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtColumnEditModeData = True ;
                }
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToColumnFontIndexData:
**
** converts a string containing integers into an IntegerArray
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColumnFontIndexData(Display    *display,
                                         XrmValuePtr argv,
                                         Cardinal   *argc,
                                         XrmValuePtr from_address,
                                         XrmValuePtr to_address,
                                         XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColumnFontIndexData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtColumnFontIndexData ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToIntegerArray(display,
                                             argv,
                                             argc,
                                             from_address,
                                             to_address,
                                             closure,
                                             "ColumnFontIndexData",
                                             OUT_OF_BOUNDS_MARKER) ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtColumnFontIndexData = True ;
                }
        }
 
        return retcode ;
}
#endif /* INTEGRATE_EDITTABLE */

#if       defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT)
/*
** XdCvtStringToFontTable:
**
** converts a string containing X font names into a StringArray
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToFontTable(Display    *display,
                               XrmValuePtr argv,
                               Cardinal   *argc,
                               XrmValuePtr from_address,
                               XrmValuePtr to_address,
                               XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToFontTable(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtFontTable ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToStringArray(display,
                                            argv,
                                            argc,
                                            from_address,
                                            to_address,
                                            closure,
                                            "FontTable",
                                            OUT_OF_BOUNDS_MARKER) ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtFontTable = True ;
                }
        }
 
        return retcode ;
}
#endif /* defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT) */

#if       defined(INTEGRATE_CHARTOBJECT)
/*
** Holder for some object classes for the Object Class converter
*/

typedef struct XintObjectClassRecord_s 
{
	ObjectClass object_class ;
	char       *object_name ;
} XintObjectClassRecord_t, *XintObjectClassRecord_p ;

extern ObjectClass xintAxisObjectClass ;
extern ObjectClass xintBarObjectClass ;
extern ObjectClass xintBar3DObjectClass ;
extern ObjectClass xintBarLineObjectClass ;
extern ObjectClass xintBarSeriesObjectClass ;
extern ObjectClass xintCellArrayObjectClass ;
extern ObjectClass xintChartObjectClass ;
extern ObjectClass xintChartBaseObjectClass ;
extern ObjectClass xintComboPlotObjectClass ;
extern ObjectClass xintDataObjectClass ;
extern ObjectClass xintDataFlagObjectClass ;
extern ObjectClass xintDataGridObjectClass ;
extern ObjectClass xintDataGroupObjectClass ;
extern ObjectClass xintDataLabelObjectClass ;
extern ObjectClass xintDataSampledObjectClass ;
extern ObjectClass xintDataSequentialSeriesObjectClass ;
extern ObjectClass xintDataSeriesObjectClass ;
extern ObjectClass xintDataTimeLabelObjectClass ;
extern ObjectClass xintFreeHandObjectClass ;
extern ObjectClass xintGraphicObjectClass ;
extern ObjectClass xintGridObjectClass ;
extern ObjectClass xintGroupObjectClass ;
extern ObjectClass xintHighLowObjectClass ;
extern ObjectClass xintHighLowSeriesObjectClass ;
extern ObjectClass xintHistogramObjectClass ;
extern ObjectClass xintHistogramSeriesObjectClass ;
extern ObjectClass xintImageObjectClass ;
extern ObjectClass xintLegendObjectClass ;
extern ObjectClass xintLineObjectClass ;
extern ObjectClass xintLineSeriesObjectClass ;
extern ObjectClass xintMultiPointObjectClass ;
extern ObjectClass xintOvalObjectClass ;
extern ObjectClass xintPieObjectClass ;
extern ObjectClass xintPlot2DObjectClass ;
extern ObjectClass xintPlot3DObjectClass ;
extern ObjectClass xintPolySeriesObjectClass ;
extern ObjectClass xintPolygonObjectClass ;
extern ObjectClass xintPolylineObjectClass ;
extern ObjectClass xintRectangleObjectClass ;
extern ObjectClass xintRoundedRectObjectClass ;
extern ObjectClass xintScaledTextObjectClass ;
extern ObjectClass xintSurface3DObjectClass ;
extern ObjectClass xintSymbolObjectClass ;
extern ObjectClass xintTextObjectClass ;
extern ObjectClass xintViewPortObjectClass ;
extern ObjectClass xintWedgeObjectClass ;
extern ObjectClass xintWedgeSeriesObjectClass ;
extern ObjectClass xintXYPlotObjectClass ;

static XintObjectClassRecord_t int_object_classes[] =
{
	{ (ObjectClass) 0, "xintAxisObjectClass"                 },
	{ (ObjectClass) 0, "xintBarObjectClass"                  },
	{ (ObjectClass) 0, "xintBar3DObjectClass"                },
	{ (ObjectClass) 0, "xintBarLineObjectClass"              },
	{ (ObjectClass) 0, "xintBarSeriesObjectClass"            },
	{ (ObjectClass) 0, "xintCellArrayObjectClass"            },
	{ (ObjectClass) 0, "xintChartObjectClass"                },
	{ (ObjectClass) 0, "xintChartBaseObjectClass"            },
	{ (ObjectClass) 0, "xintComboPlotObjectClass"            },
	{ (ObjectClass) 0, "xintDataObjectClass"                 },
	{ (ObjectClass) 0, "xintDataFlagObjectClass"             },
	{ (ObjectClass) 0, "xintDataGridObjectClass"             },
	{ (ObjectClass) 0, "xintDataGroupObjectClass"            },
	{ (ObjectClass) 0, "xintDataLabelObjectClass"            },
	{ (ObjectClass) 0, "xintDataSampledObjectClass"          },
	{ (ObjectClass) 0, "xintDataSequentialSeriesObjectClass" },
	{ (ObjectClass) 0, "xintDataSeriesObjectClass"           },
	{ (ObjectClass) 0, "xintDataTimeLabelObjectClass"        },
	{ (ObjectClass) 0, "xintFreeHandObjectClass"             },
	{ (ObjectClass) 0, "xintGraphicObjectClass"              },
	{ (ObjectClass) 0, "xintGridObjectClass"                 },
	{ (ObjectClass) 0, "xintGroupObjectClass"                },
	{ (ObjectClass) 0, "xintHighLowObjectClass"              },
	{ (ObjectClass) 0, "xintHighLowSeriesObjectClass"        },
	{ (ObjectClass) 0, "xintHistogramObjectClass"            },
	{ (ObjectClass) 0, "xintHistogramSeriesObjectClass"      },
	{ (ObjectClass) 0, "xintImageObjectClass"                },
	{ (ObjectClass) 0, "xintLegendObjectClass"               },
	{ (ObjectClass) 0, "xintLineObjectClass"                 },
	{ (ObjectClass) 0, "xintLineSeriesObjectClass"           },
	{ (ObjectClass) 0, "xintMultiPointObjectClass"           },
	{ (ObjectClass) 0, "xintOvalObjectClass"                 },
	{ (ObjectClass) 0, "xintPieObjectClass"                  },
	{ (ObjectClass) 0, "xintPlot2DObjectClass"               },
	{ (ObjectClass) 0, "xintPlot3DObjectClass"               },
	{ (ObjectClass) 0, "xintPolySeriesObjectClass"           },
	{ (ObjectClass) 0, "xintPolygonObjectClass"              },
	{ (ObjectClass) 0, "xintPolylineObjectClass"             },
	{ (ObjectClass) 0, "xintRectangleObjectClass"            },
	{ (ObjectClass) 0, "xintRoundedRectObjectClass"          },
	{ (ObjectClass) 0, "xintScaledTextObjectClass"           },
	{ (ObjectClass) 0, "xintSurface3DObjectClass"            },
	{ (ObjectClass) 0, "xintSymbolObjectClass"               },
	{ (ObjectClass) 0, "xintTextObjectClass"                 },
	{ (ObjectClass) 0, "xintViewPortObjectClass"             },
	{ (ObjectClass) 0, "xintWedgeObjectClass"                },
	{ (ObjectClass) 0, "xintWedgeSeriesObjectClass"          },
	{ (ObjectClass) 0, "xintXYPlotObjectClass"               },
	{ (ObjectClass) 0, (char *) 0                            }
} ;


/*
** XdGetObjectClass:
**
** simple routine to find a class given the name from the above table
*/
/* ARGSUSED */
#ifndef   _NO_PROTO
static ObjectClass *XdGetObjectClass(String input)
#else  /* _NO_PROTO */
static ObjectClass *XdGetObjectClass(input)
	String input ;
#endif /* _NO_PROTO */
{
	register XintObjectClassRecord_p optr  = int_object_classes ;

	register String                  tptr  = input ;
	static   Boolean                 first = True ;

	if (first == True) {
		first = False ;

		int_object_classes[ 0].object_class = xintAxisObjectClass ;
		int_object_classes[ 1].object_class = xintBarObjectClass ;
		int_object_classes[ 2].object_class = xintBar3DObjectClass ;
		int_object_classes[ 3].object_class = xintBarLineObjectClass ;
		int_object_classes[ 4].object_class = xintBarSeriesObjectClass ;
		int_object_classes[ 5].object_class = xintCellArrayObjectClass ;
		int_object_classes[ 6].object_class = xintChartObjectClass ;
		int_object_classes[ 7].object_class = xintChartBaseObjectClass ;
		int_object_classes[ 8].object_class = xintComboPlotObjectClass ;
		int_object_classes[ 9].object_class = xintDataObjectClass ;
		int_object_classes[10].object_class = xintDataFlagObjectClass ;
		int_object_classes[11].object_class = xintDataGridObjectClass ;
		int_object_classes[12].object_class = xintDataGroupObjectClass ;
		int_object_classes[13].object_class = xintDataLabelObjectClass ;
		int_object_classes[14].object_class = xintDataSampledObjectClass ;
		int_object_classes[15].object_class = xintDataSequentialSeriesObjectClass ;
		int_object_classes[16].object_class = xintDataSeriesObjectClass ;
		int_object_classes[17].object_class = xintDataTimeLabelObjectClass ;
		int_object_classes[18].object_class = xintFreeHandObjectClass ;
		int_object_classes[19].object_class = xintGraphicObjectClass ;
		int_object_classes[20].object_class = xintGridObjectClass ;
		int_object_classes[21].object_class = xintGroupObjectClass ;
		int_object_classes[22].object_class = xintHighLowObjectClass ;
		int_object_classes[23].object_class = xintHighLowSeriesObjectClass ;
		int_object_classes[24].object_class = xintHistogramObjectClass ;
		int_object_classes[25].object_class = xintHistogramSeriesObjectClass ;
		int_object_classes[26].object_class = xintImageObjectClass ;
		int_object_classes[27].object_class = xintLegendObjectClass ;
		int_object_classes[28].object_class = xintLineObjectClass ;
		int_object_classes[29].object_class = xintLineSeriesObjectClass ;
		int_object_classes[30].object_class = xintMultiPointObjectClass ;
		int_object_classes[31].object_class = xintOvalObjectClass ;
		int_object_classes[32].object_class = xintPieObjectClass ;
		int_object_classes[33].object_class = xintPlot2DObjectClass ;
		int_object_classes[34].object_class = xintPlot3DObjectClass ;
		int_object_classes[35].object_class = xintPolySeriesObjectClass ;
		int_object_classes[36].object_class = xintPolygonObjectClass ;
		int_object_classes[37].object_class = xintPolylineObjectClass ;
		int_object_classes[38].object_class = xintRectangleObjectClass ;
		int_object_classes[39].object_class = xintRoundedRectObjectClass ;
		int_object_classes[30].object_class = xintScaledTextObjectClass ;
		int_object_classes[40].object_class = xintSurface3DObjectClass ;
		int_object_classes[41].object_class = xintSymbolObjectClass ;
		int_object_classes[42].object_class = xintTextObjectClass ;
		int_object_classes[43].object_class = xintViewPortObjectClass ;
		int_object_classes[44].object_class = xintWedgeObjectClass ;
		int_object_classes[45].object_class = xintWedgeSeriesObjectClass ;
		int_object_classes[46].object_class = xintXYPlotObjectClass ;
	}

	if (STRNCASECMP(input, "xint", strlen("xint")) == 0) {
		tptr += strlen("xint") ;
	}

	while ((optr != (XintObjectClassRecord_p) 0) && (optr->object_class != (ObjectClass) 0)) {
		if (STRCASECMP(optr->object_name, tptr) == 0) {
			return &optr->object_class ;
		}
	}

	return (ObjectClass *) 0 ;
}

/*
** XdCvtStringToObjectClassList:
**
** converts a string containing object class names into an ObjectClass array
**
** Required for EditObject widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToObjectClassList(Display    *display,
                                     XrmValuePtr argv,
				     Cardinal   *argc,
				     XrmValuePtr from_address,
				     XrmValuePtr to_address,
				     XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToObjectClassList(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static ObjectClass *output = (ObjectClass *) 0 ;
	       ObjectClass *oclass = (ObjectClass *) 0 ;
               String       input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String       temp   = (String) 0 ;
               String       token  = (String) 0 ;
               int          count  = 0 ;
               char         tokeniser[MAX_TOKENISER_STRING + 1] ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) 0 ;
        }
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (ObjectClass *) XtMalloc((unsigned) (count + 1) * sizeof(ObjectClass)) ;

        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
		if ((oclass = XdGetObjectClass(token)) == (ObjectClass *) 0) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, "ObjectClassList") ;
 
                        return False ;
		}
 
                output[count] = *oclass ;
 
                output = (ObjectClass *) XtRealloc((char *) output, (++count) * sizeof(ObjectClass)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }
 
        output[count] = (ObjectClass) 0 ;
 
        XtFree((char *) temp) ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) output ;
        }
 
        return True ;
}
#endif /* defined(INTEGRATE_CHARTOBJECT) */

#if       defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT)
/*
** Holder for some pixmaps builtin to the EditObject
*/
 
typedef struct XintPixmapsRecord_s
{
        Pixmap pixmap ;
        char  *pixmap_name ;
} XintPixmapsRecord_t, *XintPixmapsRecord_p ;

static XintPixmapsRecord_t int_pixmaps[] =
{
	{ (Pixmap) 0, "Solid",            },
	{ (Pixmap) 0, "Clear",            },
	{ (Pixmap) 0, "25_percent_1",     },
	{ (Pixmap) 0, "25_percent_2",     },
	{ (Pixmap) 0, "25_percent_3",     },
	{ (Pixmap) 0, "25_percent_4",     },
	{ (Pixmap) 0, "50_percent_1",     },
	{ (Pixmap) 0, "50_percent_2",     },
	{ (Pixmap) 0, "75_percent",       },
	{ (Pixmap) 0, "Vertical",         },
	{ (Pixmap) 0, "Vertical1",        },
	{ (Pixmap) 0, "Vertical2",        },
	{ (Pixmap) 0, "Vertical3",        },
	{ (Pixmap) 0, "Horizontal",       },
	{ (Pixmap) 0, "Horizontal1",      },
	{ (Pixmap) 0, "Horizontal2",      },
	{ (Pixmap) 0, "Horizontal3",      },
	{ (Pixmap) 0, "Slant_Left",       },
	{ (Pixmap) 0, "Slant_Left1",      },
	{ (Pixmap) 0, "Slant_Left2",      },
	{ (Pixmap) 0, "Star_Left1",       },
	{ (Pixmap) 0, "Star_Left2",       },
	{ (Pixmap) 0, "Star_Right1",      },
	{ (Pixmap) 0, "Star_Right2",      },
	{ (Pixmap) 0, "Slant_Right",      },
	{ (Pixmap) 0, "Slant_Right1",     },
	{ (Pixmap) 0, "Slant_Right2",     },
	{ (Pixmap) 0, "Zigzag",           },
	{ (Pixmap) 0, "Zigzag1",          },
	{ (Pixmap) 0, "Zigzag2",          },
	{ (Pixmap) 0, "Tread",            },
	{ (Pixmap) 0, "Tread1",           },
	{ (Pixmap) 0, "Tread2",           },
	{ (Pixmap) 0, "Trellis",          },
	{ (Pixmap) 0, "Trellis1",         },
	{ (Pixmap) 0, "Trellis2",         },
	{ (Pixmap) 0, "Weave1",           },
	{ (Pixmap) 0, "Weave2",           },
	{ (Pixmap) 0, "Weave3",           },
	{ (Pixmap) 0, "Weave4",           },
	{ (Pixmap) 0, "Weave1",           },
	{ (Pixmap) 0, "12_5_percent_1",   },
	{ (Pixmap) 0, "12_5_percent_2",   },
	{ (Pixmap) 0, "12_5_percent_3",   },
	{ (Pixmap) 0, "12_5_percent_4",   },
	{ (Pixmap) 0, "12_5_percent_5",   },
	{ (Pixmap) 0, "12_5_percent_6",   },
	{ (Pixmap) 0, "12_5_percent_7",   },
	{ (Pixmap) 0, "12_5_percent_8",   },
	{ (Pixmap) 0, "Sandstone",        },
	{ (Pixmap) 0, "Siltstone",        },
	{ (Pixmap) 0, "Shale",            },
	{ (Pixmap) 0, "Limestone",        },
	{ (Pixmap) 0, "Dolomite",         },
	{ (Pixmap) 0, "Chert",            },
	{ (Pixmap) 0, "Basement",         },
	{ (Pixmap) 0, "Check1",           },
	{ (Pixmap) 0, "Check2",           },
	{ (Pixmap) 0, "Check3",           },
	{ (Pixmap) 0, "Wicker1",          },
	{ (Pixmap) 0, "Wicker2",          },
	{ (Pixmap) 0, "Wicker3",          },
	{ (Pixmap) 0, "Cross_Hatch",      },
	{ (Pixmap) 0, "Diamond1",         },
	{ (Pixmap) 0, "Diamond2",         },
	{ (Pixmap) 0, "Diamond3",         },
	{ (Pixmap) 0, "Lattice1",         },
	{ (Pixmap) 0, "Lattice2",         },
	{ (Pixmap) 0, "Lattice3",         },
	{ (Pixmap) 0, "Lattice4",         },
	{ (Pixmap) 0, "Lattice5",         },
	{ (Pixmap) 0, "Lattice6",         },
	{ (Pixmap) 0, "Lattice7",         },
	{ (Pixmap) 0, "Herring_Bone1",    },
	{ (Pixmap) 0, "Herring_Bone2",    },
	{ (Pixmap) 0, "Plaid1",           },
	{ (Pixmap) 0, "Plaid2",           },
	{ (Pixmap) 0, "Plaid3",           },
	{ (Pixmap) 0, "Plaid4",           },
	{ (Pixmap) 0, "Imbrication1",     },
	{ (Pixmap) 0, "Imbrication2",     },
	{ (Pixmap) 0, "Imbrication3",     },
	{ (Pixmap) 0, "Imbrication4",     },
	{ (Pixmap) 0, "Diaper",           },
	{ (Pixmap) 0, "Parquet1",         },
	{ (Pixmap) 0, "Parquet2",         },
	{ (Pixmap) 0, "Horizontal_Bar1",  },
	{ (Pixmap) 0, "Horizontal_Bar2",  },
	{ (Pixmap) 0, "Horizontal_Bar3",  },
	{ (Pixmap) 0, "Horizontal_Bar4",  },
	{ (Pixmap) 0, "Horizontal_Bar5",  },
	{ (Pixmap) 0, "Horizontal_Bar6",  },
	{ (Pixmap) 0, "Horizontal_Bar7",  },
	{ (Pixmap) 0, "Horizontal_Bar8",  },
	{ (Pixmap) 0, "Horizontal_Bar9",  },
	{ (Pixmap) 0, "Horizontal_Bar10", },
	{ (Pixmap) 0, "Vertical_Bar1",    },
	{ (Pixmap) 0, "Vertical_Bar2",    },
	{ (Pixmap) 0, "Vertical_Bar3",    },
	{ (Pixmap) 0, "Vertical_Bar4",    },
	{ (Pixmap) 0, "Vertical_Bar5",    },
	{ (Pixmap) 0, "Vertical_Bar6",    },
	{ (Pixmap) 0, "Vertical_Bar7",    },
	{ (Pixmap) 0, "Vertical_Bar8",    },
	{ (Pixmap) 0, "Vertical_Bar9",    },
	{ (Pixmap) 0, "Vertical_Bar10",   },
	{ (Pixmap) 0, (char *) 0          }
} ;

/*
** XdGetPixmap:
**
** simple routine to find a class given the name from the above table
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
static Pixmap *XdGetPixmap(String input)
#else  /* _NO_PROTO */
static Pixmap *XdGetPixmap(input)
        String input ;
#endif /* _NO_PROTO */
{
	extern Pixmap XintObjectEditorGetDefinedPixmap() ;

        register XintPixmapsRecord_p pptr = int_pixmaps ;
 
        while ((pptr != (XintPixmapsRecord_p) 0) && (pptr->pixmap_name != (String) 0)) {
                if (STRCASECMP(pptr->pixmap_name, input) == 0) {
			if (pptr->pixmap == (Pixmap) 0) {
				pptr->pixmap = XintObjectEditorGetDefinedPixmap((Widget) 0, pptr->pixmap_name) ;
			}

                        return &pptr->pixmap ;
                }
        }
 
        return (Pixmap *) 0 ;
}
 
/*
** XdCvtStringToPixmapList:
**
** converts a string containing builtin INT pixmap names into an PixmapList array
**
** Required for EditObject widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToPixmapList(Display    *display,
                                XrmValuePtr argv,
                                Cardinal   *argc,
                                XrmValuePtr from_address,
                                XrmValuePtr to_address,
                                XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToPixmapList(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static Pixmap *output = (Pixmap *) 0 ;
               Pixmap *pixmap = (Pixmap *) 0 ;
               String  input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String  temp   = (String) 0 ;
               String  token  = (String) 0 ;
               int     count  = 0 ;
               char    tokeniser[MAX_TOKENISER_STRING + 1] ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) 0 ;
        }
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (Pixmap *) XtMalloc((unsigned) (count + 1) * sizeof(Pixmap)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
                if ((pixmap = XdGetPixmap(token)) == (Pixmap *) 0) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, "PixmapList") ;
 
                        return False ;
                }
 
                output[count] = *pixmap ;
 
                output = (Pixmap *) XtRealloc((char *) output, (++count) * sizeof(Pixmap)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }

#ifndef   XmUNSPECIFIED_PIXMAP
#define   XmUNSPECIFIED_PIXMAP    2
#endif /* XmUNSPECIFIED_PIXMAP */

        output[count] = XmUNSPECIFIED_PIXMAP ;
 
        XtFree((char *) temp) ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) output ;
        }
 
        return True ;
}
#endif /* defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT) */

#ifdef    INTEGRATE_EDITTABLE
/*
** XdCvtStringToReferenceChar:
**
** converts a string containing an EditTable reference char
**
** Required for EditObject widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToReferenceChar(Display    *display,
                                   XrmValuePtr argv,
                                   Cardinal   *argc,
                                   XrmValuePtr from_address,
                                   XrmValuePtr to_address,
                                   XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToReferenceChar(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static int    output = (int) 0 ;
               String input  = (from_address ? (String) from_address->addr : (String) 0) ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) 0 ;
        } 

	output = 0 ;

	to_address->addr = (XtPointer) &output ;
	to_address->size = sizeof(int) ;

	if (input != (String) 0) {
		output = input[0] ;
	}

	if ((output < (int) ' ') || (output > (int) '~')) {
		to_address->addr = (XtPointer) 0 ;
		to_address->size = 0 ;

		XtStringConversionWarning(input, "ReferenceChar") ;

		return False ;
	}

	return True ;
}
#endif /* INTEGRATE_EDITTABLE */

#if       defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT)
/*
** XdCvtStringToRowAnnotationData:
**
** converts a string containing annotation specifiers into a StringArray
**
** Required for EditTable widget
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToRowAnnotationData(Display    *display,
                                       XrmValuePtr argv,
                                       Cardinal   *argc,
                                       XrmValuePtr from_address,
                                       XrmValuePtr to_address,
                                       XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToRowAnnotationData(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern Boolean xintEditTableCvtRowAnnotationData ;
 
        Boolean retcode ;
 
        retcode = _XdCvtStringToStringArray(display,
                                            argv,
                                            argc,
                                            from_address,
                                            to_address,
                                            closure,
                                            "RowAnnotationData",
                                            OUT_OF_BOUNDS_MARKER) ;
 
        if (retcode == True) {
                if (to_address->size > 0) {
                        xintEditTableCvtRowAnnotationData = True ;
                }
        }
 
        return retcode ;
}
#endif /* defined(INTEGRATE_EDITTABLE) || defined(INTEGRATE_CHARTOBJECT) */

#ifdef    INTEGRATE_VIEW3D
/*
** XdCvtStringToViewAngle:
**
** converts a string containing two doubles into an XintViewAngle structure
**
** Required for ChartObject widget
*/
/* ARGSUSED */
#ifndef  _NO_PROTO
Boolean XdCvtStringToViewAngle(Display    *display,
                               XrmValuePtr argv,
                               Cardinal   *argc,
                               XrmValuePtr from_address,
                               XrmValuePtr to_address,
                               XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToViewAngle(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static TwoDouble_t *output = (TwoDouble_t *) 0 ;
               double       azimuth ;
               double       dip ;
               String       input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String       temp   = (String) 0 ;
               String       token  = (String) 0 ;
               char         tokeniser[MAX_TOKENISER_STRING + 1] ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) 0 ;
        }

	output = (TwoDouble_t *) 0 ;
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                XtFree(temp) ;

                ConverterReturn(output, TwoDouble_t *, to_address) ;
        }
 
	if (sscanf(token, DOUBLE_CONVERSION_FORMAT, &azimuth) != 1) {
		XtFree(temp) ;
 
                XtStringConversionWarning(input, "ViewAngle") ;
                
		ConverterReturn(output, TwoDouble_t *, to_address) ;
	}


        if ((token = strtok((char *) 0, tokeniser)) == (String) 0) {
		XtFree(temp) ;
 
                XtStringConversionWarning(input, "ViewAngle") ;
		
		ConverterReturn(output, TwoDouble_t *, to_address) ;
	}

        if (sscanf(token, DOUBLE_CONVERSION_FORMAT, &dip) != 1) {
                XtFree(temp) ;
 
                XtStringConversionWarning(input, "ViewAngle") ;
		
		ConverterReturn(output, TwoDouble_t *, to_address) ;
        }
        
	output = (TwoDouble_t *) XtMalloc((unsigned) sizeof(TwoDouble_t)) ;

	output->d1 = dip ;
	output->d2 = azimuth ;
 
        XtFree((char *) temp) ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) output ;
        }

	ConverterReturn(output, TwoDouble_t *, to_address) ;

	/* NOTREACHED */

	return False ;
}
#endif /* INTEGRATE_VIEW3D */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDynamicPointArray():
**
** Mapper for String onto an array of XintPolylinePoint structures
**
** Required for the ChartObject widgets
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToDynamicPointArray(Display     *display,
                                       XrmValuePtr  argv,
                                       Cardinal    *argc,
                                       XrmValuePtr  from_address,
                                       XrmValuePtr  to_address,
                                       XtPointer   *closure,
                                       String       converter_type,
                                       double       last_value)
#else  /* _NO_PROTO */
Boolean XdCvtStringToDynamicPointArray(display, argv, argc, from_address, to_address, closure, converter_type, last_value)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
        String       converter_type ;
        double       last_value ;
#endif /* _NO_PROTO */
{ 
        static TwoFloat_t *output = (TwoFloat_t *) 0 ;
               String      input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String      temp   = (String) 0 ;
               String      token  = (String) 0 ;
               double      x      = (double) 0 ;
               double      y      = (double) 0 ;
               int         count  = 0 ;
               char        tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

	output           = (TwoFloat_t *) 0 ;
        to_address->addr = (XtPointer) 0 ;
        to_address->size = sizeof(XtPointer) ;
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (TwoFloat_t *) XtMalloc((unsigned) (count + 1) * sizeof(TwoFloat_t)) ;

        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(TwoFloat_t *) ;
 
        while (token != (String) 0) {
                x = (double) 0 ;
                y = (double) 0 ;
 
                if (sscanf(token, DOUBLE_CONVERSION_FORMAT, &x) != 1) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, converter_type) ;
 
                        return False ;
                }
 
                output[count].f1 = (float) x ;
 
                token = strtok((char *) 0, tokeniser) ;
 
                if ((token == (String) 0) || (sscanf(token, DOUBLE_CONVERSION_FORMAT, &y) != 1)) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, converter_type) ;
 
                        return False ;
                }
 
                output[count].f2 = (float) y ;
 
                output = (TwoFloat_t *) XtRealloc((char *) output, (++count) * sizeof(TwoFloat_t)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }
 
        output[count].f1 = (float) OUT_OF_BOUNDS_MARKER ;
        output[count].f2 = (float) OUT_OF_BOUNDS_MARKER ;
 
        XtFree((char *) temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}

        return True ;
}
#endif /* INTEGRATE_CHARTOBJECT */


/*
** _XdCvtStringToDynamicDouble4:
**
** Generic Mapper for String onto a set of four doubles
**
** Not really a converter, but a back end for one
*/
/* ARGSUSED */
#ifndef  _NO_PROTO
static Boolean _XdCvtStringToDynamicDouble4(Display    *display,
                                            XrmValuePtr argv,
					    Cardinal   *argc,
					    XrmValuePtr from_address,
					    XrmValuePtr to_address,
					    XtPointer  *closure,
					    char       *converter_type)
#else /* _NO_PROTO */
static Boolean _XdCvtStringToDynamicDouble4(display, argv, argc, from_address, to_address, closure, converter_type)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
	char       *converter_type ;
#endif/* _NO_PROTO */
{
        static FourDouble_p output = (FourDouble_p) 0 ;
               String       input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String       temp   = (String) 0 ;
               String       token  = (String) 0 ;
               int          count  = 1 ;
               double       value ;
               char         tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (FourDouble_t *) XtMalloc((unsigned) (count + 1) * sizeof(FourDouble_t)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(FourDouble_p) ;
 
        while (token != (String) 0) {
                if (!isdigit(token[0]) && (token[0] != '-')) {
                         to_address->addr = (XtPointer) 0 ;
                         to_address->size = 0 ;
 
                         XtFree(temp) ;
                         XtFree((char *) output) ;
 
                         XtStringConversionWarning(input, converter_type) ;
 
                         return False ;
                }
 
                /* Negative values map onto -1 throughout */
 
                if (token[0] == '-') {
                         value = -1 ;
                }
                else {
                         value = (double) atof(token) ;
                }
 
                switch (count)
                   {
                        case 1 : output->d1 = value ; break ;
                        case 2 : output->d2 = value ; break ;
                        case 3 : output->d3 = value ; break ;
                        case 4 : output->d4 = value ; break ;
                   }
 
                token = strtok((char *) 0, tokeniser) ;
                count++ ;
 
                if ((token != (String) 0) && (count == 4)) {
                         to_address->addr = (XtPointer) 0 ;
                         to_address->size = 0 ;
 
                         XtFree(temp) ;
                         XtFree((char *) output) ;
 
                         XtStringConversionWarning(input, converter_type) ;
 
                         return False ;
                }
        }
 
        XtFree((char *) temp) ;
	
	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDynamicCoord4:
**
** converts a string containing four doubles into a structure containing the same
** just calls the above...
**
** Required for ChartObject widgets
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicCoord4(Display    *display,
                                   XrmValuePtr argv,
                                   Cardinal   *argc,
                                   XrmValuePtr from_address,
                                   XrmValuePtr to_address,
                                   XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicCoord4(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        return _XdCvtStringToDynamicDouble4(display, argv, argc, from_address, to_address, closure, "DynamicCoord4") ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** _XdCvtStringToDynamicInteger4:
**
** Generic Mapper for String onto a set of four doubles
**
** Not really a converter, but a back end for one
*/
/* ARGSUSED */
#ifndef  _NO_PROTO
static Boolean _XdCvtStringToDynamicInteger4(Display    *display,
                                             XrmValuePtr argv,
					     Cardinal   *argc,
					     XrmValuePtr from_address,
					     XrmValuePtr to_address,
					     XtPointer  *closure,
					     char       *converter_type)
#else /* _NO_PROTO */
static Boolean _XdCvtStringToDynamicInteger4(display, argv, argc, from_address, to_address, closure, converter_type)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
	char       *converter_type ;
#endif/* _NO_PROTO */
{
        static FourInteger_p output = (FourInteger_p) 0 ;
               String        input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String        temp   = (String) 0 ;
               String        token  = (String) 0 ;
               int           count  = 1 ;
               int           value ;
               char          tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (FourInteger_t *) XtMalloc((unsigned) (count + 1) * sizeof(FourInteger_t)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(FourInteger_t *) ;
 
        while (token != (String) 0) {
                if (!isdigit(token[0]) && (token[0] != '-')) {
                         to_address->addr = (XtPointer) 0 ;
                         to_address->size = 0 ;
 
                         XtFree(temp) ;
                         XtFree((char *) output) ;
 
                         XtStringConversionWarning(input, converter_type) ;
 
                         return False ;
                }
 
                /* Negative values map onto -1 throughout */
                /* Negative values map onto -1 throughout */
 
                if (token[0] == '-') {
                         value = -1 ;
                }
                else {
                         value = atoi(token) ;
                }
 
                switch (count)
                   {
                        case 1 : output->i1 = value ; break ;
                        case 2 : output->i2 = value ; break ;
                        case 3 : output->i3 = value ; break ;
                        case 4 : output->i4 = value ; break ;
                   }
 
                token = strtok((char *) 0, tokeniser) ;
                count++ ;
 
                if ((token != (String) 0) && (count == 4)) {
                         to_address->addr = (XtPointer) 0 ;
                         to_address->size = 0 ;
 
                         XtFree(temp) ;
                         XtFree((char *) output) ;
 
                         XtStringConversionWarning(input, converter_type) ;
 
                         return False ;
                }
        }
 
        XtFree((char *) temp) ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
        return True ;
}

/*
** XdCvtStringToDynamicInteger4:
**
** Simple call onto the above
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicInteger4(Display    *display,
                                     XrmValuePtr argv,
                                     Cardinal   *argc,
                                     XrmValuePtr from_address,
                                     XrmValuePtr to_address,
                                     XtPointer  *closure,
                                     char       *converter_type)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicInteger4(display, argv, argc, from_address, to_address, closure, converter_type)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
        char       *converter_type ;
#endif/* _NO_PROTO */
{
	return _XdCvtStringToDynamicInteger4(display, argv, argc, from_address, to_address, closure, "DynamicInteger4") ;
}
#endif /* INTEGRATE_CHARTOBJECT */


/*
** _XdCvtStringToFloatArray():
**
** Generic Mapper for String onto an array of float of some kind
**
** Not really a converter, but a back end for one
*/
 
#define   FLOAT_CONVERSION_FORMAT     "%f.3"
/* ARGSUSED */
#ifndef   _NO_PROTO
static Boolean _XdCvtStringToFloatArray(Display     *display,
                                        XrmValuePtr  argv,
                                        Cardinal    *argc,
                                        XrmValuePtr  from_address,
                                        XrmValuePtr  to_address,
                                        XtPointer   *closure,
                                        String       converter_type,
                                        double       last_value)
#else  /* _NO_PROTO */
static Boolean _XdCvtStringToFloatArray(display, argv, argc, from_address, to_address, closure, converter_type, last_value)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
        String       converter_type ;
        double       last_value ;
#endif /* _NO_PROTO */
{
        static float  *output = (float *) 0 ;
               String  input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String  temp   = (String) 0 ;
               String  token  = (String) 0 ;
               float   value ;
               int     count  = 0 ;
               char    tokeniser[MAX_TOKENISER_STRING + 1] ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) 0 ;
        }
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (float *) XtMalloc((unsigned) (count + 1) * sizeof(float)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        while (token != (String) 0) {
                value = (float) 0 ;
 
                if (sscanf(token, FLOAT_CONVERSION_FORMAT, &value) != 1) {
                        to_address->addr = (XtPointer) 0 ;
                        to_address->size = 0 ;
 
                        XtFree(temp) ;
                        XtFree((char *) output) ;
 
                        XtStringConversionWarning(input, converter_type) ;
 
                        return False ;
                }
 
                output[count] = value ;
 
                output = (float *) XtRealloc((char *) output, (++count) * sizeof(float)) ;
                token  = strtok((char *) 0, tokeniser) ;
        }
 
        output[count] = last_value ;
 
        XtFree((char *) temp) ;
 
        if (closure != (XtPointer *) 0) {
                *closure = (XtPointer) output ;
        }
 
        return True ;
}

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToAuxPositionList():
**
** Mapper for String onto an array of float representing AuxPositionList
**
** Required for ChartObject
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToAuxPositionList(Display     *display,
                                     XrmValuePtr  argv,
                                     Cardinal    *argc,
                                     XrmValuePtr  from_address,
                                     XrmValuePtr  to_address,
                                     XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToAuxPositionList(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
	return _XdCvtStringToFloatArray(display, 
					argv, 
					argc, 
					from_address, 
					to_address, 
					closure, 
					"AuxPositionList", 
					MAXFLOAT) ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDynamicFloat():
**
** Mapper for String onto an array of float representing DynamicFloat
**
** Required for ChartObject
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean XdCvtStringToDynamicFloat(Display     *display,
                                  XrmValuePtr  argv,
                                  Cardinal    *argc,
                                  XrmValuePtr  from_address,
                                  XrmValuePtr  to_address,
                                  XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean XdCvtStringToDynamicFloat(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
        return _XdCvtStringToFloatArray(display, 
					argv,
					argc, 
					from_address, 
					to_address, 
					closure, 
					"DynamicFloat",
					MAXFLOAT) ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDynamicSymbolList:
**
** converts a string containing XintSYMBOL_X, XintSYMBOL_PLUS... into an array of integers
**
** Required for ChartObject widgets
*/

#ifndef   XintVERSION
#define   XintVERSION     3
#endif /* XintVERSION */
 
#if       (XintVERSION == 3)
#include <Xint/Symbols.h>
#endif /* (XintVERSION == 3) */
/* ARGSUSED */
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicSymbolList(Display    *display,
                                       XrmValuePtr argv,
                                       Cardinal   *argc,
                                       XrmValuePtr from_address,
                                       XrmValuePtr to_address,
                                       XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicSymbolList(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static NameValueMap_t symbol_map[] =
        {
#if       (XintVERSION == 3)
                { "X",               XintSYMBOL_X               },
                { "PLUS",            XintSYMBOL_PLUS            },
                { "SQUARE",          XintSYMBOL_SQUARE          },
                { "CIRCLE",          XintSYMBOL_CIRCLE          },
                { "TRIANGLE",        XintSYMBOL_TRIANGLE        },
                { "DIAMOND",         XintSYMBOL_DIAMOND         },
                { "FILLED_SQUARE",   XintSYMBOL_FILLED_SQUARE   },
                { "FILLED_CIRCLE",   XintSYMBOL_FILLED_CIRCLE   },
                { "FILLED_TRIANGLE", XintSYMBOL_FILLED_TRIANGLE },
                { "FILLED_DIAMOND",  XintSYMBOL_FILLED_DIAMOND  },
#endif /* (XintVERSION == 3) */
                { (char *) 0,        0                          }
        } ;
 
        return _XdCvtStringToEnumeratedArray(display,
                                             argv,
                                             argc,
                                             from_address,
                                             to_address,
                                             closure,
                                             "SymbolList",
                                             OUT_OF_BOUNDS_MARKER,
                                             symbol_map,
                                             "Xint",
                                             "SYMBOL_") ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_CHARTOBJECT 
/*
** XdCvtStringToDynamicConstraint:
**
** converts a string into an XintConstraint * structure
**
** The string is just XintANCHOR_SOUTH, ...
**
** Required for ChartObject widgets
*/
 
#include <Xint/Graphic.h>
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicConstraint(Display    *display,
                                       XrmValuePtr argv,
                                       Cardinal   *argc,
                                       XrmValuePtr from_address,
                                       XrmValuePtr to_address,
                                       XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicConstraint(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        static XintConstraint *output = (XintConstraint *) 0 ;
               String          input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String          temp   = (String) 0 ;
               String          token  = (String) 0 ;
               String          tptr   = (String) 0 ;
               char            tokeniser[MAX_TOKENISER_STRING + 1] ;
               NameValueMap_p  mptr ;
               int             len ;
               int             cmp ;
               Boolean         retcode = True ;
 
        static NameValueMap_t  constraint_map[] =
        {
                { "NORTH",       XintANCHOR_NORTH      },
                { "NORTH_EAST",  XintANCHOR_NORTH_EAST },
                { "NORTH_WEST",  XintANCHOR_NORTH_WEST },
                { "SOUTH",       XintANCHOR_SOUTH      },
                { "SOUTH_EAST",  XintANCHOR_SOUTH_EAST },
                { "SOUTH_WEST",  XintANCHOR_SOUTH_WEST },
                { "EAST",        XintANCHOR_EAST       },
                { "WEST",        XintANCHOR_WEST       },
                { "CENTER",      XintANCHOR_CENTER     },
                { "TOP",         XintANCHOR_TOP        },
                { "BOTTOM",      XintANCHOR_BOTTOM     },
                { "LEFT",        XintANCHOR_LEFT       },
                { "RIGHT",       XintANCHOR_RIGHT      },
                { (char *) 0,    0                     }
        } ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}
 
        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output                 = (XintConstraint *) XtMalloc((unsigned) sizeof(XintConstraint)) ;
        output->placement.type = XintCONSTRAIN_PLACEMENT ;
 
        to_address->addr = (XtPointer) &output ;
 
 
        if (token != (String) 0) {
               /* Skip any family prefix */
 
                tptr = token ;
 
                /* Skip any XmN style prefix */
 
                len = strlen("Xint") ;
 
                if (STRNCASECMP(tptr, "Xint", len) == 0) {
                       tptr += len ;
                }
 
                /* Skip any common resource prefix (Eg all the ANCHOR_ XintANCHOR_WEST...) */
 
                len = strlen("ANCHOR_") ;
 
                if (STRNCASECMP(tptr, "ANCHOR_", len) == 0) {
                       tptr += len ;
                }
 
                /* Now look up in the table of values provided */
 
                mptr = constraint_map ;
 
                for (;;) {
                        if (mptr->name == (char *) 0) {
                                /* Fallen off the end of the Map      */
                                /* Therefore the input does not parse */
 
                                to_address->addr = (XtPointer) 0 ;
                                to_address->size = 0 ;
                                retcode          = False ;
 
                                break ;
                        }
 
                        if ((cmp = STRCASECMP(mptr->name, tptr)) == 0) {
                                output->placement.anchor = mptr->value ;
 
                                break ;
                        }
 
                        if (cmp < 0) {
                                /* Sorted Names: Passed the point of no return */
 
                                to_address->addr = (XtPointer) 0 ;
                                to_address->size = 0 ;
 
                                retcode          = False ;
 
                                break ;
                        }
 
                        mptr++ ;
                }
 
 
        }
 
        XtFree((char *) temp) ;
 
        if (retcode == False) {
                XtStringConversionWarning(input, "DynamicConstraint") ;
        }
	
	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}
 
 
        return True ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToColorRecord:
**
** converts a string of color names into an XintColorRec * structure
**
** Required for ChartObject widgets
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToColorRecord(Display    *display,
                                 XrmValuePtr argv,
                                 Cardinal   *argc,
                                 XrmValuePtr from_address,
                                 XrmValuePtr to_address,
                                 XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToColorRecord(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        extern XintColorRec *XintChartCreateColorRecord() ;
 
        static XintColorRec *output  = (XintColorRec *) 0 ;
               Pixel        *pixels  = (Pixel *) 0 ;
               Boolean       retcode = True ;
               int           count   = 0 ;
               XrmValue      to_pixels ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}
 
        to_pixels.addr = (XtPointer) 0 ;
        to_pixels.size = 0 ;
 
        to_address->addr = (XtPointer) 0 ;
        to_address->size = 0 ;
 
        /* We need an array of pixels - and we already have a converter for this */
 
        retcode = XdCvtStringToPixelList(display, argv, argc, from_address, &to_pixels, closure) ;
 
        if (retcode == True) {
                if (to_pixels.size > 0) {
                        if ((pixels = *((Pixel **) to_pixels.addr)) == (Pixel *) 0) {
                                retcode = False ;
                        }
                        else {
                                while (pixels[count] != (Pixel) 0) {
                                        count++ ;
                                }
 
                                output = XintChartCreateColorRecord(pixels, count) ;
 
                                to_address->addr = (XtPointer) output ;
                                to_address->size = (output ? sizeof(XtPointer) : 0) ;
                        }
                }
        }
	
	if (retcode == True) {
		if (closure != (XtPointer *) 0) {
			*closure = (XtPointer) output ;
		}
	}
 
 
        return retcode ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDynamicArrayStrings:
**
** converts a string into an array of strings
**
** Required for ChartObject widgets
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicArrayStrings(Display    *display,
                                         XrmValuePtr argv,
                                         Cardinal   *argc,
                                         XrmValuePtr from_address,
                                         XrmValuePtr to_address,
                                         XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicArrayStrings(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        return _XdCvtStringToStringArray(display, argv, argc, from_address, to_address, closure, "DynamicArrayStrings", 0) ;
}
#endif /* INTEGRATE_CHARTOBJECT */


/*
** _XdCvtStringToDynamicDouble:
**
** Generic Mapper for String onto a double of some kind
**
** Not really a converter, but a back end for one
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
static Boolean _XdCvtStringToDynamicDouble(Display    *display,
                                           XrmValuePtr argv,
					   Cardinal   *argc,
					   XrmValuePtr from_address,
					   XrmValuePtr to_address,
					   XtPointer  *closure,
					   char       *converter_type)
#else /* _NO_PROTO */
static Boolean _XdCvtStringToDynamicDouble(display, argv, argc, from_address, to_address, closure, converter_type)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
	char       *converter_type ;
#endif/* _NO_PROTO */
{
        static double *output = (double *) 0 ;
               String  input  = (from_address ? (String) from_address->addr : (String) 0) ;
               String  temp   = (String) 0 ;
               String  token  = (String) 0 ;
               double  value ;
               char    tokeniser[MAX_TOKENISER_STRING + 1] ;

	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) 0 ;
	}

        if (input != (String) 0) {
                temp = XtNewString(input) ;
 
                (void) sprintf(tokeniser, "\t ,") ;
 
                token = strtok(temp, tokeniser) ;
        }
 
        if (token == (String) 0) {
                to_address->addr = (XtPointer) 0 ;
                to_address->size = sizeof(XtPointer) ;
 
                XtFree((char *) temp) ;
 
                return True ;
        }
 
        output = (double *) XtMalloc((unsigned) sizeof(double)) ;
 
        to_address->addr = (XtPointer) &output ;
        to_address->size = sizeof(XtPointer) ;
 
        if (token != (String) 0) {
                value = atof(token) ;
               *output = value ;
        }
 
        XtFree((char *) temp) ;
	
	if (closure != (XtPointer *) 0) {
		*closure = (XtPointer) output ;
	}

        return True ;
}

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToDynamicCoord:
**
** takes a string and converts into a pointer to a double
**
** required for ChartObject widgets
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicCoord(Display    *display,
                                  XrmValuePtr argv,
                                  Cardinal   *argc,
                                  XrmValuePtr from_address,
                                  XrmValuePtr to_address,
                                  XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicCoord(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        /* Arithmetic performed in double precision */
 
        return _XdCvtStringToDynamicDouble(display, argv, argc, from_address, to_address, closure, "DynamicCoord") ;
}
#endif /* INTEGRATE_CHARTOBJECT */

#if       defined(INTEGRATE_VIEW3D) || defined(INTEGRATE_CHARTOBJECT)
/*
** XdCvtStringToDynamicLabelPositionArray:
**
** UNIMPLEMENTED
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToDynamicLabelPositionArray(Display    *display,
                                  XrmValuePtr argv,
                                  Cardinal   *argc,
                                  XrmValuePtr from_address,
                                  XrmValuePtr to_address,
                                  XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToDynamicLabelPositionArray(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        String  input  = (from_address ? (String) from_address->addr : (String) 0) ;

	(void) fprintf(stderr,"UNIMPLEMENTED: XdCvtStringToDynamicLabelPositionArray()\n") ;
	(void) fflush(stderr) ;

	to_address->addr = (XtPointer) 0 ;
	to_address->size = 0 ;

	XtStringConversionWarning(input, "DynamicLabelPositionArray") ;

	return False ;
}
#endif /* defined(INTEGRATE_VIEW3D) || defined(INTEGRATE_CHARTOBJECT) */

#if       defined(INTEGRATE_GRAPHPAK) || defined(INTEGRATE_CHARTOBJECT)
/*
** XdCvtStringToXiPSFontList:
**
** UNIMPLEMENTED
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToXiPSFontList(Display    *display,
                                  XrmValuePtr argv,
                                  Cardinal   *argc,
                                  XrmValuePtr from_address,
                                  XrmValuePtr to_address,
                                  XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToXiPSFontList(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        String  input  = (from_address ? (String) from_address->addr : (String) 0) ;

	(void) fprintf(stderr,"UNIMPLEMENTED: XdCvtStringToXiPSFontList()\n") ;
	(void) fflush(stderr) ;

	to_address->addr = (XtPointer) 0 ;
	to_address->size = 0 ;

	XtStringConversionWarning(input, "XiPSFontList") ;

	return False ;
}
#endif /* defined(INTEGRATE_GRAPHPAK) || defined(INTEGRATE_CHARTOBJECT) */

#ifdef    INTEGRATE_CHARTOBJECT
/*
** XdCvtStringToAuxAnnotationList:
**
** UNIMPLEMENTED
*/
/* ARGSUSED */ 
#ifndef  _NO_PROTO
Boolean XdCvtStringToAuxAnnotationList(Display    *display,
                                       XrmValuePtr argv,
				       Cardinal   *argc,
				       XrmValuePtr from_address,
				       XrmValuePtr to_address,
				       XtPointer  *closure)
#else /* _NO_PROTO */
Boolean XdCvtStringToAuxAnnotationList(display, argv, argc, from_address, to_address, closure)
        Display    *display ;
        XrmValuePtr argv ;
        Cardinal   *argc ;
        XrmValuePtr from_address ;
        XrmValuePtr to_address ;
        XtPointer  *closure ;
#endif/* _NO_PROTO */
{
        String  input  = (from_address ? (String) from_address->addr : (String) 0) ;

	(void) fprintf(stderr,"UNIMPLEMENTED: XdCvtStringToAuxAnnotationList()\n") ;
	(void) fflush(stderr) ;

	to_address->addr = (XtPointer) 0 ;
	to_address->size = 0 ;

	XtStringConversionWarning(input, "AuxAnnotationList") ;

	return False ;
}
#endif /* INTEGRATE_CHARTOBJECT */



/*
** _XdCvtStringToDynamicLabelPositionArray():
**
** takes a string and converts into an array of double
**
** required for View3D
*/
/* ARGSUSED */ 
#ifndef   _NO_PROTO
Boolean _XdCvtStringToDynamicLabelPositionArray(Display     *display,
                                                XrmValuePtr  argv,
						Cardinal    *argc,
						XrmValuePtr  from_address,
						XrmValuePtr  to_address,
						XtPointer   *closure)
#else  /* _NO_PROTO */
Boolean _XdCvtStringToDynamicLabelPositionArray(display, argv, argc, from_address, to_address, closure)
        Display     *display ;
        XrmValuePtr  argv ;
        Cardinal    *argc ;
        XrmValuePtr  from_address ;
        XrmValuePtr  to_address ;
        XtPointer   *closure ;
#endif /* _NO_PROTO */
{
	return _XdCvtStringToDoubleArray(display, 
					 argv, 
					 argc, 
					 from_address, 
					 to_address, 
					 closure, 
					 "DynamicLabelPositionArray", 
					 (double) OUT_OF_BOUNDS_MARKER) ;
}
