/* X-Designer
** (c) 1992, 1993, 1994, 1995
** Imperial Software Technology (IST),
** 95 London Street,
** Reading,
** Berkshire,
** United Kingdom RG1 4QA.
**
** Telephone: +44 1734 587055
** Fax:       +44 1734 589005
** Email:     support@ist.co.uk
*/

/*
** sccsid = "@(#)dist/src/xd/hdrs/xwidget.h	1.25" -- 03/09/05
*/

#ifndef   _XD_XWIDGET_H
#define   _XD_XWIDGET_H

#include <X11/IntrinsicP.h>
#include <X11/Intrinsic.h>

#ifndef   FILE
#include <stdio.h>
#endif /* FILE */

#include "icon.h"

/*
** Forward Declarations
*/

typedef struct Attr_s          *Attr_p ;
typedef struct Dialog_s        *Dialog_p ;
typedef struct ResWidget_s     *ResWidget_p ;
typedef struct Popup_s         *Popup_p ;
typedef struct Resource_s      *Resource_p ;
typedef struct XWidgetClass_s  *XWidgetClass_p ;
typedef struct XWidget_s       *XWidget_p ;
typedef struct File_s          *File_p ;
typedef struct Thing           *Thing_p ;
typedef struct XFamilyHeader_s *XFamilyHeader_p ;
typedef struct XFamily_s       *XFamily_p ;
typedef struct Enum_s          *Enum_p ;
typedef struct Enum_def_s      *Enum_def_p ;
typedef struct Type_alias_s    *Type_alias_p ;
typedef struct Converter_s     *Converter_p ;
typedef struct ResClass_s      *ResClass_p ;
typedef struct ResGen_s        *ResGen_p ;
typedef struct ConvCB_s        *ConvCB_p ;

/*
** An attribute: This is used for storing "local" resources
*/

typedef struct Attr_s 
{
	int    id ;
	int    value ;
	Attr_p next ;
} Attr_t ;

/*
** One of these structures exists for every resource which we are handling
** in a resource dialog. It contains pointers to the widgets which display
** the resource value as well as the toggle indicating that the resource
** is to be generated to the resource file
*/

typedef enum { xdc_free_default, xdc_free, xdc_dont_free  } xdc_free_type ;

#define    XDC_ACCESS_UNSPECIFIED    -1
#define    XDC_ACCESS_NONE            0
#define    XDC_ACCESS_CREATE          1
#define    XDC_ACCESS_GET             2
#define    XDC_ACCESS_SET             4
#define    XDC_ACCESS_ALL            (XDC_ACCESS_CREATE | XDC_ACCESS_GET | XDC_ACCESS_SET)

typedef struct ResWidget_s 
{
	Widget        widget ;
	Widget        resfile_toggle ;
	Widget        change_label ;
	Widget        java_annotation_label ;
	Widget        mfc_annotation_label ;
	XtPointer     value ;
	unsigned int  changed : 1 ;
	xdc_free_type free_on_get ;
	xdc_free_type free_on_set ;
	int           access ;
	Boolean       java_compatible ;
	Boolean       mfc_compatible ;
	char         *counter_name ;
	Dialog_p      dialog ;
} ResWidget_t ;

#define DISPLAY_PAGE    0
#define MARGINS_PAGE    1
#define SETTINGS_PAGE   2
#define CALLBACKS_PAGE  3
#define MISC_PAGE       4
#define NPAGES          5

typedef struct Dialog_s 
{
	Widget      shell ;
	Widget      form ;
	Widget      apply_button ;
	Widget      undo_button ;
	Widget      inherit_button ;
	Widget     *pages ;
	Widget     *buttons ;
	Widget     *last_on_page ;
	int        *page_index ;
	int         page_count ;
	ResWidget_p widgets ;
	Boolean     editable ;
	Boolean     modified ;
} Dialog_t ;

/*
** A widget class:
*/
typedef struct Popup_s 
{
#ifndef   _NO_PROTO
	void   (*dialog_create)(Widget) ;
	void   (*dialog_init)(Widget, Widget, char *) ;
	void   (*dialog_update)(Widget) ;
#else  /* _NO_PROTO */
	void   (*dialog_create)() ;
	void   (*dialog_init)() ;
	void   (*dialog_update)() ;
#endif /* _NO_PROTO */
	Boolean  created ;
} Popup_t ;

typedef struct Resource_s 
{
	char         *name ;
	XrmQuark      q_name ;
	Popup_p       popup ;
	int           page_index ;
	Boolean       visible ;
	xdc_free_type free_on_get ;
	xdc_free_type free_on_set ;
	int           motif_access ;
	Boolean       mfc_compatible ;
	Boolean       java_compatible ;
	char         *counter_name ;
} Resource_t ;

typedef struct XWidgetClass_s 
{
	WidgetClass *widget_class ;           /* The widget class                                           */
	char        *class_name ;             /* The name of the variable containing the Widget Class       */
	char        *include_filename ;       /* name of include file for generated code                    */
	int          page_num ;               /* Which page should it go on                                 */

#ifndef   _NO_PROTO
	char       *(*defined_name)(char *) ;
	int         (*create)(XWidget_p) ;
	int         (*destroy)(XWidget_p) ;
	Widget      (*realise)(char *, WidgetClass, Widget, ArgList, Cardinal) ;
	Boolean     (*abstract_child)(XWidget_p, Widget, Boolean *, Boolean *) ;
	int         (*resource_panel)(XWidget_p) ;
	int         (*constraint_panel)(XWidget_p, Widget) ;
	int         (*resource_update)() ;
	int         (*constraint_update)(XWidgetClass_p, XWidget_p) ;
	Boolean     (*can_add_child)(XWidget_p, WidgetClass) ;
	Boolean     (*appropriate_parent)(Widget, WidgetClass) ;
	int         (*generate)(XWidget_p, int, int, File_p, int *) ;
	int         (*write)(XWidget_p, File_p) ;
	int         (*update)(XWidgetClass_p, XWidget_p) ;
#else  /* _NO_PROTO */
	char       *(*defined_name)() ;       /* Routine to turn a resource name back to a XtN (XmN..) name */
	int         (*create)() ;             /* Routine when creating widget                               */
	int         (*destroy)() ;            /* Routine when destroying a widget                           */
	Widget      (*realise)() ;            /* Routine to realise a widget                                */
	Boolean     (*abstract_child)() ;     /* Routine to control display of abstract children            */
	int         (*resource_panel)() ;     /* Routine to pop up the resource dialogue                    */
	int         (*constraint_panel)() ;   /* Routine to pop up the constraints dialogue                 */
	int         (*resource_update)() ;    /* Routine when resource dialogue needs updating              */
	int         (*constraint_update)() ;  /* Routine when constraints dialogue needs updating           */
	Boolean     (*can_add_child)() ;      /* Routine to find out if a child can be added to this widget */
	Boolean     (*appropriate_parent)() ; /* Routine to find out this widget can be added to the parent */
	int         (*generate)() ;           /* Routine to generate code                                   */
	int         (*write)() ;              /* Routine to write save file                                 */
	int         (*update)() ;             /* Routine to update dialogues                                */
#endif /* _NO_PROTO */

	Dialog_p     resource_dialog ;        /* The resource dialogue                                      */
	Dialog_p     constraint_dialog ;      /* The constraint dialogue                                    */
	Boolean      goto_unsupported ;       /* Goto translation is buggy for the widget                   */
	Boolean      can_swap_fg ;            /* The class has a foreground resource                        */
	Boolean      can_create ;             /* The widget can be created from the palette                 */
	Boolean      can_manage ;             /* The widget can be managed (Some Data Objects Cannot)       */
	Boolean      can_edit_abstract ;      /* The widget can be edited as an abstract child              */
	Boolean      generate_initialise ;    /* Generate XtInitializeClassWidget in main program           */
	Boolean      initialised ;            /* The can   swap fg flag is valid                            */
	Boolean      layoutable ;             /* The widget may use the layout editor                       */
	XtPointer    extension ;              /* For the programmers own use                                */
	Icon_t       icon ;                   /* Default palette bitmap                                     */
	Widget       button ;                 /* The widget in the icon palette                             */
	Resource_p   popups ;                 /* Resource specific popup editors                            */
	int          user_page_count ;        /* How many user defined pages                                */
	char        *widget_help_doc ;        /* Help doc for widget help                                   */
	char        *widget_help_tag ;        /* Help tag for widget help                                   */
	char        *dialogue_help_doc ;      /* Help doc for dialogue                                      */
	char        *dialogue_help_tag ;      /* Help tag for dialogue                                      */
	Boolean      used ;                   /* Is the class used in the design                            */
	int          page_offset ;            /* Start Offset into Merged Page Name Table                   */
} XWidgetClass_t ;

extern XWidgetClass_p xwidgets[] ;

/*
** A widget instance
*/

typedef struct XWidget_s 
{
	XWidgetClass_p xw_class ;     /* Pointer to the XWidget class record    */
	Widget         widget ;       /* The actual widget                      */
	Thing_p        thing ;        /* Canonical X-Designer form of an Object */
	XtPointer      extension ;    /* For the programmers own use            */
} XWidget_t ;

/*
** Family Header required for code generation of any member of the Family
*/

typedef struct XFamilyHeader_s
{
	char *header ;
} XFamilyHeader_t ;

/*
** A family of widgets:
*/
typedef struct XFamily_s 
{
	char           *name ;                    /* The name of this family                 */
	XWidgetClass_p *widget_classes ;          /* The widgets in this family              */
	XFamilyHeader_p family_headers ;          /* All widgets in the family include these */

	char           *family_init_name ;        /* Family Initialisation Routine Name      */
	void          (*family_init)() ;          /* Family Initialisation Routine           */

	Boolean         family_init_xd ;          /* Call above inside X-Designer            */
	Boolean         family_init_code ;        /* Call above inside generated code        */
	Boolean         family_used ;             /* If any widget in the family is used     */
	Boolean         family_init_pre_shell ;   /* If any widget in the family is used     */
} XFamily_t ;

typedef struct Enum_s 
{
	char *name ;
	int   value ;
	char *code_symbol ;
	char *res_file_symbol ;
} Enum_t ;

typedef struct Enum_def_s 
{
	char    *name ;
	XrmQuark type ;
	Enum_p   enumeration ;
} Enum_def_t ;

typedef struct Type_alias_s 
{
	char     *from_name ;
	XrmQuark  from_type ;
	char     *to_name ;
	XrmQuark  to_type ;
} Type_alias_t ;

typedef struct Converter_s 
{
	char     *name ;
	XrmQuark  q_name ;
	char     *defined_name ;

#ifndef   _NO_PROTO
	Boolean (*converter_proc)(Display *, XrmValue *, Cardinal *, XrmValue *, XrmValue, XtPointer *) ;
#else  /* _NO_PROTO */
	Boolean (*converter_proc)() ;
#endif /* _NO_PROTO */

	char     *converter_name ;
	Boolean   converter_required ;
	Popup_p   popup ;
} Converter_t ;

typedef struct ResClass_s 
{
	char        *name ;
	XrmQuark     type ;
	int          page ;
	unsigned int parse_type ;

#ifndef   _NO_PROTO
	Widget     (*create_widgets)(XWidgetClass_p, Widget, Widget, XtResourceList, ResWidget_p, Enum_p) ;
	int        (*set_value)(ResWidget_p, Widget, Thing_p, XtResource *, int) ;
	XtPointer  (*get_value)(ResWidget_p, Widget, Thing_p, XtResource *, int) ;
	int        (*write)(File_p, XWidgetClass_p, Widget, Thing_p, XtResource *, int) ;
	int        (*generate)(File_p, XWidgetClass_p, Widget, Thing_p, XtResource *, int, int, int, int *) ;
#else  /* _NO_PROTO */
	Widget     (*create_widgets)() ;
	int        (*set_value)() ;
	XtPointer  (*get_value)() ;
	int        (*write)() ;
	int        (*generate)() ;
#endif /* _NO_PROTO */
} ResClass_t ;

typedef struct ResGen_s 
{
	FILE     *file ;
	int       language ;
	int       phase ;
	char     *name ;
	XtPointer value ;
	XWidget_p xwidget ;
} ResGen_t ;

#define BOOL_DEFAULT    0
#define BOOL_YES        1
#define BOOL_NO         2

typedef struct ConvCB_s 
{
	Widget  text ;
	char   *resource_name ;
	Popup_p popup ;
} ConvCB_t ;

#endif /* _XD_XWIDGET_H */
